@@ -24,11 +24,18 @@ open Options
 open CommonTypes
 open CommonOptions
 open CommonGlobals
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         TYPES                                         *)
+(*                                                                       *)
+(*************************************************************************)
   
 type 'a file_impl = {
     mutable impl_file_update : int;
     mutable impl_file_state : file_state;
 
+    mutable impl_file_comment : string;
     mutable impl_file_num : int;
     mutable impl_file_val : 'a;
     mutable impl_file_ops : 'a file_ops;
@@ -64,12 +71,19 @@ it will happen soon. *)
     mutable op_file_set_format : ('a -> CommonTypes.format -> unit);
     mutable op_file_check : ('a -> unit);
     mutable op_file_recover : ('a -> unit);
-    mutable op_file_sources : ('a -> client list);
+    mutable op_file_all_sources : ('a -> client list);
+    mutable op_file_active_sources : ('a -> client list);
     mutable op_file_comment : ('a -> string);
     mutable op_file_set_priority : ('a -> int -> unit);
     mutable op_file_print_sources_html : ('a -> Buffer.t -> unit);
     mutable op_file_debug : ('a -> string);
   }
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         as_file...                                    *)
+(*                                                                       *)
+(*************************************************************************)
   
 let as_file  (file : 'a file_impl) =
   let (file : file) = Obj.magic file in
@@ -91,7 +105,7 @@ let dummy_file_impl = {
     impl_file_ops = Obj.magic 0;
     impl_file_size = Int64.zero;
     impl_file_age = 0;
-    impl_file_fd = Unix32.create "" [Unix.O_RDONLY] 0o666;
+    impl_file_fd = Unix32.create_diskfile "" [Unix.O_RDONLY] 0o666;
     impl_file_downloaded = Int64.zero;
     impl_file_received = Int64.zero;
     impl_file_last_received = [];
@@ -99,10 +113,17 @@ let dummy_file_impl = {
     impl_file_best_name = "<UNKNOWN>";
     impl_file_priority = 0;
     impl_file_last_seen = 0;
+    impl_file_comment = "";
   }
   
 let dummy_file = as_file dummy_file_impl  
 
+(*************************************************************************)
+(*                                                                       *)
+(*                         Global values                                 *)
+(*                                                                       *)
+(*************************************************************************)
+
 module H = Weak2.Make(struct
       type t = file
       let hash file = Hashtbl.hash (file_num file)
@@ -113,13 +134,12 @@ module H = Weak2.Make(struct
 let file_counter = ref 0
 let files_by_num = H.create 1027
   
-  
-let _ = 
-  Heap.add_memstat "CommonFile" (fun buf ->
-      let counter = ref 0 in
-      H.iter (fun _ -> incr counter) files_by_num;
-      Printf.bprintf buf "  files: %d\n" !counter;
-  )
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         Stub functions                                *)
+(*                                                                       *)
+(*************************************************************************)
 
 let ni n m = 
   let s = Printf.sprintf "File.%s not implemented by %s" 
@@ -202,6 +222,14 @@ let file_resume (file : file) =
 let set_file_state file state = 
   let impl = as_file_impl file in
   update_file_state impl state
+
+let set_file_comment file comment = 
+  let impl = as_file_impl file in
+  impl.impl_file_comment <- comment
+
+let file_comment file = 
+  let impl = as_file_impl file in
+  impl.impl_file_comment
       
 let file_best_name (file : file) =
   let file = as_file_impl file in
@@ -212,7 +240,7 @@ let set_file_best_name file name =
   let name = String2.replace name '/' "::" in
   file.impl_file_best_name <- name
 
-let file_set_format (file : file) format =
+let set_file_format (file : file) format =
   let file = as_file_impl file in
   file.impl_file_ops.op_file_set_format file.impl_file_val format
 
@@ -230,9 +258,13 @@ let file_debug (file : file) =
   let file = as_file_impl file in
   file.impl_file_ops.op_file_debug file.impl_file_val
 
-let file_sources file =
+let file_all_sources file =
   let impl = as_file_impl file in
-  try impl.impl_file_ops.op_file_sources impl.impl_file_val with _ -> []
+  try impl.impl_file_ops.op_file_all_sources impl.impl_file_val with _ -> []
+
+let file_active_sources file =
+  let impl = as_file_impl file in
+  try impl.impl_file_ops.op_file_active_sources impl.impl_file_val with _ -> []
 
 let file_print_sources_html (file : file) buf =
   let file = as_file_impl file in
@@ -256,7 +288,8 @@ let new_file_ops network =
       op_file_check = (fun _ -> ni_ok network "file_check");
       op_file_recover = (fun _ -> ni_ok network "file_recover");
       op_file_set_format = (fun _ -> fni network "file_set_format");
-      op_file_sources = (fun _ -> fni network "file_sources");
+      op_file_all_sources = (fun _ -> fni network "file_all_sources");
+      op_file_active_sources = (fun _ -> fni network "file_active_sources");
       op_file_comment = (fun _ -> ni_ok network "file_comment"; "");
       op_file_set_priority = (fun _ _ -> ni_ok network "file_set_priority");
       op_file_print_sources_html = (fun _ _ -> ni_ok network "file_print_sources_html");
@@ -295,8 +328,10 @@ let check_file_implementations () =
         lprintf "op_file_recover\n";
       if c.op_file_set_format == cc.op_file_set_format then
         lprintf "op_file_set_format\n";
-      if c.op_file_sources == cc.op_file_sources then
-        lprintf "op_file_sources\n";
+      if c.op_file_all_sources == cc.op_file_all_sources then
+        lprintf "op_file_all_sources\n";
+      if c.op_file_active_sources == cc.op_file_active_sources then
+        lprintf "op_file_active_sources\n";
       if c.op_file_print_sources_html == cc.op_file_print_sources_html then
         lprintf "op_file_print_sources_html\n";
   ) !files_ops;
@@ -383,7 +418,7 @@ let file_network file =
 let file_priority file = 
   (as_file_impl file).impl_file_priority
   
-let file_set_priority file p = 
+let set_file_priority file p = 
   let impl = as_file_impl file in
   if impl.impl_file_priority <> p then begin
       impl.impl_file_priority <- p;      
@@ -391,35 +426,51 @@ let file_set_priority file p =
       file_must_update file
     end
 
+let set_file_last_seen file age =
+  let impl = as_file_impl file in
+  impl.impl_file_last_seen <- age
+    
 let file_preview (file : file) =
   let cmd = Printf.sprintf "%s \"%s\" \"%s\"" !!previewer
 	      (file_disk_name file) (file_best_name file) in
   ignore (Sys.command cmd)
 
-let com_files_by_num = files_by_num
-let files_by_num = ()
+(*************************************************************************)
+(*                                                                       *)
+(*                         file_downloaders                              *)
+(*                                                                       *)
+(*************************************************************************)
 
 module G = GuiTypes
 
 let file_downloaders file o cnt = 
   let buf = o.conn_buf in
 
-      let srcs = file_sources file in
+      let srcs = file_active_sources file in
 		
       let counter = ref cnt in
       List.iter (fun c ->
-		if client_state c = Connected_downloading then begin
-          if use_html_mods o then begin 
-              if (client_dprint_html c o file (if !counter mod 2 == 0 then "dl-1" else "dl-2";))
-					then incr counter;
+	match (client_state c) with
+            Connected_downloading _ -> 
+              (begin
+                if use_html_mods o then begin 
+                  if (client_dprint_html c o file (if !counter mod 2 == 0 then "dl-1" else "dl-2";))
+		    then incr counter;
 		  end
 		  else begin
 			client_dprint c o file;
 		  end;
-		end;
+	       end)
+          | _ -> ()
       ) srcs;
 
 	if !counter mod 2 = 0 then true else false
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         file_print                                    *)
+(*                                                                       *)
+(*************************************************************************)
   
 (* Use span for Opera DOM compatibility *)
 let colored_chunks chunks =
@@ -451,80 +502,81 @@ let file_print file o =
   let info = file_info file in
   let n = impl.impl_file_ops.op_file_network in
   let buf = o.conn_buf in
-  let srcs = file_sources file in
+  let srcs = file_all_sources file in
 
 
-	(* TODO: make multinet aware *)
-
-	if use_html_mods o then begin
-
-        html_mods_table_header buf "sourcesInfo" "sourcesInfo" [ 
-          ( "0", "srh br", "File Info", "Info" ) ; 
-          ( "0", "srh", "Value", "Value" ) ]; 
-
-			Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>";
-			 html_mods_td buf [
-    		("File number/Network", "sr br", "[#] Network");
-    		("", "sr", Printf.sprintf "[%d] %s" (file_num file) n.network_name) ];
-
-			Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
-			 html_mods_td buf [
-    		("Downloaded/Total size", "sr br", "DLed/Size");
-    		("", "sr", Printf.sprintf "%s bytes of %s bytes" 
-			(Int64.to_string info.G.file_downloaded) (Int64.to_string info.G.file_size) ) ];
-
-			Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
-			 html_mods_td buf [
-    		("File priority", "sr br", "Priority");
-    		("", "sr", Printf.sprintf "%d" (file_priority file)) ];
-			
-			Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
-			 html_mods_td buf [
-    		("Number of file sources", "sr br", "Sources");
-    		("", "sr", Printf.sprintf "%d" (List.length srcs)) ];
-
-			Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
-			 html_mods_td buf [
-    		("0=Missing, 1=Partial, 2=Complete, 3=Verified", "sr br", "Chunks");
-    		("0=Missing, 1=Partial, 2=Complete, 3=Verified", "sr", info.G.file_chunks) ];
-
+(* TODO: make multinet aware *)
+  
+  if use_html_mods o then begin
+      
+      html_mods_table_header buf "sourcesInfo" "sourcesInfo" [ 
+        ( "0", "srh br", "File Info", "Info" ) ; 
+        ( "0", "srh", "Value", "Value" ) ]; 
+      
+      Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>";
+      html_mods_td buf [
+        ("File number/Network", "sr br", "[#] Network");
+        ("", "sr", Printf.sprintf "[%d] %s" (file_num file) n.network_name) ];
+      
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
+      html_mods_td buf [
+        ("Downloaded/Total size", "sr br", "DLed/Size");
+        ("", "sr", Printf.sprintf "%s bytes of %s bytes" 
+            (Int64.to_string info.G.file_downloaded) (Int64.to_string info.G.file_size) ) ];
+      
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
+      html_mods_td buf [
+        ("File priority", "sr br", "Priority");
+        ("", "sr", Printf.sprintf "%d" (file_priority file)) ];
+      
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
+      html_mods_td buf [
+        ("Number of file sources", "sr br", "Sources");
+        ("", "sr", Printf.sprintf "%d" (List.length srcs)) ];
+      
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
+      html_mods_td buf [
+        ("0=Missing, 1=Partial, 2=Complete, 3=Verified", "sr br", "Chunks");
+        ("0=Missing, 1=Partial, 2=Complete, 3=Verified", "sr", info.G.file_chunks) ];
+      
       (match n.network_name with 
           "BitTorrent" -> (
-			Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
-			 html_mods_td buf [
-    		("Filename", "sr br", "Filename");
-    		("", "sr", (file_best_name file)) ]; )
-
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
+              html_mods_td buf [
+                ("Filename", "sr br", "Filename");
+                ("", "sr", (file_best_name file)) ]; )
+        
         | _ -> (
-
-			Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
-			 html_mods_td buf [
-    		("Fake check links", "sr br", "Fakecheck");
-    		("", "sr", Printf.sprintf "\\<a href=\\\"http://edonkeyfakes.ath.cx/fakecheck/update/fakecheck.php\\?md4=%s\\\"\\>[Donkey-Fakes]\\</a\\>
+              
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
+              html_mods_td buf [
+                ("Fake check links", "sr br", "Fakecheck");
+                ("", "sr", Printf.sprintf "\\<a href=\\\"http://donkeyfakes.gambri.net/fakecheck/update/fakecheck.php\\?md4=%s\\\"\\>[Donkey-Fakes]\\</a\\>
     				\\<a href=\\\"http://www.sharereactor.com/fakesearch.php\\?search=ed2k://|file|%s|%s|%s|/\\\"\\>[ShareReactor Fakecheck]\\</a\\>" 
-					(Md4.to_string info.G.file_md4) 
-					(info.G.file_name) (Int64.to_string info.G.file_size) (Md4.to_string info.G.file_md4)) ];
-
-
-			Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
-
-			 html_mods_td buf [
-    		("ed2k link", "sr br", "ed2k link");
-    		("", "sr", Printf.sprintf "\\<a href=\\\"ed2k://|file|%s|%s|%s|/\\\"\\>ed2k://|file|%s|%s|%s|/\\</A\\>" 
-						(info.G.file_name) (Int64.to_string info.G.file_size) (Md4.to_string info.G.file_md4)
-						(info.G.file_name) (Int64.to_string info.G.file_size) (Md4.to_string info.G.file_md4)) ];
-
-			Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
-			let optionlist = ref "" in
-			List.iter (fun name -> 
+                    (Md4.to_string info.G.file_md4) 
+                  (info.G.file_name) (Int64.to_string info.G.file_size) (Md4.to_string info.G.file_md4)) ];
+              
+              
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
+              
+              html_mods_td buf [
+                ("ed2k link", "sr br", "ed2k link");
+                ("", "sr", Printf.sprintf "\\<a href=\\\"ed2k://|file|%s|%s|%s|/\\\"\\>ed2k://|file|%s|%s|%s|/\\</A\\>" 
+                    (info.G.file_name) (Int64.to_string info.G.file_size) (Md4.to_string info.G.file_md4)
+                  (info.G.file_name) (Int64.to_string info.G.file_size) (Md4.to_string info.G.file_md4)) ];
+              
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
+              let optionlist = ref "" in
+              List.iter (fun (name,_) -> 
                   optionlist := !optionlist ^ Printf.sprintf "\\<option value=\\\"%s\\\"\\>%s\n" name name;
-            ) info.G.file_names;
- 
-
-			let input_size = (Printf.sprintf "%d" ((String.length (file_best_name file))+3)) in
-			 html_mods_td buf [
-    		("Rename file by selecting an alternate name", "sr br", "Filename");
-    		("Rename file", "sr", Printf.sprintf "\\<script language=javascript\\>
+              ) info.G.file_names;
+              
+              
+              let input_size = (Printf.sprintf "%d" ((String.length (file_best_name file))+3)) in
+              html_mods_td buf [
+                ("Rename file by selecting an alternate name", "sr br", "Filename");
+                ("Rename file", "sr", 
+                  Printf.sprintf "\\<script language=javascript\\>
 \\<!-- 
 function submitRenameForm(i) {
 var formID = document.getElementById(\\\"renameForm\\\" + i)
@@ -534,32 +586,35 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+renameTextOut+'\\\"';
 }
 //--\\>
 \\</script\\>" (file_num file) 
-              
-			  ^ "\\<table border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>\\<td\\>"
-              ^ "\\<form name=\\\"renameForm1\\\" id=\\\"renameForm1\\\" action=\\\"javascript:submitRenameForm(1);\\\"\\>"
-              ^ "\\<select name=\\\"newName\\\" id=\\\"newName\\\" onchange=\\\"this.form.submit()\\\"\\>"
-              ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\n" (file_best_name file) (file_best_name file)
-			  ^ !optionlist 
-              ^ "\\</select\\>\\</td\\>\\</tr\\>\\</form\\>\\<tr\\>\\<td\\>\\<form name=\\\"renameForm2\\\" id=\\\"renameForm2\\\" action=\\\"javascript:submitRenameForm(2);\\\"\\>"
-			  ^ "\\<input name=\\\"newName\\\" type=text size=" ^ input_size ^ " value=\\\"" ^ (file_best_name file) ^ "\\\"\\>\\</input\\>\\</td\\>\\</tr\\>\\</form\\>\\</table\\>") ];
-
+                  
+                  ^ "\\<table border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>\\<td\\>"
+                    ^ "\\<form name=\\\"renameForm1\\\" id=\\\"renameForm1\\\" action=\\\"javascript:submitRenameForm(1);\\\"\\>"
+                    ^ "\\<select name=\\\"newName\\\" id=\\\"newName\\\" onchange=\\\"this.form.submit()\\\"\\>"
+                    ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\n" (file_best_name file) (file_best_name file)
+                  ^ !optionlist 
+                    ^ "\\</select\\>\\</td\\>\\</tr\\>\\</form\\>\\<tr\\>\\<td\\>\\<form name=\\\"renameForm2\\\" id=\\\"renameForm2\\\" action=\\\"javascript:submitRenameForm(2);\\\"\\>"
+                    ^ "\\<input name=\\\"newName\\\" type=text size=" ^ input_size ^ " value=\\\"" ^ (file_best_name file) ^ "\\\"\\>\\</input\\>\\</td\\>\\</tr\\>\\</form\\>\\</table\\>"
+                  
+                  ) ];
+            
             )
       );
-
-            Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"; 
-			Printf.bprintf buf "\\</td\\>\\</tr\\>\\</table\\>\\</div\\>\\<br\\>";
+      
+      Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"; 
+      Printf.bprintf buf "\\</td\\>\\</tr\\>\\</table\\>\\</div\\>\\<br\\>";
     
-	end else
-	begin
-		Printf.bprintf buf "[%-s %5d] %-50s %10s %10s\npriority %d\n" 
-			n.network_name (file_num file) (match info.G.file_names with
-			[] -> Md4.to_string info.G.file_md4
-			| name :: _ -> name)
-			(Int64.to_string info.G.file_size)
-			(Int64.to_string info.G.file_downloaded)
-			(file_priority file);
-		Printf.bprintf buf "Chunks: [%-s]\n" info.G.file_chunks;
-		List.iter (fun name -> Printf.bprintf buf "    (%s)\n" name) info.G.file_names
+    end else
+    begin
+      Printf.bprintf buf "[%-s %5d] %s %10s %10s\npriority %d\n" 
+        n.network_name (file_num file) 
+      (match info.G.file_names with
+          [] -> Md4.to_string info.G.file_md4
+        | (name,_) :: _ -> String2.shorten !!max_name_len name)
+      (Int64.to_string info.G.file_size)
+      (Int64.to_string info.G.file_downloaded)
+      (file_priority file);
+      Printf.bprintf buf "Chunks: [%-s]\n" info.G.file_chunks;
+      List.iter (fun (name,_) -> Printf.bprintf buf "    (%s)\n" name) info.G.file_names
 	end;
   
   (try
@@ -610,12 +665,15 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+renameTextOut+'\\\"';
 
       let defaultHeader () = 
         
-        html_mods_table_header buf "sourcesTable" "sources al" [ 
+        html_mods_table_header buf "sourcesTable" "sources al" ([ 
           ( "1", "srh ac", "Client number (click to add as friend)", "Num" ) ; 
           ( "0", "srh", "[A] = Active download from client", "A" ) ; 
           ( "0", "srh", "Client state", "CS" ) ; 
           ( "0", "srh", "Client name", "Name" ) ; 
           ( "0", "srh", "Client brand", "CB" ) ; 
+	  ] @
+          (if !!emule_mods_count then [( "0", "srh", "eMule MOD", "EM" )] else [])
+	  @ [
           ( "0", "srh", "Overnet [T]rue, [F]alse", "O" ) ; 
           ( "0", "srh", "Connection [I]ndirect, [D]irect", "C" ) ; 
           ( "0", "srh br", "IP address", "IP address" ) ; 
@@ -636,7 +694,7 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+renameTextOut+'\\\"';
                 (Array.init (String.length info.G.file_chunks)
                 (fun i -> ((int_of_char info.G.file_chunks.[i])-48)))) ) ; 
           ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
-                (let fc = ref 0 in (String.iter (fun s -> if s = '2' then incr fc) info.G.file_chunks );!fc ))) ];
+                (let fc = ref 0 in (String.iter (fun s -> if s = '2' then incr fc) info.G.file_chunks );!fc ))) ]);
       
       in
       
@@ -664,7 +722,7 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+renameTextOut+'\\\"';
               Printf.bprintf buf "\\</tr\\>";
             end
           else
-            client_bprint c buf;
+            if !!all_sources_on_telnet then client_bprint c buf;
       ) srcs;
 
       if use_html_mods o && srcs <> [] then begin
@@ -673,32 +731,144 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+renameTextOut+'\\\"';
       end
 
   with _ -> ())
-(*
-let file_size file = 
-  (as_file_impl file).impl_file_size
   
-let file_disk_name file =
-  Unix32.filename (as_file_impl file).impl_file_fd
-      
-let file_fd file =
-  (as_file_impl file).impl_file_fd
 
-let set_file_disk_name file filename =
-  Unix32.rename (file_fd file) filename
+(*************************************************************************)
+(*                                                                       *)
+(*                         add_segment (internal)                        *)
+(*                                                                       *)
+(*************************************************************************)
+open Int64ops
   
-let file_downloaded file = (as_file_impl file).impl_file_downloaded
+let add_segment begin_pos end_pos segments =
+(*  lprintf "Adding segment %Ld - %Ld\n" begin_pos end_pos;  *)
+  match segments with
+    [] ->
+      (begin_pos, end_pos) :: segments
+  | (begin_pos2, end_pos2) :: tail ->
+      if begin_pos -- end_pos2 <= !!max_recover_gap then
+        (begin_pos2, end_pos) :: tail
+      else
+        (begin_pos, end_pos) :: segments
 
-let file_network file =
-  (as_file_impl file).impl_file_ops.op_file_network
-
-let file_priority file = 
-  (as_file_impl file).impl_file_priority
+(*************************************************************************)
+(*                                                                       *)
+(*                         recover_bytes                                 *)
+(*                                                                       *)
+(*************************************************************************)
   
-let file_set_priority file p = 
-  let impl = as_file_impl file in
-  if impl.impl_file_priority <> p then begin
-      impl.impl_file_priority <- p;      
-      impl.impl_file_ops.op_file_set_priority  impl.impl_file_val p;      
-      file_must_update file
-    end
+let recover_bytes file =
+  let size = file_size file in
+  let fd = file_fd file in
+  let len = 32768 in
+  let len64 = Int64.of_int len in
+  let s = String.create len in
+  
+  let rec iter_file_out file_pos segments =
+    
+(*    lprintf "iter_file_out pos = %Ld\n" file_pos; *)
+    if file_pos = size then segments else
+    
+    let max64 = min len64 (size -- file_pos) in
+    let max = Int64.to_int max64 in
+    if try
+        Unix32.read fd file_pos s 0 max;
+        true
+      with _ -> false
+    then 
+      iter_string_out (file_pos++max64) 0 max segments
+    else segments
+        
+  and iter_file_in file_pos begin_pos segments =
+(*    lprintf "iter_file_in file_pos = %Ld begin_pos = %Ld\n" file_pos begin_pos; *)
+    
+    if file_pos = size then
+      if begin_pos <> size then
+        add_segment begin_pos size segments 
+      else segments
+    else
+    
+    let max64 = min len64 (size -- file_pos) in
+    let max = Int64.to_int max64 in
+    if try
+        Unix32.read fd file_pos s 0 max;
+        true
+      with _ -> false
+    then 
+      iter_string_in (file_pos++max64) 0 max begin_pos segments
+    else
+    if begin_pos < file_pos then 
+      add_segment begin_pos file_pos segments
+    else segments
+  
+  and iter_string_out file_pos pos max segments =
+    
+(*    lprintf "iter_string_out file_pos = %Ld pos = %d len = %d\n"
+      file_pos pos max;
+    
+    iter_string_out_ file_pos pos max segments
+    
+  and  iter_string_out_ file_pos pos max segments = *)
+    if pos = max then
+      iter_file_out file_pos segments
+    else
+    if s.[pos] = '\000' then
+      iter_string_out file_pos (pos+1) max segments
+    else
+    let begin_pos = file_pos -- (Int64.of_int (max - pos)) in
+(*    lprintf "  Downloaded byte at %Ld\n" begin_pos; *)
+    iter_string_in file_pos (pos+1) max begin_pos  segments
+  
+  and iter_string_in file_pos pos max begin_pos segments =
+
+    (*
+    lprintf "iter_string_in file_pos = %Ld pos = %d len = %d begin_pos = %Ld\n"
+    file_pos pos max begin_pos;
+  
+    iter_string_in_ file_pos pos max begin_pos segments
+    
+  and iter_string_in_ file_pos pos max begin_pos segments = *)
+    
+    if pos = max then
+      iter_file_in file_pos begin_pos segments
+    else
+    if s.[pos] = '\000' then
+      let end_pos = file_pos -- (Int64.of_int (max - pos)) in
+(*      lprintf "  0 byte at %Ld\n" end_pos; *)
+      iter_string_out file_pos (pos+1) max 
+      (add_segment begin_pos end_pos segments)
+    else
+      iter_string_in file_pos (pos+1) max begin_pos segments
+    
+  in
+  let segments = List.rev (iter_file_out zero []) in
+  (*
+  lprintf "CommonFile.recover_bytes: recovered segments\n";
+  List.iter (fun (begin_pos, end_pos) ->
+      lprintf "   %Ld - %Ld\n" begin_pos end_pos
+  ) segments;
 *)
+  segments
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         MAIN                                          *)
+(*                                                                       *)
+(*************************************************************************)
+    
+let _ = 
+  Heap.add_memstat "CommonFile" (fun buf ->
+      let counter = ref 0 in
+      H.iter (fun _ -> incr counter) files_by_num;
+      Printf.bprintf buf "  files: %d\n" !counter;
+  )
+
+  
+(*************************************************************************)
+(*                                                                       *)
+(*                         Protect globals                               *)
+(*                                                                       *)
+(*************************************************************************)
+
+let com_files_by_num = files_by_num
+let files_by_num = ()
