@@ -19,7 +19,9 @@
 
 type 'a file_impl = {
   mutable impl_file_update : int;
-  mutable impl_file_state : CommonTypes.file_state;
+    mutable impl_file_state : CommonTypes.file_state;
+    
+    mutable impl_file_comment : string;
   mutable impl_file_num : int;
   mutable impl_file_val : 'a;
   mutable impl_file_ops : 'a file_ops;
@@ -46,7 +48,8 @@ and 'a file_ops = {
   mutable op_file_set_format : 'a -> CommonTypes.format -> unit;
   mutable op_file_check : 'a -> unit;
   mutable op_file_recover : 'a -> unit;
-  mutable op_file_sources : 'a -> CommonTypes.client list;
+  mutable op_file_all_sources : 'a -> CommonTypes.client list;
+  mutable op_file_active_sources : 'a -> CommonTypes.client list;
   mutable op_file_comment : 'a -> string;
   mutable op_file_set_priority : 'a -> int -> unit;
     mutable op_file_print_sources_html : 'a -> Buffer.t -> unit;
@@ -75,11 +78,12 @@ val file_resume : CommonTypes.file -> unit
 val set_file_state : CommonTypes.file -> CommonTypes.file_state -> unit
 val file_best_name : CommonTypes.file -> string
 val set_file_best_name : CommonTypes.file -> string -> unit
-val file_set_format : CommonTypes.file -> CommonTypes.format -> unit
+val set_file_format : CommonTypes.file -> CommonTypes.format -> unit
 val file_check : CommonTypes.file -> unit
 val file_recover : CommonTypes.file -> unit
 val file_preview : CommonTypes.file -> unit
-val file_sources : CommonTypes.file -> CommonTypes.client list
+val file_all_sources : CommonTypes.file -> CommonTypes.client list
+val file_active_sources : CommonTypes.file -> CommonTypes.client list
 val file_print_sources_html : CommonTypes.file -> Buffer.t -> unit
 val files_ops : (int file_ops * int file_ops) list ref
 val new_file_ops : CommonTypes.network -> 'a file_ops
@@ -103,6 +107,11 @@ val set_file_disk_name : CommonTypes.file -> string -> unit
 val file_downloaded : CommonTypes.file -> int64
 val file_network : CommonTypes.file -> CommonTypes.network
 val file_priority : CommonTypes.file -> int
-val file_set_priority : CommonTypes.file -> int -> unit
+val set_file_priority : CommonTypes.file -> int -> unit
+val set_file_last_seen : CommonTypes.file -> int -> unit
 val file_debug : CommonTypes.file -> string
+val set_file_comment : CommonTypes.file -> string -> unit
+val file_comment : CommonTypes.file -> string
+  
+val recover_bytes : CommonTypes.file -> (int64 * int64) list
   
\ No newline at end of file
