@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open Int64ops
 open Printf2
 open Options
 open CommonOptions
@@ -25,13 +26,49 @@ open CommonTypes
 open UdpSocket
 open TcpBufferedSocket
 
-let zero = Int64.zero
-let one = Int64.one
-let (++) = Int64.add
-let (--) = Int64.sub
-let ( ** ) x y = Int64.mul x (Int64.of_int y)
-let ( // ) x y = Int64.div x y
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         short_lazy                                    *)
+(*                                                                       *)
+(*************************************************************************)  
   
+(* Store the result of a computation for a very short time to avoid
+recomputing it too often. Each value is associated with a uniq
+ '(string, int1, int2)' that should be provided by the caller.
+  *)
+
+module ShortLazy : sig
+    val compute : string * int * int -> ('a -> 'b) -> 'a -> 'b
+  end = struct
+
+    type t
+    
+    let short_lazy_values = Hashtbl.create 111
+    let compute name f x =
+      let f = (Obj.magic f : t -> t) in
+      let x = (Obj.magic x : t) in
+      try
+        let (f',x',v) = Hashtbl.find short_lazy_values name in
+        
+        if f' != f || x <> x' then
+          (Hashtbl.remove short_lazy_values name; raise Not_found);
+        Obj.magic (v : t)
+      with _ ->
+          let v = f x in
+          Hashtbl.add short_lazy_values name (f,x,v);
+          Obj.magic (v : t)
+    
+    let _ =
+      add_infinite_timer 5. (fun _ -> Hashtbl.clear short_lazy_values)
+  end
+  
+(*************************************************************************)
+(*                                                                       *)
+(*                         ............                                  *)
+(*                                                                       *)
+(*************************************************************************)
+
 let networks_string = ref ""
   
 let version () = 
@@ -43,6 +80,18 @@ let version () =
 in an option, and then change the option accordingly. ?> *)
 let find_other_port = ref false
 
+let shorten str limit =
+  let name = String.escaped str in
+  let len = String.length name in
+  let max_len = maxi limit 10 in
+  if len > max_len then
+    let prefix = String.sub name 0 (max_len -7) in
+    let suffix = String.sub name (len-4) 4 in
+    Printf.sprintf "%s...%s" prefix suffix
+  else name
+
+let client_short_name c =
+  shorten c !!max_client_name_len
   
 let find_port server_name bind_addr port_option handler =
   if !!port_option <> 0 then
@@ -126,9 +175,11 @@ let connection_delay cc =
   cc.control_state <- 0
   
 let upload_control = TcpBufferedSocket.create_write_bandwidth_controler 
+    "Upload"
     (!!max_hard_upload_rate * 1024)
   
 let download_control = TcpBufferedSocket.create_read_bandwidth_controler 
+  "Download"
     (!!max_hard_download_rate * 1024)
 
       
@@ -145,67 +196,16 @@ let udp_write_controler = UdpSocket.new_bandwidth_controler upload_control
 
 let udp_read_controler = UdpSocket.new_bandwidth_controler download_control
   
-let can_open_connection () =
-  let ns = nb_sockets () in
-  let max = mini !!max_opened_connections MlUnix.max_sockets in
-  (*
-  if !!debug_net then begin
-      lprintf "CAN OPEN (conns: %d < %d && upload U/D: %d %d)\n" ns max 
-        (UdpSocket.remaining_bytes udp_write_controler)
-        (UdpSocket.remaining_bytes udp_read_controler);
-    end; *)
-  ns < max
-  
-(*
-  
-    
-val verify_ip : TcpBufferedSocket.t -> unit
-
-  
-val ip_verified : int ref
-  
-  
-val can_open_connection : unit -> bool
-      
-val new_connection_control : float -> CommonTypes.connection_control
-val connection_ok : CommonTypes.connection_control -> unit
-val connection_failed : CommonTypes.connection_control -> unit
-val connection_can_try : CommonTypes.connection_control -> bool
-val connection_must_try : CommonTypes.connection_control -> unit
-val connection_set_last_conn : CommonTypes.connection_control -> float -> unit
-val connection_last_conn :  CommonTypes.connection_control -> float
-val connection_try : CommonTypes.connection_control -> unit
-val connection_delay : CommonTypes.connection_control -> unit
-  
-val printf_char : char -> unit
-val printf_string : string -> unit
-  
-val one_day : float
-val half_day : float
-
-val upload_control : TcpBufferedSocket.bandwidth_controler
-val download_control : TcpBufferedSocket.bandwidth_controler
-
-*)
-  
 let gui_server_sock = ref (None : TcpServerSocket.t option)
-      
-let exit_handlers = ref []
+  
+let pid = Unix.getpid ()
+  
 let do_at_exit f =
-  exit_handlers := f :: !exit_handlers
+  Pervasives.at_exit (fun _ -> 
+      if Unix.getpid () = pid then
+        try f () with e -> ())
       
-let exit_properly n =
-(*  lprintf "exit_properly handlers\n"; *)
-  List.iter (fun f -> try 
-(*        lprintf "exit_properly handler ...\n";  *)
-        f () ;
-(*        lprintf "exit_properly done\n"; *)
-      with e -> 
-          lprintf "exit_properly (exception %s)"
-            (Printexc2.to_string e); lprint_newline ();
-  ) !exit_handlers;
-(*  lprintf "exit_properly DONE\n"; *)
-  Pervasives.exit n
+let exit_properly n = Pervasives.exit n
 
 let user_socks = ref ([] : TcpBufferedSocket.t list)
 let dialog_history = ref ([] : (int * string * string) list )
@@ -330,6 +330,13 @@ let string_of_tags tags =
   ) tags;
   Buffer.contents buf
 
+let rec find_tag name tags =
+  match tags with
+    [] -> raise Not_found
+  | { tag_name = tag_name; tag_value = v } :: _ when tag_name = name -> v
+  | _ :: tail -> find_tag name tail
+      
+      
       
   
   (* first GUI have gui_num = 2, since newly created objects have _update = 1 *)
@@ -382,13 +389,15 @@ let  upload_counter = ref Int64.zero
 let  download_counter = ref Int64.zero
 let nshared_files = ref 0
 let shared_counter = ref Int64.zero
-  
-  
+
+let tag_name tag =
+  String.escaped tag.tag_name 
+        
 let rec print_tags tags =
   match tags with
     [] -> ()
   | tag :: tags ->
-      lprintf "  \"%s\" = " (String.escaped tag.tag_name);
+      lprintf "  \"%s\" = " (tag_name tag);
       begin
         match tag.tag_value with
         | Uint64 n -> lprintf "%s" (Int64.to_string n)
@@ -401,10 +410,9 @@ let rec print_tags tags =
 
 let rec fprint_tags oc tags =
   match tags with
-    [] -> Printf.fprintf oc "\n";
-          ()
+    [] -> Printf.fprintf oc "\n"
   | tag :: tags ->
-      Printf.fprintf oc "%s = " (String.escaped tag.tag_name);
+      Printf.fprintf oc "%s = " (tag_name tag);
       begin
         match tag.tag_value with
         | Uint64 n -> Printf.fprintf oc "%s" (Int64.to_string n)
@@ -417,10 +425,9 @@ let rec fprint_tags oc tags =
 
 let rec bprint_tags buf tags =
   match tags with
-    [] -> Printf.bprintf buf "\n";
-          ()
+    [] -> Printf.bprintf buf "\n"
   | tag :: tags ->
-      Printf.bprintf buf "%s = " (String.escaped tag.tag_name);
+      Printf.bprintf buf "%s = " (tag_name tag);
       begin
         match tag.tag_value with
         | Uint64 n -> Printf.bprintf buf "%s" (Int64.to_string n)
@@ -449,44 +456,6 @@ let add_init_hook f =
   init_hooks := f :: !init_hooks
 
 
-let file_kinds = ref []
-
-let add_web_kind kind f =
-  file_kinds := (kind,f) :: !file_kinds
-
-let mldonkey_wget url f = 
-  let module H = Http_client in
-  let r = {
-      H.basic_request with
-      H.req_url = Url.of_string url;
-      H.req_proxy = !CommonOptions.http_proxy;
-      H.req_user_agent = 
-      Printf.sprintf "MLdonkey %s" Autoconf.current_version;
-    } in
-  
-  H.wget r f  
-  
-  
-let load_url kind url =
-  lprintf "QUERY URL %s\n" url; 
-  let f = 
-    try 
-      List.assoc kind !file_kinds 
-    with e -> failwith (Printf.sprintf "Unknown kind [%s]" kind)
-  in 
-  try
-    mldonkey_wget url f
-  with e -> failwith (Printf.sprintf "Exception %s while loading %s"
-          (Printexc2.to_string e) url)
-  
-let load_file kind file =
-  try 
-    (List.assoc kind !file_kinds) file
-  with e -> 
-      lprintf "Exception %s while loading kind %s\n" 
-        (Printexc2.to_string e)
-      kind
-
 let chat_message_fifo = (Fifo.create () : (int * string * int * string * string) Fifo.t) 
 
 let log_chat_message i num n s = 
@@ -666,34 +635,204 @@ let detected_uplink_capacity () =
 let int_tag s i = 
   { tag_name = s; tag_value = Uint64 (Int64.of_int i) }
 
-let int32_tag s i = 
+let int64_tag s i = 
   { tag_name = s; tag_value = Uint64 i }
 
 let string_tag s i = 
   { tag_name = s; tag_value = String i }
 
+let for_int_tag tag f =
+  match tag.tag_value with
+    Uint64 i | Fint64 i -> f (Int64.to_int i)
+  | String _ -> ()
+  | Addr _ -> ()
+
+let for_int64_tag tag f =
+  match tag.tag_value with
+    Uint64 i | Fint64 i -> f i
+  | String _ -> ()
+  | Addr _ -> ()
+
+let for_string_tag tag f =
+  match tag.tag_value with
+    Uint64 _ | Fint64 _ -> ()
+  | String s -> f s
+  | Addr _ -> ()
+      
 (* Name,FrameHeight *)
 let html_mods_styles = ref
-  [| ("Green",45) ; ("Tang",45); ("L.Blue",45); 
-	 ("L.Purple",45); ("Monochrome",45); ("Corona",45); |]
+  [| ("Green",42) ; ("Tang",42); ("L.Blue",42); 
+	 ("L.Purple",42); ("Monochrome",42); ("Corona",42); |]
 
 let partial_chunk c =
   match c with
     '0' | '1' -> true
   | _ -> false
-    
-let waiting_connections = Fifo.create ()
-  
-let schedule_connections () =
-  let max_wanted = mini 
-      (!!max_connections_per_second + nb_sockets ()) MlUnix.max_sockets in
-  try
-    while nb_sockets () < max_wanted do
-      let f = Fifo.take waiting_connections in
-      try f () with _ -> ()
-    done
-  with _ -> ()
+
+module Connections = struct
       
-let add_pending_connection f =
-  Fifo.put waiting_connections f
+  end
   
+      
+let parse_magnet url =
+  let url = Url.of_string url in
+  if url.Url.short_file = "magnet:" then 
+    let uids = ref [] in
+    let name = ref "" in
+    List.iter (fun (value, arg) ->
+        if String2.starts_with value "xt" then
+          uids := Uid.expand (Uid.of_string arg :: !uids)
+        else 
+        if String2.starts_with value "dn" then
+          name := Url.decode arg
+        else 
+        if arg = "" then
+(* This is an error in the magnet, where a & has been kept instead of being
+  url-encoded *)
+          name := Printf.sprintf "%s&%s" !name value
+        else
+          lprintf "MAGNET: unused field %s = %s\n"
+            value arg
+    ) url.Url.args;
+    !name, !uids
+  else raise Not_found
+
+    (*
+module CanBeCompressed = struct
+    
+    let to_deflate = ref []
+    let to_deflate_len = ref 0
+        
+    let compression_buffer_len = 20000
+    let compression_buffer = String.create compression_buffer_len
+    
+    let deflate_connection sock =
+      lprintf "Creating deflate connection\n";
+      let comp = Deflate (Zlib.inflate_init true, Zlib.deflate_init 6 true) in
+      CompressedConnection (comp, 
+        buf_create !max_buffer_size, buf_create !max_buffer_size, sock)
+    
+    let rec iter_deflate sock zs wbuf =
+      if wbuf.len > 0 then begin
+          lprintf "iter_deflate\n";
+          let (_, used_in, used_out) = Zlib.deflate zs
+              wbuf.buf wbuf.pos wbuf.len 
+              compression_buffer 0 compression_buffer_len
+              Zlib.Z_SYNC_FLUSH in      
+          lprintf "deflated %d/%d -> %d\n" used_in wbuf.len used_out;
+          lprintf "[%s]\n" (String.escaped (String.sub compression_buffer 0 used_out));
+          write sock compression_buffer 0 used_out;
+          buf_used wbuf used_in;
+          if used_in > 0 || used_out > 0 then
+            iter_deflate sock zs wbuf
+        end
+        
+    let deflate_timer _ =
+      List.iter (fun conn ->
+          try 
+            match conn with
+              CompressedConnection (comp, _, wbuf, sock) ->
+                if closed sock then raise Exit;
+                let Deflate (_, zs) = comp in
+                iter_deflate sock zs wbuf
+            | _ -> ()
+          with e -> 
+              lprintf "[ERROR] Exception %s in CanBeCompressed.deflate_timer\n"
+                (Printexc2.to_string e)
+      ) !to_deflate;
+      to_deflate := [];
+      to_deflate_len := 0
+    
+    let to_deflate conn =
+      if not (List.memq conn !to_deflate) then
+        to_deflate := conn :: !to_deflate;
+      if !to_deflate_len > 1000000 then
+        deflate_timer ()
+    
+    let write_string conn s =
+      lprintf "write_string\n";
+      let len = String.length s in
+      match conn with
+        Connection sock -> write_string sock s
+      | CompressedConnection (_,_,wbuf,sock) ->
+          lprintf "CanBeCompressed.write_string %d\n" len;
+          to_deflate_len := !to_deflate_len + len;
+          to_deflate conn;
+          buf_add sock wbuf s 0 len
+      | _ -> assert false
+    
+    let rec iter_inflate zs sock b rbuf =
+      if b.len > 0 then begin
+          lprintf "iter_inflate %d\n" b.len;
+          lprintf "[%s]\n" (String.escaped (String.sub b.buf b.pos b.len));
+          let (_, used_in, used_out) = Zlib.inflate zs b.buf b.pos b.len 
+              compression_buffer 0 compression_buffer_len
+              Zlib.Z_SYNC_FLUSH in
+          lprintf "inflated %d/%d -> %d\n" used_in b.len used_out;
+          lprintf "[%s]\n" (String.escaped (String.sub compression_buffer 0 used_out));
+          buf_add sock rbuf compression_buffer 0 used_out;
+          buf_used b used_in;
+          if used_in > 0 || used_out > 0 then
+            iter_inflate zs sock b rbuf
+        end
+        
+    let buf conn =
+      lprintf "CanBeCompressed.buf\n";
+      try
+        match conn with
+          Connection sock -> buf sock
+        | CompressedConnection (comp,rbuf,_,sock) ->
+            
+            let b = buf sock in
+            let Deflate (zs, _) = comp in
+            if b.len > 0 then iter_inflate zs sock b rbuf;
+            rbuf
+        | _ -> assert false
+      with e ->
+          lprintf "[ERROR] Exception %s in CanBeCompressed.buf\n"
+            (Printexc2.to_string e);
+          raise e
+  end
+    *)
+
+let ip_reachable ip =
+  !!allow_local_network || Ip.reachable ip
+  
+let do_if_connected tcp_connection f =
+  match tcp_connection with
+    Connection sock -> f sock
+  | _ -> ()
+  
+let print_localtime () =
+let t = Unix.localtime (Unix.time ()) in
+  let { Unix.tm_mon = tm_mon; Unix.tm_mday = tm_mday; Unix.tm_hour = tm_hour; Unix.tm_min = tm_min; Unix.tm_sec = tm_sec } = t in
+  lprintf " on localtime: %2d/%2d, %02d:%02d:%02d\n" tm_mday (tm_mon+1) tm_hour tm_min tm_sec
+
+      
+let new_activity () = {
+    activity_begin = BasicSocket.last_time ();
+    activity_client_overnet_connections = 0;
+    activity_client_overnet_indirect_connections = 0;
+    activity_client_overnet_successful_connections = 0;
+    activity_client_edonkey_connections = 0;
+    activity_client_edonkey_indirect_connections = 0;
+    activity_client_edonkey_successful_connections = 0;
+    activity_server_edonkey_connections = 0;
+    activity_server_edonkey_successful_connections = 0;
+    }
+
+let nactivities = ref 0
+let activities = Fifo.create ()
+let activity = ref (new_activity ())
+  
+let _ =
+  add_infinite_timer 60. (fun _ ->
+      Fifo.put activities !activity;
+      incr nactivities;
+      if !nactivities > 2000 then begin
+          ignore (Fifo.take activities);
+          decr nactivities
+        end;
+      activity := new_activity ()
+  )
+  
\ No newline at end of file
