@@ -23,14 +23,267 @@ open Printf2
 open CommonOptions
 open BasicSocket  
 open TcpBufferedSocket
+open Options
+  
+open CommonClient
+open CommonServer
+open CommonNetwork
+open CommonOptions
+open CommonTypes
+open CommonFile
 open CommonGlobals
 open CommonSearch
-open Options
-open CommonNetwork
 open CommonResult
 open CommonServer
 open CommonTypes
+open CommonComplexOptions
+  
+  
+  
+(*************  ADD/REMOVE FUNCTIONS ************)
 
+let canonize_basename name =
+  let name = String.copy name in
+  for i = 0 to String.length name - 1 do
+    match name.[i] with
+    | '/' | '\\' -> name.[i] <- '_'
+    | _ -> ()
+  done;
+  name
+  
+let file_commited_name file =   
+  let network = file_network file in
+  let best_name = file_best_name file in
+  let file_name = file_disk_name file in
+  let incoming_dir =
+    if network.network_incoming_subdir () <> "" then
+      Filename.concat !!incoming_directory
+        (network.network_incoming_subdir ())
+    else !!incoming_directory
+  in
+  (try Unix2.safe_mkdir incoming_dir with _ -> ());
+  let new_name = 
+    Filename.concat incoming_dir (canonize_basename 
+        (file_best_name file))
+  in
+  let new_name = 
+    if Sys.file_exists new_name then
+      let rec iter num =
+        let new_name = Printf.sprintf "%s.%d" new_name num in
+        if Sys.file_exists new_name then
+          iter (num+1)
+        else new_name
+      in
+      iter 1
+    else new_name in
+  new_name  
+  
+(********
+These two functions 'file_commit' and 'file_cancel' should be the two only
+functions in mldonkey able to destroy a file, the first one by moving it, 
+the second one by deleting it. 
+
+Note that when the network specific file_commit function is called, the
+file has already been moved to the incoming/ directory under its new
+name.
+*)
+  
+let file_commit file =
+  let impl = as_file_impl file in
+  if impl.impl_file_state = FileDownloaded then
+    let new_name = file_commited_name file in
+    try
+      set_file_disk_name file new_name;
+      let best_name = file_best_name file in  
+      Unix32.destroy (file_fd file);
+      (* Commit the file first, and share it after... *)
+      impl.impl_file_ops.op_file_commit impl.impl_file_val new_name;
+      if not (Unix2.is_directory new_name) then 
+        ignore (CommonShared.new_shared 
+            !!incoming_directory !!incoming_directory_prio best_name new_name);
+      done_files =:= List2.removeq file !!done_files;
+      update_file_state impl FileShared;
+    with e ->
+      lprintf "Exception in file_commit: %s\n" (Printexc2.to_string e)
+      
+let file_cancel file =
+  try
+    let impl = as_file_impl file in    
+    if impl.impl_file_state <> FileCancelled then begin
+        update_file_state impl FileCancelled;
+        impl.impl_file_ops.op_file_cancel impl.impl_file_val;
+        (try  Unix32.remove (file_fd file)  with e -> 
+              lprintf "Sys.remove %s exception %s\n" 
+                (file_disk_name file)
+              (Printexc2.to_string e); );
+        Unix32.destroy (file_fd file);
+        files =:= List2.removeq file !!files;
+    end
+  with e ->
+      lprintf "Exception in file_cancel: %s\n" (Printexc2.to_string e)
+
+        
+let mail_for_completed_file file =
+  if !!mail <> "" then
+    let module M = Mailer in
+    let line1 = "mldonkey has completed the download of:\r\n\r\n" in
+
+    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\nComment: %s\r\n" 
+      (file_best_name file)
+      (file_size file)
+      (file_comment file)
+    in
+    
+    let subject = if !!filename_in_subject then
+        Printf.sprintf "[mldonkey] file received - %s"
+        (file_best_name file)
+      else
+        Printf.sprintf "mldonkey, file received";        
+    in
+    
+    let mail = {
+        M.mail_to = !!mail;
+        M.mail_from = !!mail;
+        M.mail_subject = subject;
+        M.mail_body = line1 ^ line2;
+      } in
+    M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
+
+let chat_for_completed_file file =
+  CommonChat.send_warning_for_downloaded_file (file_best_name file)
+
+      
+let file_completed (file : file) =
+  try
+    let impl = as_file_impl file in
+    if impl.impl_file_state = FileDownloading then begin
+        files =:= List2.removeq file !!files;
+        done_files =:= file :: !!done_files;
+        update_file_state impl FileDownloaded;  
+        let file_name = file_disk_name file in
+        let file_id = Filename.basename file_name in
+        ignore (CommonShared.new_shared "completed" 0 (
+            file_best_name file )
+          file_name);
+        (try mail_for_completed_file file with e ->
+              lprintf "Exception %s in sendmail\n" (Printexc2.to_string e);
+              );
+        if !!CommonOptions.chat_warning_for_downloaded then
+          chat_for_completed_file file;
+        
+        if !!file_completed_cmd <> "" then begin
+            MlUnix.fork_and_exec  !!file_completed_cmd 
+              [|
+              file_name;
+              file_id;
+              Int64.to_string (file_size file);
+              file_best_name file
+            |]
+          
+          end
+      
+      
+      end
+  with e ->
+      lprintf "Exception in file_completed: %s\n" (Printexc2.to_string e)
+      
+let file_add impl state = 
+  try
+    let file = as_file impl in
+    if impl.impl_file_state = FileNew then begin
+        update_file_num impl;
+        (match state with
+            FileDownloaded -> 
+              done_files =:= file :: !!done_files;
+          | FileShared
+          | FileNew
+          | FileCancelled -> ()
+              
+          | FileAborted _
+          | FileDownloading
+          | FileQueued
+          | FilePaused -> 
+              files =:= file :: !!files);
+        update_file_state impl state
+      end
+  with e ->
+      lprintf "Exception in file_add: %s\n" (Printexc2.to_string e)
+      
+let server_remove server =
+  try
+    let impl = as_server_impl server in
+    if impl.impl_server_state <> RemovedHost then begin
+        set_server_state server RemovedHost;
+        (try impl.impl_server_ops.op_server_remove impl.impl_server_val
+          with _ -> ());
+        servers =:= Intmap.remove (server_num server) !!servers;
+      end
+  with e ->
+      lprintf "Exception in server_remove: %s\n" (Printexc2.to_string e)
+  
+let server_add impl =
+  let server = as_server impl in
+  if impl.impl_server_state = NewHost then begin
+      server_update_num impl;
+      servers =:= Intmap.add (server_num server) server !!servers;
+      impl.impl_server_state <- NotConnected (BasicSocket.Closed_by_user, -1);
+    end
+
+let friend_add c =
+  let impl = as_client_impl c in
+  if not (is_friend c) then begin
+      set_friend c;
+      client_must_update c;
+      friends =:= c :: !!friends;
+      contacts := List2.removeq c !contacts;
+      impl.impl_client_ops.op_client_browse impl.impl_client_val true
+    end
+    
+(* Maybe we should not add the client to the contact list and completely remove
+it ? *)
+let friend_remove c =
+  try
+    let impl = as_client_impl c in
+    if is_friend c then begin
+        set_not_friend c;
+        client_must_update c;
+        friends =:= List2.removeq c !!friends;
+        impl.impl_client_ops.op_client_clear_files impl.impl_client_val
+      end else
+    if is_contact c then begin
+        set_not_contact c;
+        client_must_update c;
+        contacts := List2.removeq c !contacts;
+        impl.impl_client_ops.op_client_clear_files impl.impl_client_val
+      end        
+      
+  with e ->
+      lprintf "Exception in friend_remove: %s\n" (Printexc2.to_string e)
+  
+let contact_add c =
+  let impl = as_client_impl c in
+  if not (is_friend c || is_contact c) then begin
+      set_contact c;
+      client_must_update c;
+      contacts := c :: !contacts;
+      impl.impl_client_ops.op_client_browse impl.impl_client_val true
+    end
+    
+let contact_remove c =
+  try
+    let impl = as_client_impl c in
+    if is_contact c then begin
+        set_not_contact c;
+        client_must_update c;
+        contacts := List2.removeq c !contacts;
+        impl.impl_client_ops.op_client_clear_files impl.impl_client_val
+      end
+  with e ->
+      lprintf "Exception in contact_remove: %s\n" (Printexc2.to_string e)
+
+  
+  
+  
 let time_of_sec sec = 
   let hours = sec / 60 / 60 in
   let rest = sec - hours * 60 * 60 in
@@ -67,84 +320,6 @@ let size_of_int64 size =
     Int64.to_string size
 
   
-let days = ref 0      
-let hours = ref 0    
-  
-let cut_messages f sock nread =
-  let b = buf sock in
-  try
-    while b.len >= 4 do
-      let msg_len = LittleEndian.get_int b.buf b.pos in
-      if b.len >= 4 + msg_len then
-        begin
-          let s = String.sub b.buf (b.pos+4) msg_len in
-          buf_used sock (msg_len + 4);
-          let opcode = LittleEndian.get_int16 s 0 in
-          (f opcode s : unit)
-        end
-      else raise Not_found
-    done
-  with Not_found -> ()
-  
-let load_web_infos () =
-(* Try to connect the redirector to get interesting information, since we
-are not allowed to use savannah anymore. The redirector should be able to
-support the charge, at least, currently. *)
-  let (name, port) = !!mlnet_redirector in
-  Ip.async_ip name (fun ip ->
-      try
-        lprintf "connecting to redirector\n";
-        let sock = TcpBufferedSocket.connect "connect redirector"
-            (Ip.to_inet_addr ip) port            
-            (fun sock event ->
-              match event with
-              | BASIC_EVENT (LTIMEOUT | RTIMEOUT) -> 
-                  TcpBufferedSocket.close sock Closed_for_timeout
-              | _ -> ())
-        in
-
-        TcpBufferedSocket.set_rtimeout sock 30.;
-        
-        let to_read = ref [] in
-        set_reader sock (cut_messages (fun opcode s ->
-              lprintf "redirector info received\n";
-              let module L = LittleEndian in
-              
-              let motd_html_s, pos = L.get_string16 s 2 in
-              let servers_met_s, pos = L.get_string16 s pos in
-              let peers_ocl_s, pos = L.get_string16 s pos in
-              let motd_conf_s, pos = L.get_string16 s pos in
-              
-              motd_html =:= motd_html_s;
-              
-              let servers_met_file = Filename.temp_file "servers" ".met" in
-              File.from_string servers_met_file servers_met_s;
-              load_file "servers.met" servers_met_file;
-
-              let peers_ocl_file = Filename.temp_file "peers" ".ocl" in
-              File.from_string peers_ocl_file peers_ocl_s;
-              load_file "ocl" peers_ocl_file;
-
-              let motd_conf_file = Filename.temp_file "motd" ".conf" in
-              File.from_string motd_conf_file motd_conf_s;
-              load_file "motd.conf" motd_conf_file;              
-              
-              lprintf "Redirector info loaded\n";
-              TcpBufferedSocket.close sock Closed_by_user
-          ))
-      with e -> 
-          lprintf "Exception %s while connecting redirector\n"
-            (Printexc2.to_string e)
-  );
-  
-  if !!network_update_url <> "" then begin
-    load_url "motd.html" (Filename.concat !!network_update_url "motd.html");
-    load_url "motd.conf" (Filename.concat !!network_update_url "motd.conf");
-  end;
-  List.iter (fun (kind, period, url) ->
-      if !days mod period = 0 then load_url kind url
-  ) !!CommonOptions.web_infos
-
 let display_vd = ref false
   
 let print_results o =
@@ -162,6 +337,16 @@ let print_results o =
           incr counter;
       ) q.search_results
 
+
+let start_download file =
+  
+  if !!file_started_cmd <> "" then 
+      MlUnix.fork_and_exec  !!file_started_cmd 
+      [|
+      !!file_started_cmd;
+        "-file";
+        string_of_int (CommonFile.file_num file);
+      |]
       
 let download_file o arg =
   let user = o.conn_user in
@@ -172,7 +357,8 @@ let download_file o arg =
         None -> "no last search"
       | Some s ->
           let result = List.assoc (int_of_string arg) user.ui_last_results  in
-          CommonResult.result_download result [] false; 
+          let file = CommonResult.result_download result [] false in
+          start_download file;
           "download started"
     with
     | Failure s -> s
@@ -220,7 +406,7 @@ let print_connected_servers o =
   
 let send_custom_query user buf query args = 
   try
-    let q = List.assoc query !!CommonComplexOptions.customized_queries in
+    let q = List.assoc query (CommonComplexOptions.customized_queries()) in
     let args = ref args in
     let get_arg arg_name = 
 (*      lprintf "Getting %s\n" arg_name; *)
@@ -378,69 +564,80 @@ let send_custom_query user buf query args =
         (Printexc2.to_string e)
 
 let sort_options l =
-  List.sort (fun (option_name1, _) (option_name2, _) ->
-	       String.compare option_name1 option_name2) l
-
+  List.sort (fun o1 o2 ->
+      String.compare o1.option_name o2.option_name) l
+    
+let opfile_args r opfile =
+  let prefix = r.network_prefix () in
+  let args = simple_options prefix opfile in
+  args
+  
 let all_simple_options () =
-  let options = ref (
-      sort_options ((simple_options downloads_ini) 
-		    @ (simple_options downloads_expert_ini))
+  let options = ref (sort_options
+      (simple_options "" downloads_ini) 
     )
   in
   networks_iter_all (fun r ->
       List.iter (fun opfile ->
-          let args = simple_options opfile in
-          let prefix = r.network_prefix () in
-          let args = 
-            if prefix = "" then args else 
-              List2.tail_map (fun (arg, value) ->
-                (Printf.sprintf "%s%s" prefix arg, value)) 
-            args
-          in
-          options := !options @ args)
-      r.network_config_file 
-  );
-  !options
-
-let all_simple_options_html () =
-  let options = ref (
-      (simple_options_html downloads_ini) @
-      (simple_options_html downloads_expert_ini)) in
-  networks_iter_all (fun r ->
-      List.iter (fun opfile ->
-          let args = simple_options_html opfile in
-          let prefix = r.network_prefix () in
-          let args = 
-            if prefix = "" then args else 
-              List2.tail_map (fun (arg, value, def, help) ->
-                (Printf.sprintf "%s%s" prefix arg, value, def, help)) 
-            args
-          in
-          options := !options @ args
+          options := !options @ (opfile_args r opfile)
       )
       r.network_config_file 
   );
   !options
 
+let parse_simple_options args = 
+  let v = all_simple_options () in
+  match args with
+    [] -> v 
+    | args ->
+      let match_star = Str.regexp "\\*" in
+      let options_filter = Str.regexp ("^\\(" 
+        ^ (List.fold_left (fun acc a -> acc 
+        ^ (if acc <> "" then "\\|" else "") 
+        ^ (Str.global_replace match_star ".*" a)) "" args) 
+        ^ "\\)$") in
+      List.filter (fun o -> Str.string_match options_filter o.option_name 0) v
+
+let some_simple_options num =
+               let cnt = ref 0 in
+               let options = ref [] in
+               networks_iter_all (fun r ->
+                       List.iter (fun opfile ->
+                               if !cnt = num then begin
+          options := !options @ (opfile_args r opfile)
+
+                       end;
+                       incr cnt
+                       ) r.network_config_file
+               );
+               !options
+
+let all_active_network_opfile_network_names () =
+       let names = ref [] in
+       networks_iter_all (fun r ->
+               List.iter (fun opfile ->
+                       names := !names @ [r.network_name]
+               ) r.network_config_file
+       );
+       !names
+
 let apply_on_fully_qualified_options name f =
-  if !verbose then begin
-      lprintf "For option %s\n" name; 
-    end;
+  if !verbose then lprintf "For option %s\n" name; 
   let rec iter prefix opfile =
-    let args = simple_options opfile in
-    List.iter (fun (old_name, old_value) ->
-        let new_name = Printf.sprintf "%s%s" prefix old_name in
-        if new_name = name then
-          (f opfile old_name old_value; raise Exit))
+    let args = simple_options prefix opfile in
+    List.iter (fun o ->
+(*        lprintf "Compare [%s] [%s]\n" o.option_name name; *)
+        if o.option_name = name then
+          (f opfile o.option_shortname o.option_value; raise Exit))
     args
   in
   try
     iter "" downloads_ini;
-    iter "" downloads_expert_ini;
     if not (networks_iter_all_until_true (fun r ->
             try
               List.iter (fun opfile ->
                   let prefix = r.network_prefix () in
+(*                  lprintf "Prefix [%s]\n" prefix; *)
                   iter prefix opfile;
               )
               r.network_config_file ;
@@ -455,8 +652,7 @@ let apply_on_fully_qualified_options name f =
 
 let set_fully_qualified_options name value =
   apply_on_fully_qualified_options name (fun opfile old_name old_value ->
-    set_simple_option opfile old_name value)
-
+      set_simple_option opfile old_name value)
 
 let get_fully_qualified_options name =
   let value = ref None in
@@ -521,7 +717,7 @@ let register_gui_options_panel name panel =
     
     
 let _ =
-  add_infinite_option_timer filter_search_delay (fun _ ->
+  add_infinite_timer filter_search_delay (fun _ ->
 (*      if !!filter_search then *) begin
 (*          lprintf "Filter search results\n"; *)
           List.iter (fun user ->
