@@ -72,6 +72,8 @@ let check_network_implementations () =
         lprintf "op_network_file_of_option";
       if c.op_network_client_of_option == cc.op_network_client_of_option then 
         lprintf "op_network_client_of_option";
+      if c.op_network_recover_temp == cc.op_network_recover_temp then 
+        lprintf "op_network_recover_temp\n";
       if c.op_network_search == cc.op_network_search then 
         lprintf "op_network_search\n";
       if c.op_network_share == cc.op_network_share then 
@@ -102,9 +104,10 @@ let network_load_complex_options n = n.op_network_load_complex_options ()
 let network_enable n =  n.op_network_enable ()
 let network_disable n = n.op_network_disable () 
 let network_share n s = n.op_network_share s
+let network_recover_temp n = n.op_network_recover_temp ()
 let network_add_server n s = n.op_network_add_server s
 let network_server_of_option n s = n.op_network_server_of_option s
-let network_file_of_option n f = n.op_network_file_of_option f
+let network_file_of_option n f = n.op_network_file_of_option f 
 let network_client_of_option n f = n.op_network_client_of_option f
 
   
@@ -170,9 +173,18 @@ let register_escape_char (c : char) (f : unit -> string) =
 let escape_char c = (List.assq c !network_escape_chars) ()
 *)
   
-let network_commands = ref ([] : (string * GuiTypes.arg_kind * string) list)
+let network_commands = ref ([] : (string * string * CommonTypes.arg_kind * string) list)
+
+let commands_by_kind = Hashtbl.create 11
   
 let register_commands list = 
+  List.iter (fun (s, n, f, h) ->
+      try
+        let ss = Hashtbl.find commands_by_kind n in
+        ss := (s,h) :: !ss
+      with _ ->
+          Hashtbl.add commands_by_kind n (ref [s,h])
+  ) list;
   network_commands := list @ !network_commands
   
 let network_connect_servers n = n.op_network_connect_servers ()
@@ -188,14 +200,17 @@ let network_clean_servers r = r.op_network_clean_servers ()
 let network_parse_url n url = n.op_network_parse_url url
 let network_info n = n.op_network_info ()
 
-let new_network name prefix_option subdir_option = 
+let new_network name flags prefix_option subdir_option = 
+  let manager = TcpBufferedSocket.create_connection_manager name in
   let r =
     {
       network_name = name;
       network_num = network_uid ();
       network_prefix = prefix_option;
+      network_flags = flags;
       network_incoming_subdir = subdir_option;
       network_config_file = [];
+      network_connection_manager = manager;
       op_network_connected_servers = (fun _ -> fni name "connected_servers");
       op_network_is_enabled =  (fun _ -> fni name "is_enabled");
       op_network_save_complex_options =  (fun _ -> ni_ok name "save_complex_options");
@@ -205,6 +220,7 @@ let new_network name prefix_option subdir_option =
       op_network_server_of_option =  (fun _ -> fni name "op_network_server_of_option");
       op_network_file_of_option =  (fun _ _ -> fni name "op_network_file_of_option");
       op_network_client_of_option =  (fun _ -> fni name "op_network_client_of_option");
+      op_network_recover_temp = (fun _ -> ni_ok name "recover_temp");
       op_network_search = (fun _ _ -> ni_ok name "search");
       op_network_share = (fun _ _ _ -> ni_ok name "share");
       op_network_private_message = (fun _ _ -> ni_ok name "private message");
@@ -217,6 +233,7 @@ let new_network name prefix_option subdir_option =
       op_network_info = (fun _ -> fni name "network_info");
       op_network_connected = (fun _ -> ni_ok name "connected"; false);
       op_network_add_server = (fun _ -> fni name "op_network_add_server");
+      op_network_gui_message = (fun _ -> ni_ok name "gui_message");
     }
   in
   let rr = (Obj.magic r: network) in
