@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open Int64ops
 open Md4
 open CommonShared
 open Printf2
@@ -32,8 +33,18 @@ open TcpBufferedSocket
 open CommonGlobals
 open CommonOptions
 
-(* We should implement a common uploader too for all networks where
-upload is done linearly. *)
+(* 
+PROBLEMS: most of the time, users won't share their files on all networks.
+We should provide a different directory than incoming/, where files
+would be shared, per directory ?
+
+We should move to a per-network approach. This module would be a functor,
+and each network would be configured to share or not share different
+directories. Moreover, we should have a different sharing strategy.
+
+Default would be: share all files greater than 1 MB in incoming/ on Edonkey.
+  
+*)
   
 (*******************************************************************
 
@@ -43,7 +54,8 @@ upload is done linearly. *)
   
 *******************************************************************)
 
-  
+let ed2k_block_size = Int64.of_int 9728000
+let tiger_block_size = Int64.of_int (1024 * 1024)
   
 type shared_file = {
     shared_fullname : string;
@@ -53,9 +65,12 @@ type shared_file = {
     shared_id : int;
     shared_format : CommonTypes.format;
     shared_impl : shared_file shared_impl;
-    mutable shared_uids : file_uid list;
+    mutable shared_md4s : Md4.t array;
+    mutable shared_tiger : TigerTree.t array;
+    shared_mtime : float;
+    mutable shared_uids : Uid.t list;
     mutable shared_uids_wanted : 
-    (file_uid_id * (shared_file -> file_uid -> unit)) list;
+    (file_uid_id * (shared_file -> Uid.t -> unit)) list;
   }
 
 and shared_tree =
@@ -66,13 +81,27 @@ and shared_tree =
   }
 
 let network = CommonNetwork.new_network "Global Shares"
-    (fun _ -> "")
+    [ VirtualNetwork ]
+  (fun _ -> "")
     (fun _ -> "")
 
 let _ = 
   network.op_network_connected <- (fun _ -> false);
   network.op_network_is_enabled <- (fun _ -> raise IgnoreNetwork);
-  network.op_network_info <- (fun _ -> raise Not_found)
+  network.op_network_info <- (fun _ -> raise Not_found);
+  network.op_network_info <- (fun n ->
+      { 
+        network_netnum = network.network_num;
+        network_config_filename = (match network.network_config_file with
+            [] -> "" | opfile :: _ -> options_file_name opfile);
+        network_netname = network.network_name;
+        network_netflags = network.network_flags;
+        network_enabled = true;
+        network_uploaded = Int64.zero;
+        network_downloaded = Int64.zero;
+        network_connected = 0;
+      });
+  network.op_network_connected_servers <- (fun _ -> [])
   
 let (shared_ops : shared_file CommonShared.shared_ops) = 
   CommonShared.new_shared_ops network
@@ -90,6 +119,130 @@ let (shared_ops : shared_file CommonShared.shared_ops) =
 *******************************************************************)
   
   
+let md4_of_list md4s =
+  let len = List.length md4s in
+  let s = String.create (len * 16) in
+  let rec iter list i =
+    match list with
+      [] -> ()
+    | md4 :: tail ->
+        let md4 = Md4.direct_to_string md4 in
+        String.blit md4 0 s i 16;
+        iter tail (i+16)
+  in
+  iter md4s 0;
+  Md4.string s
+
+let rec tiger_of_array array pos block =
+  if block = 1 then
+    array.(pos)
+  else
+  let len = Array.length array in
+  if pos + block / 2 >= len then
+    tiger_of_array array pos (block/2)
+  else
+  let d1 = tiger_of_array array pos (block/2) in
+  let d2 = tiger_of_array array (pos+block/2) (block/2) in
+  let s = String.create (1 + Tiger.length * 2) in
+  s.[0] <- '\001';
+  String.blit (TigerTree.direct_to_string d1) 0 s 1 Tiger.length;
+  String.blit (TigerTree.direct_to_string d2) 0 s (1+Tiger.length) Tiger.length;
+  let t = Tiger.string s in
+  let t = TigerTree.direct_of_string (Tiger.direct_to_string t) in
+  t
+  
+let rec tiger_max_block_size block len =
+  if block >= len then block
+  else tiger_max_block_size (block*2) len
+  
+let tiger_of_array array =
+  tiger_of_array array 0 (tiger_max_block_size  1 (Array.length array))
+
+let rec tiger_pos nblocks =
+  if nblocks < 2 then 0 else
+  let half = nblocks / 2 in
+  let acc = nblocks - 2 * half in
+  let half = half + acc in
+  half + tiger_pos half
+
+let rec tiger_pos2 nblocks =
+  if nblocks < 2 then 0, [] else
+  let half = nblocks / 2 in
+  let acc = nblocks - 2 * half in
+  let half = half + acc in
+  let pos, list = tiger_pos2 half in
+  let list = (nblocks, pos) :: list in
+  let pos = half + pos in
+  pos, list
+
+let tiger_node d1 d2 = 
+  let s = String.create (1 + Tiger.length * 2) in
+  s.[0] <- '\001';
+  String.blit (TigerTree.direct_to_string d1) 0 s 1 Tiger.length;
+  String.blit (TigerTree.direct_to_string d2) 0 s (1+Tiger.length) Tiger.length;
+  let t = Tiger.string s in
+  let t = TigerTree.direct_of_string (Tiger.direct_to_string t) in
+  t
+  
+let rec tiger_tree s array pos block =
+  if block = 1 then
+    array.(pos)
+  else
+  let len = Array.length array in
+  if pos + block / 2 >= len then
+    tiger_tree s array pos (block/2)
+  else
+  let d1 = tiger_tree s array pos (block/2) in
+  let d2 = tiger_tree s array (pos+block/2) (block/2) in
+  tiger_node d1 d2
+  
+let rec fill_tiger_tree s list =
+  match list with
+    [] -> ()
+  | (nblocks, pos) :: tail ->
+(* nblocks: the number of blocks in the next level
+   pos: the position of the blocks in to be created
+*)
+      let half = nblocks / 2 in
+      let acc = nblocks - 2 * half in
+
+      let next_pos = pos + half + acc in
+      for i = 0 to half - 1 do
+        let d1 = s.(next_pos+2*i) in
+        let d2 = s.(next_pos+2*i+1) in
+        s.(pos+i) <- tiger_node d1 d2;
+      done;
+      if acc = 1 then s.(pos+half) <- s.(next_pos+2*half);
+      fill_tiger_tree s tail
+
+let flatten_tiger_array array = 
+  let len = Array.length array in
+  let s = String.create ( len * TigerTree.length) in  
+  for i = 0 to len - 1 do
+    String.blit (TigerTree.direct_to_string array.(i)) 0
+      s (i * TigerTree.length) TigerTree.length
+  done;
+  s
+
+let unflatten_tiger_array s = 
+  let len = String.length s / TigerTree.length in
+  let array = Array.create len TigerTree.null in  
+  for i = 0 to len - 1 do
+    array.(i) <- TigerTree.direct_of_string 
+      (String.sub s (i * TigerTree.length) TigerTree.length)
+  done;
+  array
+  
+let make_tiger_tree array =
+  let len = Array.length array in
+  let pos, list = tiger_pos2 len in
+  let s = Array.create (pos + len) TigerTree.null in
+  for i = 0 to len - 1 do
+    s.(pos+i) <- array.(i)
+  done;
+  fill_tiger_tree s list;
+  flatten_tiger_array s
+  
 let waiting_shared_files = ref []
 
 (* The shared files indexed by the strings (lowercase), corresponding to
@@ -101,7 +254,7 @@ let current_job = ref None
 let rec start_job_for sh (wanted_id, handler) = 
   try
     List.iter (fun id ->
-        match wanted_id,id with
+        match wanted_id,Uid.to_uid id with
           BITPRINT, Bitprint _ 
         | SHA1, Sha1 _
         | ED2K, Ed2k _
@@ -119,19 +272,113 @@ let rec start_job_for sh (wanted_id, handler) =
                 current_job := None;
               end else
               begin
-                let sha1 = Sha1.direct_of_string job.CommonHasher.job_result
+                let sha1 = job.CommonHasher.job_result
                 in
                 let urn = Printf.sprintf "urn:sha1:%s" (Sha1.to_string sha1)
                 in
                 lprintf "%s has uid %s (%s)\n" sh.shared_fullname urn
-                (Base16.to_string 20 job.CommonHasher.job_result)
+                  (Sha1.to_string job.CommonHasher.job_result)
                 ;
-                let uid = Sha1 (urn,sha1) in
+                let uid = Uid.create (Sha1 sha1) in
                 sh.shared_uids <- uid :: sh.shared_uids;
                 Hashtbl.add shareds_by_uid (String.lowercase urn) sh;
                 start_job_for sh (wanted_id, handler)  
               end
         );
+    
+    | BITPRINT ->
+        let sha1 = ref None in
+        let tiger = ref None in
+        List.iter (fun id ->
+            match Uid.to_uid id with
+            | Sha1 (s) -> sha1 := Some s
+            | TigerTree (s) -> tiger := Some s
+            | _ -> ()
+        ) sh.shared_uids;
+        begin
+          match !sha1, !tiger with
+            Some sha1, Some tiger ->
+              let uid = Uid.create (Bitprint (sha1, tiger)) in
+              let urn = Uid.to_string uid in
+              sh.shared_uids <- uid :: sh.shared_uids;
+              Hashtbl.add shareds_by_uid (String.lowercase urn) sh;
+              start_job_for sh (wanted_id, handler)
+          
+          | _ -> ()
+(*
+(* Not enough information to compute the bitprint. Ask for the corresponding
+information. What happens if there is an error during SHA1 or TIGER
+computation ??? *)
+              ask_for_uid sh BITPRINT handler;
+              (match !sha1 with
+                  None ->  ask_for_uid sh SHA1 (fun _ _ -> ())
+                | _ -> ());
+              (match !tiger with
+                  None ->  ask_for_uid sh TIGER (fun _ _ -> ())
+| _ -> ());
+  *)
+        end
+    
+    | MD5EXT ->
+        let md5ext =  
+          let fd = Unix32.create_rw sh.shared_fullname in
+          let file_size = Unix32.getsize64 fd in
+          let len64 = min (Int64.of_int 307200) file_size in
+          let len = Int64.to_int len64 in
+          let s = String.create len in
+          Unix32.read fd zero s 0 len;
+          Md5Ext.string s
+        in
+        let uid = Uid.create (Md5Ext (md5ext)) in
+        let urn = Uid.to_string uid in
+        sh.shared_uids <- uid :: sh.shared_uids;
+        Hashtbl.add shareds_by_uid (String.lowercase urn) sh;
+        start_job_for sh (wanted_id, handler)  
+    
+    | ED2K ->
+        let size = sh.shared_size in
+        let chunk_size = ed2k_block_size  in
+        let nhashes = Int64.to_int (size // chunk_size) + 1 in
+        let rec iter pos hashes =
+          if pos < size then
+            CommonHasher.compute_md4 sh.shared_fullname
+              pos (min (size -- pos) chunk_size)
+            (fun job ->
+                iter (pos ++ chunk_size) (job.CommonHasher.job_result :: hashes))
+          else
+          let list = List.rev hashes in
+          let ed2k = md4_of_list list in
+          let uid = Uid.create (Ed2k (ed2k)) in
+          let urn = Uid.to_string uid in
+          sh.shared_md4s <- Array.of_list list;
+          sh.shared_uids <- uid :: sh.shared_uids;
+          Hashtbl.add shareds_by_uid (String.lowercase urn) sh;
+          start_job_for sh (wanted_id, handler)                
+        in
+        iter zero []
+    
+    | TIGER -> 
+        let size = sh.shared_size in
+        let chunk_size = tiger_block_size in
+        let nhashes = Int64.to_int (size // chunk_size) + 1 in
+        let rec iter pos hashes =
+          if pos < size then
+            CommonHasher.compute_tiger sh.shared_fullname
+              pos (min (size -- pos) chunk_size)
+            (fun job ->
+                iter (pos ++ chunk_size) (job.CommonHasher.job_result :: hashes))
+          else
+          let array = Array.of_list (List.rev hashes) in
+          let tiger = tiger_of_array array in
+          let uid = Uid.create (TigerTree (tiger)) in
+          let urn = Uid.to_string uid in
+          sh.shared_tiger <- array;
+          sh.shared_uids <- uid :: sh.shared_uids;
+          Hashtbl.add shareds_by_uid (String.lowercase urn) sh;
+          start_job_for sh (wanted_id, handler)                
+        in
+        iter zero []
+        
     | _ -> raise Exit
     
   with Exit -> 
@@ -194,7 +441,7 @@ let filesize_field = "size"
   
 module Indexer = struct
         
-let stem s =
+ let stem s =
   let s = String.lowercase (String.copy s) in
   for i = 0 to String.length s - 1 do
     let c = s.[i] in
@@ -216,7 +463,7 @@ let format_bit = 32 (* "format" *)
 let index_string doc s fields =
   let words = stem s in
   List.iter (fun s ->
-(*      lprintf "ADD [%s] in index" s; lprint_newline (); *)
+(*      lprintf "ADD [%s] in index\n" s;  *)
       DocIndexer.add  index s doc fields
   ) words 
   
@@ -252,10 +499,10 @@ let rec query_to_query t =
           match field with 
           | Field_Size -> s.shared_size <= maxval
           | _ -> true)
-    
-      
+  
+  
   | _ -> failwith "Query not implemented by server"
-      
+
 (*
   
 type query =
@@ -307,7 +554,7 @@ let add file =
 
     if file <> file2 then
       index_file doc file;
-    (*lprintf "Must check files with same md4"; lprint_newline ();*)
+    (*lprintf "Must check files with same md4\n"; *)
     ()
   with _ ->
       let doc = Store.add store file in
@@ -382,6 +629,8 @@ let add_shared full_name codedname size =
   with Not_found ->
       incr shareds_counter;
       
+      let fd = Unix32.create_ro full_name in
+      
       let rec impl = {
           impl_shared_update = 1;
           impl_shared_fullname = full_name;
@@ -399,23 +648,25 @@ let add_shared full_name codedname size =
           shared_codedname = codedname;
           shared_size = size;
           shared_id = !shareds_counter;
-          shared_fd=  Unix32.create full_name [Unix.O_RDONLY] 0o444;
+          shared_fd=  fd;
           shared_format = CommonMultimedia.get_info full_name;
           shared_impl = impl;
           shared_uids_wanted = [];
           shared_uids = [];
+          shared_mtime = Unix32.mtime64 fd;
+          shared_md4s = [||];
+          shared_tiger = [||];
         } in
       
       update_shared_num impl;
       
-      lprintf "FILE ADDED: %s" codedname; lprint_newline ();
+      lprintf "FILE ADDED: %s\n" codedname; 
       Hashtbl.add table !shareds_counter sh;
       Hashtbl.add shared_files codedname sh;
       Indexer.index_file sh;
       add_shared_file shared_tree sh (String2.split codedname '/');
       shared_counter := Int64.add !shared_counter size;
-      lprintf "Total shared : %s" (Int64.to_string !shared_counter);
-      lprint_newline () ;
+      lprintf "Total shared : %s\n" (Int64.to_string !shared_counter);
       sh
       
 let query q = Indexer.find (Indexer.query_to_query q)
@@ -498,9 +749,10 @@ let rec next_uploads () =
 
 let next_uploads () = 
   sent_bytes.(!counter-1) <- sent_bytes.(!counter-1) - !remaining_bandwidth;
+  (*
   if !verbose_upload then begin
       lprintf "Left %d\n" !remaining_bandwidth; 
-    end;
+    end; *)
   complete_bandwidth := !complete_bandwidth + !remaining_bandwidth;
   incr counter;
   if !counter = 11 then begin
@@ -509,9 +761,9 @@ let next_uploads () =
       (if !!max_hard_upload_rate = 0 then 10000 * 1024
         else (maxi (!!max_hard_upload_rate - 1) 1) * 1024 );
       complete_bandwidth := !total_bandwidth;
-      if !verbose_upload then begin
+(*      if !verbose_upload then begin
           lprintf "Init to %d\n" !total_bandwidth; 
-        end;
+        end; *)
       remaining_bandwidth := 0          
     end;
   
@@ -519,25 +771,25 @@ let next_uploads () =
   for i = 0 to 9 do
     last_sec := !last_sec + sent_bytes.(i)
   done;
-  
-  if !verbose_upload then begin
-      lprintf "last sec: %d/%d (left %d)" !last_sec !total_bandwidth
+
+(*  if !verbose_upload then begin
+      lprintf "last sec: %d/%d (left %d)\n" !last_sec !total_bandwidth
         (!total_bandwidth - !last_sec);
-      lprint_newline (); (*
+ (*
       for i = 0 to 9 do
         lprintf "    last[%d] = %d\n" i  sent_bytes.(i)
       done; *)
       
-    end;
+    end; *)
   
   remaining_bandwidth := mini (mini (mini 
         (maxi (!remaining_bandwidth + !total_bandwidth / 10) 10000) 
       !total_bandwidth) !complete_bandwidth) 
   (!total_bandwidth - !last_sec);
   complete_bandwidth := !complete_bandwidth - !remaining_bandwidth;
-  if !verbose_upload then begin
+(*  if !verbose_upload then begin
       lprintf "Remaining %d[%d]\n" !remaining_bandwidth !complete_bandwidth; 
-    end;
+    end; *)
   sent_bytes.(!counter-1) <- !remaining_bandwidth;
   if !remaining_bandwidth > 0 then 
     next_uploads ()
@@ -558,16 +810,15 @@ let ready_for_upload c =
     
 let add_pending_slot c =
   if client_has_a_slot c then begin
-      lprintf "Avoided inserting an uploader in pending slots!";
-      lprint_newline ()
+      lprintf "Avoided inserting an uploader in pending slots!\n";
     end 
   else 
   if not (Intmap.mem (client_num c) !pending_slots_map) then
     begin
 (* This is useless since it is the goal of the pending_slots_map 
         else if Fifo.mem pending_slots_fifo (client_num c) then begin
-	lprintf "Avoided inserting a client twice in pending slots";
-	lprint_newline ()
+	lprintf "Avoided inserting a client twice in pending slots\n";
+	
       end else *)
       pending_slots_map := Intmap.add (client_num c) c !pending_slots_map;
     end
@@ -575,7 +826,7 @@ let add_pending_slot c =
 let remove_pending_slot c =
   if Intmap.mem (client_num c) !pending_slots_map then
     pending_slots_map := Intmap.remove (client_num c) !pending_slots_map
-    
+
 let rec give_a_slot c =
   remove_pending_slot c;
   if not (client_is_connected c) then begin
@@ -589,7 +840,7 @@ let rec give_a_slot c =
 and find_pending_slot () =
   try
     let rec iter () =
-      let c = Intmap.top !pending_slots_map in
+      let _, c = Intmap.top !pending_slots_map in
       give_a_slot c
     in
     iter ()
@@ -638,8 +889,7 @@ let dynamic_refill_upload_slots () =
   let open_slots n =
     let i = ref n in
     if !verbose_upload then begin
-      lprintf "try to allocate %d more slots" n;
-      lprint_newline ()
+      lprintf "try to allocate %d more slots\n" n;
     end;
     while !i > 0 do
       find_pending_slot ();
@@ -651,11 +901,10 @@ let dynamic_refill_upload_slots () =
 (*  let estimated_capacity = !!max_hard_upload_rate * 1024 in *)
   let estimated_capacity = detected_uplink_capacity () in
   if !verbose_upload then begin
-    lprintf "usage: %d(%d) capacity: %d "
+    lprintf "usage: %d(%d) capacity: %d\n"
       (short_delay_upload_usage ()) 
       (upload_usage ()) 
       estimated_capacity;
-    lprint_newline ()
   end;
   let len = Intmap.length !CommonClient.uploaders in
   if len < !!max_upload_slots then begin
@@ -663,16 +912,14 @@ let dynamic_refill_upload_slots () =
 (* enough free bw for another slot *)
     if short_delay_upload_usage () + slot_bw < estimated_capacity then begin
       if !verbose_upload then begin
-	lprintf "uplink not fully used";
-	lprint_newline ()
+	lprintf "uplink not fully used\n";
       end;
       incr not_saturated_count
     end else reset_state ();
           
     if len < min_upload_slots then begin
       if !verbose_upload then begin
-	lprintf "too few upload slots";
-	lprint_newline ()
+	lprintf "too few upload slots\n";
       end;
       open_slots (min_upload_slots - len);
       reset_state ()
@@ -718,7 +965,6 @@ let remaining_bandwidth () = !remaining_bandwidth
 
 let download_credit = ref 0 
 let download_fifo = Fifo.create ()
-
     
 let download_engine () =
   if not (Fifo.empty download_fifo) then begin
@@ -745,12 +991,6 @@ let queue_download_request f len =
   else
     Fifo.put download_fifo (f,len)    
 
-    (*
-  (* timer started every 1/10 seconds *)
-let download_timer () =
-  *)
-
-
   (* timer started every 1/10 seconds *)
 let upload_download_timer () =
   (try download_engine () 
@@ -762,4 +1002,23 @@ let upload_download_timer () =
               
   
   
-  
+open LittleEndian  
+
+let _ =
+  CommonWeb.add_redirector_info "SHARED" (fun buf ->
+      let module S = CommonShared in
+      let total_shared = ref Int64.zero in
+      let total_uploaded = ref Int64.zero in
+      
+      S.shared_iter (fun s ->
+          let i = S.as_shared_impl s in
+          total_uploaded := 
+          Int64.add !total_uploaded i.S.impl_shared_uploaded;
+          total_shared := 
+          Int64.add !total_shared i.S.impl_shared_size
+      );
+      
+      buf_int64 buf !total_shared;
+      buf_int64 buf !total_uploaded;
+      
+  )
\ No newline at end of file
