@@ -17,6 +17,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open BasicSocket
+open Int64ops
 open Printf2
 open Md4
   
@@ -29,6 +31,8 @@ open AnyEndian
 open LittleEndian
 open TcpBufferedSocket
 
+exception FromGuiMessageNotImplemented
+  
 (*
 gui_cut_messages is a reader for TcpBufferedSocket.t that will cut the stream
   in GUI messages, and call f on each message.
@@ -42,7 +46,7 @@ let gui_cut_messages f sock nread =
       if b.len >= 4 + msg_len then
         begin
           let s = String.sub b.buf (b.pos+4) msg_len in
-          buf_used sock (msg_len + 4);
+          buf_used b (msg_len + 4);
           let opcode = get_int16 s 0 in
           (f opcode s : unit)
         end
@@ -59,8 +63,12 @@ let gui_cut_messages f sock nread =
 
 let get_string s pos = 
   let len = get_int16 s pos in
-  String.sub s (pos+2) len, pos+2+len
-
+  if len land 0xffff = 0xffff then
+    let len = get_int s (pos+2) in
+    String.sub s (pos+6) len, pos+6+len
+  else
+    String.sub s (pos+2) len, pos+2+len
+    
 let get_list f s pos =
   let len = get_int16 s pos in
   let rec iter n pos =
@@ -75,10 +83,10 @@ let get_array f s pos =
   let list, pos = get_list f s pos in
   Array.of_list list, pos
 
-let get_bool s pos = (get_int8 s pos) = 1
+let get_bool s pos = (get_uint8 s pos) = 1
 
 let rec get_query s pos =
-  let op = get_int8 s pos in
+  let op = get_uint8 s pos in
   match op with
   | 0 ->
       let list, pos = get_list get_query s (pos+1) in
@@ -138,13 +146,13 @@ let rec get_query s pos =
   | _ -> assert false
 
 let get_search_type s pos =
-  match get_int8 s pos with
+  match get_uint8 s pos with
     0 -> LocalSearch
   | 1 -> RemoteSearch
   | 2 -> SubscribeSearch
   | _ -> assert false
 
-let get_search proto s pos =
+let get_search get_query proto s pos =
   let num = get_int s pos in
   let q, pos = get_query s (pos+4) in
   let max = get_int s pos in
@@ -198,7 +206,7 @@ let dummy_info =
   }
   
 let get_format s pos =
-  match get_int8 s pos with
+  match get_uint8 s pos with
   | 0 -> FormatUnknown, pos+1
   | 1 ->
       let s1, pos = get_string s (pos+1) in
@@ -223,14 +231,22 @@ let get_format s pos =
       MP3 (t, dummy_info), pos
   | _ -> assert false    
 
+let get_uint64_2 proto s pos = 
+  let i, pos =
+    if proto > 24 then
+      get_int64 s pos, pos + 8
+    else
+      get_uint64_32 s pos, pos + 4
+  in i, pos
+
 let get_tag s pos =
   let name, pos = get_string s pos in
   let value, pos =
-    match get_int8 s pos with
+    match get_uint8 s pos with
       0 -> 
-        Uint64 (get_int64_32 s (pos+1)), pos+5
+        Uint64 (get_uint64_32 s (pos+1)), pos+5
     | 1 -> 
-        Fint64 (get_int64_32 s (pos+1)), pos+5
+        Fint64 (get_uint64_32 s (pos+1)), pos+5
     | 2 -> let s, pos = get_string s (pos+1) in
         String s, pos
     | 3 -> Addr (get_ip s (pos+1)), pos+5
@@ -238,13 +254,13 @@ let get_tag s pos =
   in
   { tag_name = name; tag_value = value }, pos
 
-let get_result s pos =
+let get_result proto s pos =
   let num  = get_int s pos in
   let net = get_int s (pos+4) in
   let names, pos = get_list get_string s (pos+8) in
   let md4 = get_md4 s pos in
-  let size = get_int64_32 s (pos+16) in
-  let format, pos = get_string s (pos+20) in
+  let size, pos = get_uint64_2 proto s (pos+16) in
+  let format, pos = get_string s pos in
   let t, pos = get_string s pos in
   let tags, pos = get_list get_tag s pos in
   let comment, pos = get_string s pos in
@@ -263,7 +279,7 @@ let get_result s pos =
   }, pos+1
 
 let get_message s pos =
-  match get_int8 s pos with
+  match get_uint8 s pos with
     0 ->
       let s, pos = get_string s (pos+1) in
       ServerMessage s, pos
@@ -278,7 +294,7 @@ let get_message s pos =
   | _ -> assert false
 
 let get_file_state s pos =
-  match get_int8 s pos with
+  match get_uint8 s pos with
   | 0 -> FileDownloading, pos+1
   | 1 -> FilePaused, pos+1
   | 2 -> FileDownloaded, pos+1
@@ -293,25 +309,54 @@ let get_float s pos =
   let s, pos = get_string s pos in
   float_of_string s, pos
 
-let get_int_float s pos = 
-  let s, pos = get_string s pos in
-  BasicSocket.normalize_time (int_of_float (float_of_string s)), pos
+let get_float_date proto s pos = 
+  let date, pos =
+    if proto > 23 then
+      let date = get_int s pos in
+      last_time () - date, pos + 4
+    else
+    let s, pos = get_string s pos in
+    BasicSocket.normalize_time (int_of_float (float_of_string s)), pos
+  in
+(*  lprintf "get_float_date(%d): %d\n" proto date; *)
+  date, pos
+  
+let get_int_date proto s pos = 
+  if proto > 23 then
+    let date = get_int s pos in
+    last_time () - date, pos + 4
+  else
+  get_int s pos, pos + 4
 
+
+let get_int_pos s pos =
+  get_int s pos, pos + 4
+  
 let get_file proto s pos = 
   let num = get_int s pos in
   let net = get_int s (pos+4) in
-  let names, pos = get_list get_string s (pos+8) in
+  let names, pos = 
+    get_list get_string s (pos+8) in
   let md4 = get_md4 s pos in
-  let size = get_int64_32 s (pos+16) in
-  let downloaded = get_int64_32 s (pos+20) in
-  let nlocations = get_int s (pos+24) in
-  let nclients = get_int s (pos+28) in
-  let state, pos = get_file_state s (pos+32) in
+  let size,pos = get_uint64_2 proto s (pos+16) in
+  let downloaded, pos = get_uint64_2 proto s pos in
+  let nlocations = get_int s pos in
+  let nclients = get_int s (pos+4) in
+  let state, pos = get_file_state s (pos+8) in
   let chunks, pos = get_string s pos in
-  let availability, pos = get_string s pos in
+  let availability, pos = 
+    if proto > 17 then
+      get_list (fun s pos ->
+          let net = get_int s pos in
+          let avail, pos = get_string s (pos+4) in
+          (net, avail), pos
+      ) s pos
+    else
+    let avail, pos = get_string s pos in
+    [net, avail], pos in
   let rate, pos = get_float s pos in
-  let chunks_age, pos = get_array get_int_float s pos in
-  let age, pos = get_int_float s pos in
+  let chunks_age, pos = get_array (get_float_date proto) s pos in
+  let age, pos = get_float_date proto s pos in
   let format, pos = get_format s pos in
   let name, pos = if proto >= 8 then
       get_string s pos else List.hd names, pos in
@@ -319,26 +364,36 @@ let get_file proto s pos =
       get_int s pos, pos+4 else BasicSocket.last_time (), pos in
   let priority, pos = if proto >= 12 then
       get_int s pos, pos+4 else 0, pos in
+  let comment, pos = if proto > 21 then
+      get_string s pos
+    else "", pos in
+  let names = List.map (fun name -> name, noips()) names in
+  let last_seen = BasicSocket.last_time () - last_seen in
+  
   (*
-  assert (num = file_info.file_num);
-  assert (net = file_info.file_network);
-  assert (names = file_info.file_names);
-  assert (md4 = file_info.file_md4);
-  assert (size = file_info.file_size);
-  assert (downloaded = file_info.file_downloaded);
-  assert (nlocations = file_info.file_nlocations);
-  assert (nclients = file_info.file_nclients);
-  assert (state = file_info.file_state);
-  assert (chunks = file_info.file_chunks);
-  assert (availability = file_info.file_availability);
-  assert (rate = file_info.file_download_rate);
-  assert (chunks_age = file_info.file_chunks_age);
-  assert (age = file_info.file_age);
-assert (last_seen = file_info.file_last_seen);
-  assert (name = file_info.file_name);
-assert (priority = file_info.file_priority);
-  *)
+  assert (num = file_info_test.file_num);
+  assert (net = file_info_test.file_network);
+  assert (names = file_info_test.file_names);
+  assert (md4 = file_info_test.file_md4);
+  assert (size = file_info_test.file_size);
+  assert (downloaded = file_info_test.file_downloaded);
+  assert (nlocations = file_info_test.file_nlocations);
+  assert (nclients = file_info_test.file_nclients);
+  assert (state = file_info_test.file_state);
+  assert (chunks = file_info_test.file_chunks);
+  assert (availability = file_info_test.file_availability);
+  assert (rate = file_info_test.file_download_rate);
+  assert (chunks_age = file_info_test.file_chunks_age);
+  assert (age = file_info_test.file_age);
+  assert (last_seen = file_info_test.file_last_seen);
+  assert (name = file_info_test.file_name);
+  assert (priority = file_info_test.file_priority);
+*)
+  
   {
+    file_fields = Fields_file_info.all;
+    
+    file_comment = comment;
     file_num = num;
     file_network = net;
     file_names = names;
@@ -356,30 +411,35 @@ assert (priority = file_info.file_priority);
     file_format = format;
     file_sources = None;
     file_name = name;
-    file_last_seen = BasicSocket.last_time () - last_seen;
+    file_last_seen = last_seen;
     file_priority = priority;
+    file_uids = [];
   }, pos
 
 let get_host_state proto s pos =
   if proto < 12 then
-  (match get_int8 s pos with
+  (match get_uint8 s pos with
   | 0 -> NotConnected (BasicSocket.Closed_by_user, -1)
   | 1 -> Connecting
   | 2 -> Connected_initiating
-  | 3 -> Connected_downloading
+  | 3 -> Connected_downloading (-1)
   | 4 -> Connected (-1)
   | 5 -> Connected 0
   | 6 -> NewHost
   | 7 -> RemovedHost
   | 8 -> BlackListedHost
   | 9 -> NotConnected (BasicSocket.Closed_by_user,0)
+  | 10 -> Connected (-2) 
   | _ -> assert false), pos+1
   else
-  match get_int8 s pos with
+  match get_uint8 s pos with
   | 0 -> NotConnected (BasicSocket.Closed_by_user,-1), pos+1
   | 1 -> Connecting, pos+1
   | 2 -> Connected_initiating, pos+1
-  | 3 -> Connected_downloading, pos+1
+  | 3 -> 
+      if proto < 21 
+      then Connected_downloading (-1), pos+1
+      else  Connected_downloading(get_int s (pos+1)), pos+5
   | 4 -> Connected (-1), pos+1
   | 5 -> Connected (get_int s (pos+1)), pos+5
   | 6 -> NewHost, pos+1
@@ -387,11 +447,12 @@ let get_host_state proto s pos =
   | 8 -> BlackListedHost, pos+1
   | 9 -> NotConnected (BasicSocket.Closed_by_user,
      get_int s (pos+1)), pos+5
+  | 10 -> Connected (-2), pos+1
   | _ -> assert false
 
 
 let get_addr s pos =
-  match get_int8 s pos with
+  match get_uint8 s pos with
     0 ->
       let ip = get_ip s (pos+1) in
       Ip.addr_of_ip ip, pos+5
@@ -428,19 +489,19 @@ let get_server proto s pos =
     server_state = state;
     server_name = name;
     server_description = description;
-	server_banner = "";
+  server_banner = "";
     server_users = None;
   }, pos
 
 let get_client_type s pos = 
-  match get_int8 s pos with
-    0 -> NormalClient
-  | 1 -> FriendClient
-  | 2 -> ContactClient
+  match get_uint8 s pos with
+    0 -> 0
+  | 1 -> client_friend_tag
+  | 2 -> client_contact_tag
   | _ -> assert false
 
 let get_kind s pos =
-  match get_int8 s pos with
+  match get_uint8 s pos with
     0 ->
       let ip = get_ip s (pos+1) in
       let port = get_int16 s (pos+5) in
@@ -452,6 +513,36 @@ let get_kind s pos =
   | _ -> assert false
 
 let get_client proto s pos =
+  if proto <= 18 then
+    let num = get_int s pos in
+    let net = get_int s (pos+4) in
+    let kind, pos = get_kind s (pos+8) in
+    let state,pos = get_host_state proto s pos in
+    let t = get_client_type s pos in
+    let tags, pos = get_list get_tag s (pos+1) in
+    let name, pos = get_string s pos in
+    let rating = get_int s pos in
+    let chat_port = get_int s (pos+4) in
+    {
+      client_num = num;
+      client_network = net;
+      client_kind = kind;
+      client_state = state;
+      client_type = t;
+      client_tags = tags;
+      client_name = name;
+      client_rating = rating;
+      client_chat_port = chat_port;
+      client_files = None;
+      client_connect_time = 0;
+      client_software = "";
+      client_emulemod = "";
+      client_downloaded = zero;
+      client_uploaded = zero;
+      client_upload = None;
+(*      client_sock_addr = ""; *)
+    }, pos+8
+  else
   let num = get_int s pos in
   let net = get_int s (pos+4) in
   let kind, pos = get_kind s (pos+8) in
@@ -460,7 +551,25 @@ let get_client proto s pos =
   let tags, pos = get_list get_tag s (pos+1) in
   let name, pos = get_string s pos in
   let rating = get_int s pos in
-  let chat_port = get_int s (pos+4) in
+  let software, pos = get_string s (pos+4) in
+  let downloaded = get_int64 s pos in
+  let uploaded = get_int64 s (pos+8) in
+(*  let sock_addr, pos = get_string s (pos+16) in *)
+  let pos = pos + 16 in
+  let upload, pos = match get_string s pos with
+      "", pos -> None, pos
+    | s, pos -> Some s, pos
+  in
+  let connect_time, pos = 
+    if proto >= 20 then
+      get_int_date proto s pos
+    else 0, pos
+  in
+  let emulemod, pos =
+    if proto >= 21 then
+      get_string s pos
+      else "", pos
+  in 
   {
     client_num = num;
     client_network = net;
@@ -470,30 +579,63 @@ let get_client proto s pos =
     client_tags = tags;
     client_name = name;
     client_rating = rating;
-    client_chat_port = chat_port;
+    client_chat_port = 0;
     client_files = None;
-    client_connect_time = 0;
-    client_software = "";
-    client_uploaded = zero;
-    client_downloaded = zero;
-    client_upload = None;
-	client_sock_addr = "";
+    client_connect_time = connect_time;
+    client_software = software;
+    client_emulemod = emulemod;
+    client_downloaded = downloaded;
+    client_uploaded = uploaded;
+    client_upload = upload;
+(*    client_sock_addr = sock_addr; *)
   }, pos
+  
 
-let get_network s pos =
+let default_flags = [
+    NetworkHasServers ;
+    NetworkHasRooms;
+    NetworkHasMultinet;
+    NetworkHasSearch;
+    NetworkHasChat;
+    NetworkHasUpload
+  ]
+  
+let get_network proto s pos =
   let num = get_int s pos in
   let name, pos = get_string s (pos+4) in
   let enabled = get_bool s pos in
   let config_file, pos = get_string s (pos+1) in
   let uploaded = get_int64 s pos in
   let downloaded = get_int64 s (pos+8) in
+  let connected, flags, pos = 
+    if proto > 17 then 
+      let connected = get_int s (pos+16) in
+      let flags, pos = 
+        get_list (fun s pos ->
+            (match get_int16 s pos with
+                0 -> NetworkHasServers
+              | 1 -> NetworkHasRooms
+              | 2 -> NetworkHasMultinet
+              | 3 -> VirtualNetwork
+              | 4 -> NetworkHasSearch
+              | 5 -> NetworkHasChat
+              | 6 -> NetworkHasSupernodes
+              | 7 -> NetworkHasUpload
+              | _ -> UnknownNetworkFlag
+            ), pos+2
+        ) s (pos+20)  
+      in
+      connected, flags, pos
+    else 0, default_flags, pos+16 in
   { network_netnum = num;
     network_netname = name;
+    network_netflags = flags;
     network_enabled = enabled;
     network_config_filename = config_file;
     network_uploaded = uploaded;
     network_downloaded = downloaded;
-  }, pos+16
+    network_connected = connected;
+  }, pos
 
 
 let get_user s pos = 
@@ -516,7 +658,7 @@ let get_user s pos =
 
 
 let get_room_state s pos =
-  match get_int8 s pos with
+  match get_uint8 s pos with
     0 -> RoomOpened
   | 1 -> RoomClosed
   | 2 -> RoomPaused
@@ -538,13 +680,13 @@ let get_room proto s pos =
     room_nusers = nusers;
   }, pos + 1 
 
-let get_shared_info s pos =
+let get_shared_info proto s pos =
   let num = get_int s pos in
   let network = get_int s (pos+4) in
   let name, pos = get_string s (pos+8) in
-  let size = get_int64_32 s pos in
-  let uploaded = get_int64 s (pos+4) in
-  let requests = get_int s (pos+12) in
+  let size,pos = get_uint64_2 proto s pos in
+  let uploaded = get_int64 s pos in
+  let requests = get_int s (pos+8) in
   {
     shared_num = num;
     shared_network = network;
@@ -555,14 +697,14 @@ let get_shared_info s pos =
     shared_id = Md4.null;
   }
 
-let get_shared_info_version_10 s pos =
+let get_shared_info_version_10 proto s pos =
   let num = get_int s pos in
   let network = get_int s (pos+4) in
   let name, pos = get_string s (pos+8) in
-  let size = get_int64_32 s pos in
-  let uploaded = get_int64 s (pos+4) in
-  let requests = get_int s (pos+12) in
-  let md4 = get_md4 s (pos+16) in
+  let size,pos = get_uint64_2 proto s pos in
+  let uploaded = get_int64 s pos in
+  let requests = get_int s (pos+8) in
+  let md4 = get_md4 s (pos+12) in
   {
     shared_num = num;
     shared_network = network;
@@ -572,6 +714,8 @@ let get_shared_info_version_10 s pos =
     shared_requests = requests;
     shared_id = md4;
   }
+
+  
   
 (***************
 
@@ -579,8 +723,11 @@ let get_shared_info_version_10 s pos =
 
 ****************)
 
-let from_gui proto opcode s =
+let from_gui (proto : int array) opcode s =
   try
+
+    let proto = if opcode > from_gui_last_opcode then 0 else proto.(opcode) in    
+(*    lprintf "FROM GUI: Opcode %d\n" opcode; *)
     match opcode with
       0 -> GuiProtocol (get_int s 2)
     
@@ -588,10 +735,17 @@ let from_gui proto opcode s =
     | 2 -> CleanOldServers
     | 3 -> KillServer
     | 4 -> ExtendedSearch (-1, ExtendSearchRemotely)
-    | 5 -> let pass,_ = get_string s 2 in Password ("admin", pass)
+    | 5
+    | 52 -> 
+        if proto < 14 then
+           let pass,_ = get_string s 2 in Password ("admin", pass)
+        else
+        let pass,pos = get_string s 2 in
+        let login,pos = get_string s pos in
+        Password (login, pass)
     | 6 -> 
         let local = get_bool s 2 in
-        let search, pos = get_search proto s 3 in
+        let search, pos = get_search get_query proto s 3 in
         search.search_type <- if local then LocalSearch else RemoteSearch;
         Search_query search
     | 7 -> 
@@ -600,7 +754,8 @@ let from_gui proto opcode s =
         Download_query (list, result_num, false)
     
     | 8 -> let string, pos = get_string s 2 in
-        Url string
+        lprintf "Received string: [%s]\n" (String.escaped string);
+        Url string 
     | 9 -> let int = get_int s 2 in RemoveServer_query int
     | 10 ->
         let list, pos = get_list (fun s pos ->
@@ -745,7 +900,7 @@ let from_gui proto opcode s =
     | 41 ->
         let int = get_int s 2 in 
         BrowseUser  int
-    | 42 -> let s, pos = get_search proto s 2 in Search_query s
+    | 42 -> let s, pos = get_search get_query proto s 2 in Search_query s
     | 43 -> 
         let int = get_int s 2 in 
         let message, pos = get_string s 6 in
@@ -755,7 +910,7 @@ let from_gui proto opcode s =
     | 46 -> GetDownloadedFiles
     | 47 -> 
         let list, pos = get_list (fun s pos -> 
-              (get_int s pos, 1 = get_int8 s (pos+4)), pos+5) s 2 in
+              (get_int s pos, 1 = get_uint8 s (pos+4)), pos+5) s 2 in
         GuiExtensions list
     | 48 ->
         SetRoomState (get_int s 2, get_room_state s 6)
@@ -766,15 +921,10 @@ let from_gui proto opcode s =
         let list, pos = get_list get_string s 2 in
         let result_num = get_int s pos in
         let force = get_bool s (pos+4) in
-        Download_query (list, result_num, false)
+        Download_query (list, result_num, force)
 
     | 51 ->
         SetFilePriority(get_int s 2, get_int s 6)      
-
-    | 52 -> 
-        let pass,pos = get_string s 2 in
-        let login,pos = get_string s pos in
-        Password (login, pass)
             
     | 53 ->
         let int = get_int s 2 in 
@@ -787,15 +937,52 @@ let from_gui proto opcode s =
         let port = get_int16 s 10 in
         AddServer_query (net, ip, port) 
         
+    | 55 ->
+        let list, pos = get_list  (fun s pos ->
+              let opcode = get_int16 s pos in
+              let from_guip = get_bool s (pos+2) in
+              let proto = get_int s (pos+3) in
+              (opcode, from_guip, proto), pos+7
+          )  s 2 in
+        MessageVersions list
+
+
+    | 56 ->
+        let num = get_int s 2 in
+        let new_name, pos = get_string s 6 in
+        RenameFile(num, new_name)
+
+    | 57 ->
+        GetUploaders
+
+    | 58 ->
+        GetPending
+
+    | 59 ->
+        GetSearches
+        
+    | 60 ->
+        GetSearch (get_int s 2)
+
+    | 61 ->
+        ConnectClient (get_int s 2)
+        
+    | 62 ->
+        DisconnectClient (get_int s 2)
+        
+    | 63 ->
+        let n = get_int s 2 in
+        let s, pos = get_string s 6 in
+        NetworkMessage (n, s)
+        
     | _ -> 
-        lprintf "FROM GUI:Unknown message %d" opcode; lprint_newline ();
-        raise Not_found
+        lprintf "FROM GUI:Unknown message %d\n" opcode; 
+        raise FromGuiMessageNotImplemented
   
   
   with e ->
-      lprintf "Decoding gui proto[%d]: exception %s, opcode %d" proto
+      lprintf "Decoding gui proto[%d]: exception %s, opcode %d\n" proto.(opcode)
         (Printexc2.to_string e) opcode;
-      lprint_newline ();
       dump s;
       lprint_newline ();
       raise e
@@ -805,17 +992,45 @@ let from_gui proto opcode s =
      Decoding of messages from the Core to the GUI 
 
 ****************)
-let to_gui proto opcode s =
+      
+let dummy_option = 
+  let module M = Options in
+  {
+    M.option_name = "";
+    M.option_shortname = "";
+    M.option_desc = "";
+    M.option_default = "";
+    M.option_value = "";
+    M.option_help = "";
+    M.option_type = "";
+    M.option_advanced = false;
+  }
+      
+let to_gui (proto : int array) opcode s =
   try
+    let proto = if opcode > to_gui_last_opcode then 0 else proto.(opcode) in    
     
+(*    lprintf "TO GUI: Opcode %d\n" opcode;   *)
     match opcode with
-    | 0 -> CoreProtocol (get_int s 2)
+    | 0 ->
+        let version = get_int s 2 in
+        let (max_to_gui, max_from_gui) =
+          if String.length s > 10 then
+            get_int s 6, get_int s 10
+          else
+            57, 62
+        in
+        CoreProtocol (version, max_to_gui, max_from_gui)
     
     | 1 ->
         let list, pos = get_list (fun s pos ->
+              let module M = Options in
               let name, pos = get_string s pos in
               let value, pos = get_string s pos in
-              (name, value), pos
+              { dummy_option with
+                M.option_name = name;
+                M.option_value = value;
+              }, pos
           ) s 2 in
         Options_info list
     
@@ -827,13 +1042,13 @@ let to_gui proto opcode s =
         DefineSearches list
     
     | 4 -> 
-        let r, pos = get_result s 2 in
+        let r, pos = get_result proto s 2 in
         Result_info r
     
     | 5 ->
         let n1 = get_int s 2 in
         let n2 = get_int s 6 in
-        Search_result (n1,n2)
+        Search_result (n1,n2, None)
     
     | 6 -> 
         let n1 = get_int s 2 in
@@ -846,8 +1061,8 @@ let to_gui proto opcode s =
     
     | 8 ->
         let n = get_int s 2 in
-        let size = get_int64_32 s 6 in
-        let rate, pos = get_float s 10 in
+        let size,pos = get_uint64_2 proto s 6 in
+        let rate, pos = get_float s pos in
         File_downloaded (n, size, rate, BasicSocket.last_time ())
     
     | 9 ->
@@ -906,7 +1121,7 @@ let to_gui proto opcode s =
         Console string
     
     | 20 -> 
-        let network_info, pos = get_network s 2 in
+        let network_info, pos = get_network proto s 2 in
         Network_info network_info
     
     | 21 ->
@@ -973,7 +1188,7 @@ let to_gui proto opcode s =
         let user = get_int s 6 in
         Room_remove_user (room, user)
     | 33 ->
-        let s = get_shared_info s 2 in
+        let s = get_shared_info proto s 2 in
         Shared_file_info s
     
     | 34 ->
@@ -990,14 +1205,40 @@ let to_gui proto opcode s =
         let section, pos = get_string s 2 in
         let message, pos = get_string s pos in 
         let option, pos = get_string s pos in
-        let optype = 
-          match get_int8 s pos with
-            0 -> StringEntry 
-          | 1 -> BoolEntry 
-          | 2 -> FileEntry
-          | _ -> assert false in
-        Add_section_option (section, message, option, optype)
-    
+        let module M = Options in
+        let o = if proto > 16 then 
+            let optype, pos = get_string s pos in
+            let help, pos = get_string s pos in
+            let value, pos = get_string s pos in
+            let default, pos = get_string s pos in
+            let advanced = get_bool s pos in
+            {
+              M.option_desc = message;
+              M.option_name = option;
+              M.option_shortname = option;
+              M.option_type = optype;
+              M.option_help = help;
+              M.option_value = value;
+              M.option_default = default;
+              M.option_advanced = advanced;
+            }
+          else
+          let optype = 
+            match get_uint8 s pos with
+              0 -> "String" 
+            | 1 -> "Bool" 
+            | 2 -> "Filename"
+            | _ -> assert false in
+          {
+            dummy_option with
+            M.option_desc = message;
+            M.option_name = option;
+            M.option_shortname = option;
+            
+            M.option_type = optype;
+          } in
+        Add_section_option (section, o)
+        
     | 37 ->
         let upload = get_int64 s 2 in
         let download = get_int64 s 10 in
@@ -1025,13 +1266,37 @@ let to_gui proto opcode s =
         let section, pos = get_string s 2 in
         let message, pos = get_string s pos in 
         let option, pos = get_string s pos in
-        let optype = 
-          match get_int8 s pos with
-            0 -> StringEntry 
-          | 1 -> BoolEntry 
-          | 2 -> FileEntry
-          | _ -> assert false in
-        Add_plugin_option (section, message, option, optype)
+        let module M = Options in
+        let o = if proto > 16 then 
+            let optype, pos = get_string s pos in
+            let help, pos = get_string s pos in
+            let value, pos = get_string s pos in
+            let default, pos = get_string s pos in
+            let advanced = get_bool s pos in
+            {
+              M.option_desc = message;
+              M.option_name = option;
+              M.option_shortname = option;
+              M.option_type = optype;
+              M.option_help = help;
+              M.option_value = value;
+              M.option_default = default;
+              M.option_advanced = advanced;
+            }
+          else
+          let optype = 
+            match get_uint8 s pos with
+              0 -> "String" 
+            | 1 -> "Bool" 
+            | 2 -> "Filename"
+            | _ -> assert false in
+          {
+            dummy_option with
+            M.option_desc = message;
+            M.option_name = option;
+            M.option_type = optype;
+          } in
+        Add_plugin_option (section, o)
     
     | 39 ->
         let upload = get_int64 s 2 in
@@ -1083,8 +1348,8 @@ let to_gui proto opcode s =
     
     | 46 ->
         let n = get_int s 2 in
-        let size = get_int64_32 s 6 in
-        let rate, pos = get_float s 10 in
+        let size,pos = get_uint64_2 proto s 6 in
+        let rate, pos = get_float s pos in
         let last_seen = get_int s pos in
         File_downloaded (n, size, rate, 
           BasicSocket.last_time () - last_seen)
@@ -1092,7 +1357,7 @@ let to_gui proto opcode s =
     | 47 -> BadPassword
     
     | 48 ->
-        let s = get_shared_info_version_10 s 2 in
+        let s = get_shared_info_version_10 proto s 2 in
         Shared_file_info s      
     
     | 49 ->
@@ -1107,7 +1372,12 @@ let to_gui proto opcode s =
         let ndownloading_files = get_int s 46 in
         let ndownloaded_files = get_int s 50 in
         let connected_networks, pos = get_list
-            (fun s pos -> get_int s pos, pos+4)  s 54 in
+            (fun s pos -> 
+              if proto > 17 then 
+                (get_int s pos, get_int s (pos+4)), pos+8
+              else
+                ( (get_int s pos,0),  pos+4 )
+          )  s 54 in
         
         Client_stats {
           upload_counter = upload;
@@ -1144,15 +1414,27 @@ let to_gui proto opcode s =
     | 54 -> 
         let list, pos = get_list (get_file proto) s 2 in
         DownloadedFiles list
+
+    | 55 ->
+        let list, pos = get_list get_int_pos s 2 in
+        Uploaders list
+    | 56 ->
+        let list, pos = get_list get_int_pos s 2 in
+        Pending list
+        
+    | 57 ->
+        let s, pos = get_search get_string proto s 2 in
+        Search s
+        
     | _ -> 
-        lprintf "TO GUI:Unknown message %d" opcode; lprint_newline ();
+        lprintf "TO GUI:Unknown message %d\n" opcode; 
         raise Not_found
 
 
   with e ->
-      lprintf "Decoding gui proto[%d]: exception %s, opcode %d" proto
+      lprintf "Decoding gui proto[%d]: exception %s, opcode %d\n" proto.(opcode)
         (Printexc2.to_string e) opcode;
-      lprint_newline ();
+      
       dump s;
       lprint_newline ();
       raise e
