@@ -17,8 +17,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+exception FromGuiMessageNotImplemented
 
-val to_gui : int -> int -> string -> GuiProto.to_gui
-val from_gui : int -> int -> string -> GuiProto.from_gui
+val to_gui : int array -> int -> string -> GuiProto.to_gui
+val from_gui : int array -> int -> string -> GuiProto.from_gui
   
-val gui_cut_messages : (int -> string -> unit) -> TcpBufferedSocket.t -> 'a -> unit
\ No newline at end of file
+val gui_cut_messages : (int -> string -> unit) -> TcpBufferedSocket.t -> 'a -> unit
