@@ -20,31 +20,13 @@
 open Md4
 open CommonTypes
 
-(*
-type options = {
-    mutable connection_port : int;
-    mutable control_port : int;
-    mutable gui_port : int;
-    
-    mutable save_options_delay : float;
-    mutable check_client_connections_delay : float;
-    mutable check_server_connections_delay : float;
-    mutable small_retry_delay : float;
-    mutable medium_retry_delay : float;
-    mutable long_retry_delay : float;
-    
-    mutable name : string;
-    mutable max_connected_servers : int;
-    mutable upload_limit : int;
-    mutable features : string;
-    
-    mutable server_timeout: float;
-    mutable client_timeout: float;
-    mutable max_server_age : int;
-    mutable password : string;
+type ips_list = {
+    mutable nips : int;
+    mutable ips : Ip.t list;
   }
-    *)
 
+let noips () = { ips = []; nips = 0; }
+  
 type 'a search_request = {
     mutable search_num : int;
     mutable search_type : search_type;
@@ -53,22 +35,105 @@ type 'a search_request = {
     mutable search_network : int;
   }
 
+module Fields_file_info = struct
+
+(* To decrease the size of the "file_info" messages sent to the GUI, send 
+only useful fields. The "op_file_info" function should take this bitmap
+as an argument, and "land" it with the plugin bitmap (to remove fields that
+are irrelevant for the plugin) and put that into the file_info.
+
+In the future, it might also be interesting that the GUI tells the core
+which fields it is interesting in (why send the availability when the
+  GUI cannot display it ?).
+  *)
+    
+    type t = {
+        mutable file_network : bool;
+        mutable file_comment : bool;
+        mutable file_name : bool;
+        mutable file_names : bool;
+        mutable file_md4 : bool;        
+        mutable file_size : bool;
+        mutable file_downloaded : bool;
+        mutable file_nlocations : bool;
+        mutable file_nclients: bool;
+        mutable file_state : bool;
+        mutable file_chunks : bool;
+        mutable file_availability : bool;
+        mutable file_sources : bool;
+        mutable file_download_rate : bool;
+        mutable file_format : bool;
+        mutable file_chunks_age : bool;
+        mutable file_age : bool;
+        mutable file_last_seen : bool;
+        mutable file_priority : bool;
+        mutable file_uids : bool;
+      }
+    
+    let all = {
+        file_network = true;
+        file_comment = true;
+        file_name = true;
+        file_names = true;
+        file_md4 = true;        
+        file_size = true;
+        file_downloaded = true;
+        file_nlocations = true;
+        file_nclients= true;
+        file_state = true;
+        file_chunks = true;
+        file_availability = true;
+        file_sources = true;
+        file_download_rate = true;
+        file_format = true;
+        file_chunks_age = true;
+        file_age = true;
+        file_last_seen = true;
+        file_priority = true;
+        file_uids = true;
+      }
+    
+    type file_info_fields =    
+    | File_network of int
+    | File_comment of string
+    | File_name of string
+    | File_names of (string * ips_list) list
+    | File_md4 of Md4.t        
+    | File_size of int64
+    | File_downloaded of int64
+    | File_nlocations of int
+    | File_nclients of int
+    | File_state of file_state
+    | File_chunks of string
+    | File_availability of (int * string) list
+    | File_download_rate of float
+    | File_format of format
+    | File_chunks_age of int array
+    | File_age of int
+    | File_last_seen of int
+    | File_priority of int
+(*    | File_uids of Uid.t list   *)
+(*    | File_sources of int list option *)
+      
+  end
+
   
 type file_info = {
     file_num : int;    
-    file_network : int;
+    file_fields : Fields_file_info.t;
     
+    file_network : int;
+    mutable file_comment : string;
     mutable file_name : string;
-    mutable file_names : string list;
+    mutable file_names : (string * ips_list) list;
     mutable file_md4 : Md4.t;        
     mutable file_size : int64;
     mutable file_downloaded : int64; (* LOT OF CHANGES *)
     mutable file_nlocations : int; (* MANY CHANGES *)
     mutable file_nclients: int;
-
     mutable file_state : file_state;
     mutable file_chunks : string;
-    mutable file_availability : string; (* MANY CHANGES *)
+    mutable file_availability : (int * string) list; (* MANY CHANGES *)
     mutable file_sources : int list option;
     mutable file_download_rate : float; (* LOT OF CHANGES *)
     mutable file_format : format;
@@ -76,6 +141,7 @@ type file_info = {
     mutable file_age : int;
     mutable file_last_seen : int;
     mutable file_priority : int;
+    mutable file_uids : Uid.t list;
   }
   
 type user_info = {
@@ -123,7 +189,7 @@ type file_tree =
   
 and file_tree_item =
   TreeDirectory of file_tree
-| TreeFile of result_info
+| TreeFile of  result_info
   
 type client_info = {
     client_num : int;
@@ -139,10 +205,11 @@ type client_info = {
     mutable client_chat_port : int;
     mutable client_connect_time : int;
     mutable client_software : string;
+    mutable client_emulemod : string;
     mutable client_downloaded : int64;
     mutable client_uploaded : int64;
     mutable client_upload : string option;
-	mutable client_sock_addr : string;
+(*  mutable client_sock_addr : string;  *)
   }
 
 type client_stats = {
@@ -154,7 +221,7 @@ type client_stats = {
     mutable tcp_download_rate : int; (* bytes/second *)
     mutable udp_upload_rate : int; (* bytes/second *)
     mutable udp_download_rate : int; (* bytes/second *)
-    mutable connected_networks : int list;
+    mutable connected_networks : (int * int) list; (* network/connected servers *)
     mutable ndownloaded_files : int;
     mutable ndownloading_files : int;
   }
@@ -220,40 +287,37 @@ let list_files tree =
   in
   iter [] tree.file_tree_list
   
-type arg_handler =  ui_conn -> string
-type arg_kind = 
-  Arg_none of arg_handler
-| Arg_multiple of (string list -> arg_handler)
-| Arg_one of (string -> arg_handler)
-| Arg_two of (string -> string -> arg_handler)
-| Arg_three of (string -> string -> string -> arg_handler)
-
 type option_widget = 
   StringEntry
 | BoolEntry
 | FileEntry
 
-
+open BasicSocket
+  
 let file_info_test = 
   {
+    file_fields = Fields_file_info.all;
+    
+    file_comment = "";
     file_name = "tratra";
     file_num = 356;
     file_network = 873;
-    file_names = ["toto"; "tutu"];
+    file_names = ["toto", noips(); "tutu", noips()];
     file_md4 = Md4.random ();
     file_size = Int64.of_string "68758765";
-    file_downloaded = Int64.of_string "68758764";
     file_nlocations = 12;
     file_nclients = 18;
     file_state = FileDownloading;
     file_sources = None;
     file_download_rate = 2.2;
     file_chunks = "1010100";
-    file_availability = "";
+    file_downloaded = Int64.of_string "68758764";
+    file_availability = [0,""];
     file_format = FormatUnknown;
-    file_chunks_age = [| 2 |];
-    file_age = 3;
+    file_chunks_age = [| last_time () - 2 |];
+    file_age = last_time () - 3;
     file_last_seen = BasicSocket.last_time ();
     file_priority = 0;
-  } 
+   file_uids = [];
+ } 
   
