@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open Int64ops
 open Printf2
 open CommonResult
 open CommonInteractive
@@ -62,6 +63,11 @@ let rec dollar_escape o with_frames s =
               Buffer.add_string b Terminal.ANSI.ansi_BLUE;
             false
             
+        | 'g' ->
+            if o.conn_output = ANSI then
+              Buffer.add_string b Terminal.ANSI.ansi_GREEN;
+            false
+            
         | 'n' ->
             if o.conn_output = ANSI then 
               Buffer.add_string b Terminal.ANSI.ansi_NORMAL;
@@ -89,12 +95,35 @@ let eval auth cmd o =
   let l = String2.tokens cmd in
   match l with
     [] -> ()
-    | ["longhelp"] | ["??"] ->
-          let module M = CommonMessages in
-          Buffer.add_string  buf !!M.available_commands_are;
-          List.iter (fun (cmd, _, help) ->
-              Printf.bprintf  buf "$r%s$n %s\n" cmd help) 
-          !CommonNetwork.network_commands
+  | ["longhelp"] | ["??"] ->
+      let module M = CommonMessages in
+      Buffer.add_string  buf M.available_commands_are;
+      if use_html_mods o then begin
+          let counter = ref 0 in
+          List.iter (fun (cmd, _, _, help) ->
+              incr counter;
+              let ncmd = ref cmd in
+              let nhelp = ref help in
+              Printf.bprintf buf "\\<tr class=\\\"%s\\\"\\>"(if (!counter mod 2 == 0) then "dl-1" else "dl-2";);
+              html_mods_td buf [ ("", "sr", !ncmd); ("", "srw", Str.global_replace (Str.regexp "\n") "\\<br\\>" !nhelp) ];
+              Printf.bprintf buf "\\</tr\\>\n";
+          ) 
+          (List.sort (fun (c1,_, _,_) (c2,_, _,_) -> compare c1 c2)
+            !CommonNetwork.network_commands);
+          Printf.bprintf buf "\\</table\\>\\</div\\>"
+        end else        
+        begin
+          let list = Hashtbl2.to_list2 commands_by_kind in
+          let list = List.sort (fun (s1,_) (s2,_) -> compare s1 s2) list in
+          List.iter (fun (s,list) ->
+              Printf.bprintf buf "\n   $b%s$n:\n" s;
+              let list = List.sort (fun (s1,_) (s2,_) -> compare s1 s2) !list in
+              List.iter (fun (cmd, help) ->
+                  Printf.bprintf buf "$r%s$n %s\n" cmd help;
+              ) list
+          ) list;
+        end 
+        
     | ["help"] | ["?"] ->
           let module M = CommonMessages in
           Buffer.add_string  buf
@@ -138,7 +167,7 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
             ";
     | "?" :: args | "help" :: args ->
           List.iter (fun arg ->
-              List.iter (fun (cmd, _, help) ->
+              List.iter (fun (cmd, _, _, help) ->
                   if cmd = arg then    
                     Printf.bprintf  buf "%s %s\n" cmd help) 
               !CommonNetwork.network_commands)
@@ -158,17 +187,17 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
             auth := true;
             o.conn_user <- find_ui_user user;
             let module M = CommonMessages in
-            Buffer.add_string buf !!M.full_access
+            Buffer.add_string buf M.full_access
           end else 
         let module M = CommonMessages in
-        Buffer.add_string buf !!M.bad_login
+        Buffer.add_string buf M.bad_login
       else
       if !auth then
         DriverCommands.execute_command 
           !CommonNetwork.network_commands o cmd args      
       else
       let module M = CommonMessages in
-      Buffer.add_string buf !!M.command_not_authorized
+      Buffer.add_string buf M.command_not_authorized
 
               
 (* This function is called every hour to check if we have something to do 
@@ -190,9 +219,8 @@ let check_calendar () =
       if (List.mem tm.Unix.tm_wday days || days = [])  &&
         (List.mem tm.Unix.tm_hour hours || hours = []) then begin
           eval (ref true) command calendar_options;
-          lprintf "Calendar execute: %s\n%s" command
+          lprintf "Calendar execute: %s\n%s\n" command
             (Buffer.contents calendar_options.conn_buf);
-          lprint_newline ();
           Buffer.clear calendar_options.conn_buf;          
         end
   ) !!calendar
@@ -300,7 +328,7 @@ let user_reader o telnet sock nread  =
   let rec iter () =
     if b.len > 0 then
       let c = b.buf.[b.pos] in
-      buf_used sock 1;
+      buf_used b 1;
 (*      lprintf "char %d\n" (int_of_char c); *)
       if c = '\255' && not telnet.telnet_iac then begin
           telnet.telnet_iac <- true;
@@ -416,7 +444,8 @@ let telnet_handler t event =
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
       if Ip.matches from_ip !!allowed_ips then 
-        let sock = TcpBufferedSocket.create_simple 
+        let token = create_token unlimited_connection_manager in
+        let sock = TcpBufferedSocket.create_simple token
           "telnet connection"
           s in
         let telnet = {
@@ -434,7 +463,8 @@ let telnet_handler t event =
             conn_width = 80; 
             conn_height = 0;
           } in
-        TcpBufferedSocket.set_max_write_buffer sock !!interface_buffer;
+        TcpBufferedSocket.prevent_close sock;
+        TcpBufferedSocket.set_max_output_buffer sock !!interface_buffer;
         TcpBufferedSocket.set_reader sock (user_reader o telnet);
         TcpBufferedSocket.set_closer sock user_closed;
         user_socks := sock :: !user_socks;
@@ -558,40 +588,36 @@ let read_theme_page page =
 	close_in file; s
   
 let add_simple_commands buf =
-	let this_page = "commands.html" in
+  let this_page = "commands.html" in
   Buffer.add_string buf (
-		if !!html_mods_theme != "" && theme_page_exists this_page then
-			read_theme_page this_page else
-			if !!html_mods then !!CommonMessages.web_common_header_mods0
-			else !!CommonMessages.web_common_header_old)
+    if !!html_mods_theme != "" && theme_page_exists this_page then
+      read_theme_page this_page else
+    if !!html_mods then !!CommonMessages.web_common_header_mods0
+    else !!CommonMessages.web_common_header_old)
+  
+let http_add_gen_header r =
+  add_reply_header r "Server" "MLdonkey";
+  add_reply_header r "Connection" "close"
 
-let http_add_gen_header buf =
-  Buffer.add_string  buf "HTTP/1.0 200 OK\r\n";
-  Buffer.add_string  buf "Server: MLdonkey\r\n";
-  Buffer.add_string  buf "Connection: close\r\n"
+let http_add_html_header r = 
+  http_add_gen_header r;
+  add_reply_header r "Pragma" "no-cache";
+  add_reply_header r "Content-Type" "text/html; charset=iso-8859-1"
 
-let http_add_html_header buf = 
-  http_add_gen_header buf;
-  Buffer.add_string  buf "Pragma: no-cache\r\n";
-  Buffer.add_string  buf "Content-Type: text/html; charset=iso-8859-1\r\n";
-  Buffer.add_string  buf "\r\n"
-
-let http_add_css_header buf = 
-  http_add_gen_header buf;
-  Buffer.add_string  buf "Content-Type: text/css; charset=iso-8859-1\r\n";
-  Buffer.add_string  buf "\r\n"
-
-let http_add_js_header buf =
-  http_add_gen_header buf;
-  Buffer.add_string  buf "Content-Type: text/javascript; charset=iso-8859-1\r\n";
-  Buffer.add_string  buf "\r\n"
+let http_add_css_header r = 
+  http_add_gen_header r;
+  add_reply_header r "Content-Type" "text/css; charset=iso-8859-1"
 
+let http_add_js_header r =
+  http_add_gen_header r;
+  add_reply_header  r "Content-Type" "text/javascript; charset=iso-8859-1"
+  
 let any_ip = Ip.of_inet_addr Unix.inet_addr_any
   
 let html_open_page buf t r open_body =
   Buffer.clear buf;
   html_page := true;
-  http_add_html_header buf;
+  http_add_html_header r;
   
   if not !!html_mods then 
     (Buffer.add_string buf
@@ -626,40 +652,99 @@ let html_close_page buf =
   Buffer.add_string buf "</body>\n";  
   Buffer.add_string buf "</html>\n";
   ()
+
+let clear_page buf =
+  Buffer.clear buf;
+  html_page := false
   
 let http_handler o t r =
   CommonInteractive.display_vd := false;
-  
+  clear_page buf;
   
   let user = if r.options.login = "" then "admin" else r.options.login in
   if not (valid_password user r.options.passwd) then begin
-      Buffer.clear buf;  
-      need_auth buf !!http_realm
+      clear_page buf;
+      need_auth r !!http_realm
     end
   else
     begin
       let user = find_ui_user user  in
       let o = match user.ui_http_conn with
-        Some oo -> oo.conn_buf <- o.conn_buf; oo
-      | None -> let oo = { o with conn_user = user } in
-           user.ui_http_conn <- Some oo; oo
+          Some oo -> oo.conn_buf <- o.conn_buf; oo
+        | None -> let oo = { o with conn_user = user } in
+            user.ui_http_conn <- Some oo; oo
       in
       try
-        match r.get_url.Url.file with
+        match r.get_url.Url.short_file with
+        | "/wap.wml" ->
+            begin
+              clear_page buf;
+              add_reply_header r "Server" "MLdonkey";
+              add_reply_header r "Connection" "close";
+              add_reply_header r "Content-Type" "text/vnd.wap.wml";
+              let dlkbs = 
+                (( (float_of_int !udp_download_rate) +. (float_of_int !control_download_rate)) /. 1024.0) in
+              let ulkbs =
+                (( (float_of_int !udp_upload_rate) +. (float_of_int !control_upload_rate)) /. 1024.0) in 
+              Printf.bprintf buf "
+<?xml version=\"1.0\"?>
+<!DOCTYPE wml PUBLIC \"-//WAPFORUM//DTD WML 1.1//EN\" \"http://www.wapforum.org/DTD/wml_1.1.xml\">
+
+<wml>
+<card id=\"main\" title=\"MLDonkey Index Page\">  ";
+(* speed *)
+              Printf.bprintf buf "<p align=\"left\">
+   <small>
+    DL %.1f KB/s (%d|%d) UL: %.1f KB/s (%d|%d)
+   </small>
+</p>" dlkbs !udp_download_rate !control_download_rate ulkbs !udp_upload_rate !control_upload_rate;
+
+
+(* functions *)
+              List.iter (fun (arg, value) ->
+                  match arg with
+                    "VDC" -> 
+                      let num = int_of_string value in
+                      let file = file_find num in
+                      file_cancel file
+                  | "VDP" -> 
+                      let num = int_of_string value in
+                      let file = file_find num in
+                      file_pause file
+                  | "VDR" -> 
+                      let num = int_of_string value in
+                      let file = file_find num in
+                      file_resume file
+                  | _ -> ()
+              ) r.get_url.Url.args;
+
+(* downloads *)
+              Printf.bprintf buf "<p align=\"left\"><small>";
+              let mfiles = List2.tail_map file_info !!files in
+              List.iter (fun file ->
+                  Printf.bprintf buf  "<a href=\"wap.wml?%s=%d\">%s</a> <a href=\"wap.wml?VDC=%d\">C</a> [%-5d] %5.1f %s %s/%s <br />" (if downloading file then "VDP" else "VDR" ) (file.file_num) (if downloading file then "P" else "R" ) (file.file_num) (file.file_num) (file.file_download_rate /. 1024.)(short_name file) (print_human_readable file (Int64.sub file.file_size file.file_downloaded)) (print_human_readable file file.file_size);
+              
+              ) mfiles;
+              Printf.bprintf buf "<br />Downloaded %d/%d files " (List.length !!done_files) (List.length !!files);
+              Printf.bprintf buf "</small></p>";
+              
+              
+              Printf.bprintf buf "</card></wml>";
+            end	
         | "/commands.html" ->
             html_open_page buf t r true;
-			let this_page = "commands.html" in 
+            let this_page = "commands.html" in 
             Buffer.add_string buf (
-				if !!html_mods_theme != "" && theme_page_exists this_page then
-					read_theme_page this_page else
-				if !!html_mods then !!CommonMessages.web_common_header_mods0
-              	else !!CommonMessages.web_common_header_old)
+              if !!html_mods_theme != "" && theme_page_exists this_page then
+                read_theme_page this_page else
+              if !!html_mods then !!CommonMessages.web_common_header_mods0
+              else !!CommonMessages.web_common_header_old)
         | "/" | "/index.html" -> 
             if !!use_html_frames then begin
                 html_open_page buf t r false;
-				let this_page = "frames.html" in 
-				if !!html_mods_theme != "" && theme_page_exists this_page then
-					Buffer.add_string buf (read_theme_page this_page) else
+                let this_page = "frames.html" in 
+                if !!html_mods_theme != "" && theme_page_exists this_page then
+                  Buffer.add_string buf (read_theme_page this_page) else
                 if !!html_mods then
                   Printf.bprintf buf "
 			 <frameset src=\"index\" rows=\"%d,25,*\">
@@ -737,7 +822,7 @@ let http_handler o t r =
                     { o with conn_filter = !filter };
                   
                   Buffer.add_string buf (html_escaped 
-                    (Buffer.contents b))
+                      (Buffer.contents b))
               
               | _ -> 
                   Buffer.add_string buf "Bad filter"
@@ -753,7 +838,8 @@ let http_handler o t r =
                       try
                         let num = int_of_string value in 
                         let r = result_find num in
-                        result_download r [] false;
+                        let file = result_download r [] false in
+                        CommonInteractive.start_download file;
                         
                         let module M = CommonMessages in
                         Gettext.buftext buf M.download_started num
@@ -804,16 +890,14 @@ let http_handler o t r =
                       | "Avail" -> o.conn_sortvd <- ByAvail
                       | _ -> ()
                     end
-                | _ -> 
-                    lprintf "files: Unbound argument \"%s=%s\"" arg value;
-                    lprint_newline ();
+                | _ -> ()
             ) r.get_url.Url.args;
             let b = Buffer.create 10000 in
             
             DriverInteractive.display_file_list b o;
             html_open_page buf t r true;
             Buffer.add_string buf (html_escaped (Buffer.contents b))
-
+        
         | "/submit" ->
             begin
               match r.get_url.Url.args with
@@ -830,109 +914,169 @@ let http_handler o t r =
                   ) other_args;
                   let s = 
                     let b = o.conn_buf in
-                    Buffer.clear b;
+                    clear_page b;
                     eval (ref true) cmd o;
                     html_escaped (Buffer.contents b)
                   in
                   html_open_page buf t r true;
 
-				(* Konqueror doesn't like html within <pre> *)
-			  	  let drop_pre = ref false in
-				  let rawcmd = ref cmd in
-
-				  if String.contains cmd ' ' then
-					 rawcmd := String.sub cmd 0 (String.index cmd ' ');
-				  
-				  (match !rawcmd with 
-					| "vm" | "vma" | "view_custom_queries"  | "xs" | "vr" | "afr" | "friend_remove" | "reshare" | "recover_temp"
-					| "c" | "commit" | "bw_stats" | "ovweb" | "friends" | "message_log" | "friend_add" | "remove_old_servers"
-					| "downloaders" | "uploaders" | "scan_temp" | "cs" | "version" | "rename" | "force_download" | "close_fds"
-					| "vd" | "vo" | "voo" | "upstats" | "shares" | "share" | "unshare" -> drop_pre := true;
-					| _ -> ());
-
+(* Konqueror doesn't like html within <pre> *)
+                  let drop_pre = ref false in
+                  let rawcmd = ref cmd in
+                  
+                  if String.contains cmd ' ' then
+                    rawcmd := String.sub cmd 0 (String.index cmd ' ');
+                  
+                  (match !rawcmd with 
+                    | "vm" | "vma" | "view_custom_queries"  | "xs" | "vr"
+                    | "afr" | "friend_remove" | "reshare" | "recover_temp"
+                    | "c" | "commit" | "bw_stats" | "ovweb" | "friends"
+                    | "message_log" | "friend_add" | "remove_old_servers"
+                    | "downloaders" | "uploaders" | "scan_temp" | "cs"
+                    | "version" | "rename" | "force_download" | "close_fds"
+                    | "vd" | "vo" | "voo" | "upstats" | "shares" | "share"
+                    | "unshare" -> drop_pre := true;
+                    | _ -> ());
+                  
                   Printf.bprintf buf "%s\n" 
-					(if use_html_mods o && !drop_pre then s else "\n<pre>\n" ^ s ^ "</pre>");
-
+                    (if use_html_mods o && !drop_pre then s else "\n<pre>\n" ^ s ^ "</pre>");
+              
               | [ ("custom", query) ] ->
                   html_open_page buf t r true;
                   CommonSearch.custom_query buf query
-                  
+              
               | ("custom", query) :: args ->
                   html_open_page buf t r true;
                   send_custom_query o.conn_user buf query  args
-  
+              
               | [ "setoption", _ ; "option", name; "value", value ] ->
                   html_open_page buf t r true;
                   CommonInteractive.set_fully_qualified_options name value;
                   Buffer.add_string buf "Option value changed"
-
+              
               | args -> 
                   List.iter (fun (s,v) ->
                       lprintf "[%s]=[%s]" (String.escaped s) (String.escaped v);
                       lprint_newline ()) args;
                   
                   raise Not_found
-end
+            end
+        
+        | "/preview_download" ->
+            begin
+              clear_page buf;
+              match r.get_url.Url.args with
+                ["q", file_num] ->
+                  let file_num = int_of_string file_num in
+                  let file = file_find file_num in
+                  let fd = file_fd file in
+                  let size = file_size file in
+                  
+                  let (begin_pos, end_pos) = 
+                    try
+                      let (begin_pos, end_pos) = request_range r in
+                      let end_pos = match end_pos with
+                          None -> size
+                        | Some end_pos -> end_pos in
+                      let range_size = end_pos -- begin_pos in
+                      add_reply_header r "Content-Length"
+                        (Int64.to_string range_size);                      
+                      add_reply_header r "Content-Range" 
+                        (Printf.sprintf "bytes %Ld-%Ld/%Ld"
+                          begin_pos (end_pos -- one)
+                        size);
+                      r.reply_head <- "206 Partial Content";
+                      begin_pos, end_pos
+                    with _ ->
+                        add_reply_header r "Content-Length"
+                          (Int64.to_string size);
+                        zero, size
+                  in
+                  
+                  add_reply_header r "Content-type" "application/binary";
+                  add_reply_header r "Accept-Ranges" "bytes";
 
+                  let s = String.create 200000 in
+                  set_max_output_buffer r.sock (String.length s);
+                  set_rtimeout r.sock 10000.;                  
+                  let rec stream_file file pos sock =
+                    let max = (max_refill sock) - 1 in
+                    if max > 0 && !pos < end_pos then
+                      let max64 = min (end_pos -- !pos) (Int64.of_int max)  in
+                      let max = Int64.to_int max64 in
+                      Unix32.read fd !pos s 0 max;
+                      pos := !pos ++ max64;
+                      set_lifetime sock 60.;
+(*                      lprintf "HTTPSEND: refill %d %Ld\n" max !pos;*)
+(*                    lprintf "HTTPSEND: [%s]\n" (String.escaped 
+                        (String.sub s 0 max)); *)
+                      write sock s 0 max;
+                      if output_buffered sock = 0 then begin
+(*                          lprintf "Recursing STREAM\n"; *)
+                          stream_file file pos sock
+                        end
+                  in
+                  r.reply_stream <- Some (stream_file file (ref begin_pos))
+                  
+              | args -> 
+                  List.iter (fun (s,v) ->
+                      lprintf "[%s]=[%s]" (String.escaped s) (String.escaped v);
+                      lprint_newline ()) args;
+                  
+                  raise Not_found                  
+            end
+            
         | "/h.css" ->
-            Buffer.clear buf;
-            html_page := false; 
-            http_add_css_header buf;
-			let this_page = "h.css" in
+            clear_page buf;
+            http_add_css_header r;
+            let this_page = "h.css" in
             Buffer.add_string buf (
-				if !!html_mods_theme != "" && theme_page_exists this_page then
-					read_theme_page this_page else
-						if !!html_mods then !CommonMessages.html_css_mods
-               			else !!CommonMessages.html_css_old)
-
+              if !!html_mods_theme != "" && theme_page_exists this_page then
+                read_theme_page this_page else
+              if !!html_mods then !CommonMessages.html_css_mods
+              else !!CommonMessages.html_css_old)
+        
         | "/dh.css" ->          
-            Buffer.clear buf;
-            html_page := false; 
-            http_add_css_header buf;
-			let this_page = "dh.css" in
+            clear_page buf;
+            http_add_css_header r;
+            let this_page = "dh.css" in
             Buffer.add_string buf (
-				if !!html_mods_theme != "" && theme_page_exists this_page then
-					read_theme_page this_page else
-						if !!html_mods then !CommonMessages.download_html_css_mods
-               			else !!CommonMessages.download_html_css_old)
-
+              if !!html_mods_theme != "" && theme_page_exists this_page then
+                read_theme_page this_page else
+              if !!html_mods then !CommonMessages.download_html_css_mods
+              else !!CommonMessages.download_html_css_old)
+        
         | "/i.js" ->
-            Buffer.clear buf;
-            html_page := false; 
-            http_add_js_header buf;
-			let this_page = "i.js" in
+            clear_page buf;
+            http_add_js_header r;
+            let this_page = "i.js" in
             Buffer.add_string buf (
-				if !!html_mods_theme != "" && theme_page_exists this_page then
-					read_theme_page this_page else
-						if !!html_mods then !!CommonMessages.html_js_mods0
-               			else !!CommonMessages.html_js_old)
-
+              if !!html_mods_theme != "" && theme_page_exists this_page then
+                read_theme_page this_page else
+              if !!html_mods then !!CommonMessages.html_js_mods0
+              else !!CommonMessages.html_js_old)
+        
         | "/di.js" ->          
-            Buffer.clear buf;
-            html_page := false; 
-            http_add_js_header buf;
-			let this_page = "di.js" in
+            clear_page buf;
+            http_add_js_header r;
+            let this_page = "di.js" in
             Buffer.add_string buf (
-				if !!html_mods_theme != "" && theme_page_exists this_page then
-					read_theme_page this_page else
-						if !!html_mods then !!CommonMessages.download_html_js_mods0
-             			else !!CommonMessages.download_html_js_old)
+              if !!html_mods_theme != "" && theme_page_exists this_page then
+                read_theme_page this_page else
+              if !!html_mods then !!CommonMessages.download_html_js_mods0
+              else !!CommonMessages.download_html_js_old)
         | cmd ->
             html_open_page buf t r true;
-            Printf.bprintf buf "No page named %s" cmd
+            Printf.bprintf buf "No page named %s" (html_escaped cmd)
       with e ->
           Printf.bprintf buf "\nException %s\n" (Printexc2.to_string e);
+          r.reply_stream <- None
     end;
   
-  if !html_page then
-  html_close_page buf;
+  if !html_page then  html_close_page buf;
   let s = Buffer.contents buf in
   let s = dollar_escape o !!use_html_frames s in
-  let len = String.length s in
-(*  TcpBufferedSocket.set_monitored t; *)
-  TcpBufferedSocket.set_max_write_buffer t (len + 100);
-  TcpBufferedSocket.write t s 0 len;
-  TcpBufferedSocket.close_after_write t
+  r.reply_content <- s
         
 let http_options = { 
     conn_buf = Buffer.create 10000;
