@@ -81,49 +81,74 @@ let calc_file_eta f =
   end
   )
 
+      
 let file_availability f =
-  let rec loop i p n =
-    if i < 0
-    then 
-      if n < 0.0001
-      then 0.0
-      else (p /. n *. 100.0)
-    else
-      if partial_chunk f.file_chunks.[i]
-      then
-    if f.file_availability.[i] <> (char_of_int 0)
-    then loop (i - 1) (p +. 1.0) (n +. 1.0)
-    else loop (i - 1) p (n +. 1.0)
-      else loop (i - 1) p n
-  in  
-    loop ((String.length f.file_availability) - 1) 0.0 0.0
-
+  match f.file_availability with
+    (_,avail) :: _ ->
+      let rec loop i p n =
+        if i < 0
+        then 
+          if n < 0.0001
+          then 0.0
+          else (p /. n *. 100.0)
+        else
+        if partial_chunk f.file_chunks.[i]
+        then
+          if avail.[i] <> (char_of_int 0)
+          then loop (i - 1) (p +. 1.0) (n +. 1.0)
+          else loop (i - 1) p (n +. 1.0)
+        else loop (i - 1) p n
+      in  
+      loop ((String.length avail) - 1) 0.0 0.0
+  | _ -> 0.0
+      
 let string_availability s =
-  let len = String.length s in
-  let p = ref 0 in
-  for i = 0 to len - 1 do
-    if int_of_char s.[i] <> 0 then begin
-        incr p
-      end
-  done;
-  if len = 0 then 0.0 else 
-    (float_of_int !p /. float_of_int len *. 100.)
-
+  match s with
+    (_,s) :: _ ->
+      
+      let len = String.length s in
+      let p = ref 0 in
+      for i = 0 to len - 1 do
+        if int_of_char s.[i] <> 0 then begin
+            incr p
+          end
+      done;
+      if len = 0 then 0.0 else 
+        (float_of_int !p /. float_of_int len *. 100.)
+  | _ -> 0.0
+      
 let get_file_availability f = 
 if !!html_mods_use_relative_availability
 	then file_availability f 
 	else string_availability f.file_availability
 
-let number_of_sources gf = List.length (file_sources (file_find gf.file_num))
+    
+(* WARNING: these computations are much more expensive as they seem.
+We use the ShortLazy to avoid recomputing the result too many times,
+in particular when sorting the files depending on their number of sources... 
+  *)
+  
+let number_of_sources gf = 
+  List.length (file_all_sources (file_find gf.file_num))
+
+let number_of_sources gf =
+  ShortLazy.compute ("number_of_sources", gf.file_num, 0)
+  number_of_sources gf
     
 let number_of_active_sources gf =
   let nasrcs = ref 0 in
   List.iter (fun fsrc ->
-   if client_state fsrc = Connected_downloading then 
-    incr nasrcs;
-  ) (file_sources (file_find gf.file_num));
+    match (client_state fsrc) with
+        Connected_downloading _ -> incr nasrcs
+      | _ -> ()
+  ) (file_active_sources (file_find gf.file_num));
   !nasrcs
 
+let number_of_active_sources gf =
+  ShortLazy.compute ("number_of_active_sources", gf.file_num, 0)
+  number_of_active_sources gf
+  
+  
 let net_name gf =
 	let n = network_find_by_num gf.file_network in
 	n.network_name
@@ -133,6 +158,7 @@ let short_net_name gf =
 	match nn with 
    | "Open Napster" -> "N"
    | "Direct Connect" -> "C"
+   | "FileTP" -> "T"
    | _ -> String.sub nn 0 1 
    
   
@@ -157,9 +183,8 @@ let save_config () =
         Printf2.lprintf "Exception %s while flushing\n" (Printexc2.to_string e)
   );
   Options.save_with_help downloads_ini;
-  Options.save_with_help downloads_expert_ini;
   CommonComplexOptions.save ();
-  networks_iter (fun r -> 
+  networks_iter_all (fun r -> 
       List.iter (fun opfile ->
           Options.save_with_help opfile          
       ) r.network_config_file);
@@ -177,21 +202,14 @@ let percent file =
   else (downloaded *. 100.) /. size
 
 let short_name file =
-  let name = file.file_name in
-  let len = String.length name in
-  let max_name_len = maxi !!max_name_len 10 in
-  if len > max_name_len then
-    let prefix = String.sub name 0 (max_name_len -7) in
-    let suffix = String.sub name (len-4) 4 in
-    Printf.sprintf "%s...%s" prefix suffix
-  else name
+  shorten file.file_name !!max_name_len
 
 type table_align = 
   Align_Left
 | Align_Right
 | Align_Center
 
-let col_sep = "  "
+let col_sep = " "
 let add buf s align max_len =
   let slen = String.length s in
   let diff = max_len - slen in
@@ -229,9 +247,10 @@ let print_table_text buf alignments titles lines =
   let aligns = Array.create ncols Align_Center in
   Array.iteri (fun i al -> aligns.(i) <- al) alignments; 
   List.iter (fun line ->
+      let len = Array.length line in
       Array.iteri (fun i s ->
           add buf s aligns.(i) cols.(i);
-      Buffer.add_string buf col_sep;
+          if i+1 < len then  Buffer.add_string buf col_sep;
       ) line;
       Buffer.add_char buf '\n';      
   ) lines
@@ -354,7 +373,7 @@ function cancelAll(x){for(i=0;i\\<document.selectForm.elements.length;i++){var j
   |] 
     (List.map (fun file ->
         [|
-          (Printf.sprintf "[\\<a href=\\\"submit\\?q\\=vd+%d\\\"\\>%-5d\\</a\\> \\<a href=http://edonkeyfakes.ath.cx/fakecheck/update/fakecheck.php\\?size\\=%s\\&md4=%s\\>%s\\</a\\>]" 
+          (Printf.sprintf "[\\<a href=\\\"submit\\?q\\=vd+%d\\\"\\>%-5d\\</a\\> \\<a href=http://donkeyfakes.gambri.net/fakecheck/update/fakecheck.php\\?size\\=%s\\&md4=%s\\>%s\\</a\\>]" 
               file.file_num
               file.file_num
               (Int64.to_string file.file_size)
@@ -461,14 +480,17 @@ function submitPriority(num,cp,sel) {
 	if (np.charAt(0) == \\\"=\\\") {var p = parseInt(np.substring(1,99));} 
 	else {var p = parseInt(cp) + parseInt(selectID.value);}
 	var str='\\<select id=\\\"selectPriority' + num + '\\\" name=\\\"selectPriority' + num + '\\\" style=\\\"font-size: 8px; font-family: verdana\\\" onchange=\\\"javascript:submitPriority(' + num + ',' + p + ',this)\\\"\\>';
-	if (p != 10 \\&\\& p != 0 \\&\\& p != -10) { str += '\\<OPTION value=\\\"=' + p + '\\\" SELECTED\\>' + p; }
+	if (p != 20 \\&\\& p != 10 \\&\\& p != 0 \\&\\& p != -10 \\&\\& p != -20) { str += '\\<OPTION value=\\\"=' + p + '\\\" SELECTED\\>' + p; }
+	str += '\\<option value=\\\"=20\\\"'; if (p==20) {str += \\\" SELECTED\\\"}; str += '\\>Very High';
 	str += '\\<option value=\\\"=10\\\"'; if (p==10) {str += \\\" SELECTED\\\"}; str += '\\>High';
 	str += '\\<option value=\\\"=0\\\"'; if (p==0) {str += \\\" SELECTED\\\"}; str += '\\>Normal';
 	str += '\\<option value=\\\"=-10\\\"'; if (p==-10) {str += \\\" SELECTED\\\"}; str += '\\>Low';
+	str += '\\<option value=\\\"=-20\\\"'; if (p==-20) {str += \\\" SELECTED\\\"}; str += '\\>Very Low';
+	str += '\\<option value=\\\"10\\\"\\>+10';
 	str += '\\<option value=\\\"5\\\"\\>+5';
 	str += '\\<option value=\\\"1\\\"\\>+1';
 	str += '\\<option value=\\\"-1\\\"\\>-1';
-	str += '\\<option value=\\\"-5\\\"\\>-5';
+	str += '\\<option value=\\\"-10\\\"\\>-10';
 	str += \\\"\\</select\\>\\\";
 	divID.innerHTML = str;
 	parent.fstatus.location.href='submit?q=priority' + params;
@@ -495,7 +517,9 @@ function submitPriority(num,cp,sel) {
 
 \\<table width=\\\"100%%\\\" class=\\\"downloaders\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>
 
-\\<td title=\\\"Pause/Resume/Cancel\\\" class=\\\"dlheader\\\"\\>P/R/C\\</td\\>"
+\\<td title=\\\"Pause\\\"  class=\\\"dlheader np\\\"\\>P\\</td\\>
+\\<td title=\\\"Resume\\\" class=\\\"dlheader np\\\"\\>R\\</td\\>
+\\<td title=\\\"Cancel\\\" class=\\\"dlheader brs\\\"\\>C\\</td\\>"
 (if !qnum > 0 then begin
 	Printf.sprintf "title=\\\"Active(%d): %s/%s | Queued(%d): %s/%s\\\""
 	(List.length guifiles - !qnum) (size_of_int64 (Int64.sub !tdl !qdl)) (size_of_int64 (Int64.sub !tsize !qsize))
@@ -546,16 +570,17 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
           (if downloading file then
               Printf.sprintf "
 				onMouseOver=\\\"mOvr(this);return true;\\\" onMouseOut=\\\"mOut(this);\\\"\\>
-                \\<td class=\\\"dl al\\\"\\>\\<input class=checkbox name=pause type=checkbox value=%d\\> R
-                \\<input class=checkbox name=cancel type=checkbox value=%d\\>\\</td\\>"
+                \\<td class=\\\"dl al np\\\"\\>\\<input class=checkbox name=pause type=checkbox value=%d\\>\\</td\\>
+                \\<td class=\\\"dl al np\\\"\\>R\\</td\\>
+                \\<td class=\\\"dl al brs\\\"\\>\\<input class=checkbox name=cancel type=checkbox value=%d\\>\\</td\\>"
                 file.file_num
                 file.file_num
             else 
               Printf.sprintf "
 				onMouseOver=\\\"mOvr(this);return true;\\\" onMouseOut=\\\"mOut(this);\\\"\\>
-                \\<td class=\\\"dl al\\\"\\>P
-                \\<input class=checkbox name=resume type=checkbox value=%d\\>
-                \\<input class=checkbox name=cancel type=checkbox value=%d\\>\\</td\\>"
+                \\<td class=\\\"dl al np\\\"\\>P\\</td\\>
+                \\<td class=\\\"dl al np\\\"\\>\\<input class=checkbox name=resume type=checkbox value=%d\\>\\</td\\>
+                \\<td class=\\\"dl al brs\\\"\\>\\<input class=checkbox name=cancel type=checkbox value=%d\\>\\</td\\>"
                 file.file_num
                 file.file_num);
 
@@ -623,15 +648,19 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
 	      (Printf.sprintf "\\<td class=\\\"dl ar\\\"\\>\\<div id=\\\"divSelectPriority%d\\\"\\>\\<select id=\\\"selectPriority%d\\\" name=\\\"selectPriority%d\\\" 
 			style=\\\"font-size: 8px; font-family: verdana\\\" onchange=\\\"javascript:submitPriority(%d,%d,this)\\\"\\>\n" 
 			file.file_num file.file_num file.file_num file.file_num file.file_priority)
-			^ (match file.file_priority with 0 | -10 | 10 -> "" | _ ->
+			^ (match file.file_priority with 0 | -10 | 10 | -20 | 20 -> "" | _ ->
 			  Printf.sprintf "\\<option value=\\\"=%d\\\" SELECTED\\>%d\n" file.file_priority file.file_priority)
+			^ "\\<option value=\\\"=20\\\""  ^ (if file.file_priority = 20 then " SELECTED" else "") ^ "\\>Very high\n"
 			^ "\\<option value=\\\"=10\\\""  ^ (if file.file_priority = 10 then " SELECTED" else "") ^ "\\>High\n"
 			^ "\\<option value=\\\"=0\\\""  ^ (if file.file_priority = 0 then " SELECTED" else "") ^ "\\>Normal\n"
 			^ "\\<option value=\\\"=-10\\\""  ^ (if file.file_priority = -10 then " SELECTED" else "") ^ "\\>Low\n"
+			^ "\\<option value=\\\"=-20\\\""  ^ (if file.file_priority = -20 then " SELECTED" else "") ^ "\\>Very Low\n"
+			^ "\\<option value=\\\"10\\\"\\>+10\n"
 			^ "\\<option value=\\\"5\\\"\\>+5\n"
 			^ "\\<option value=\\\"1\\\"\\>+1\n"
 			^ "\\<option value=\\\"-1\\\"\\>-1\n"
 			^ "\\<option value=\\\"-5\\\"\\>-5\n"
+			^ "\\<option value=\\\"-10\\\"\\>-10\n"
 			^ "\\</select\\>\\</div\\>"
 	       else "");
 
@@ -687,12 +716,24 @@ let html_mods_done_files buf files =
         |]
     ) files);
   Printf.bprintf buf "\\</form\\>"
-  
+
+let print_human_readable file size =
+ (if Int64.to_float size > 1024. && Int64.to_float size < 1048576. then
+    (Printf.sprintf "%5.1f%s" (Int64.to_float size /. 1024.) ("kb") )
+  else if size > Int64.of_float 1048576. && Int64.to_float size < 1073741824. then
+    (Printf.sprintf "%5.1f%s" (Int64.to_float size /. 1048576.) ("mb") )
+  else if size > Int64.of_float 1073741824. then
+    (Printf.sprintf "%5.1f%s" (Int64.to_float size /. 1073741824.) ("gb") )
+  else if size < Int64.zero then
+    (Printf.sprintf "%d chunks" 
+    ((String.length file.file_chunks)-(String.length (String2.replace (String2.replace file.file_chunks '0' "") '1' ""))))
+  else (Printf.sprintf "%8s%s" (Int64.to_string size) ("b") ) )
+
 let simple_print_file_list finished buf files format =
   let print_table = if format.conn_output = HTML then print_table_html 2
     else print_table_text in
   if not finished then
-    if format.conn_output = HTML && ( !!html_checkbox_file_list  ) then
+    if format.conn_output = HTML && ( !!html_checkbox_vd_file_list  ) then
       begin
         if !!html_mods then print_file_html_mods buf files
         else
@@ -717,31 +758,34 @@ let simple_print_file_list finished buf files format =
           [|
             "[ Num ]"; 
             "File";
-            "Percent"; 
-            "Downloaded";
-            "Size";
+            "    %"; 
+            "    Done";
+            "    Size";
+            "    Left";
             "Old";
+            " Active";
             "Rate";
-	    "Priority";
+            "Prio";
           |]     
       )
       (List.map (fun file ->
             let rate, color =
               match file.file_state with
               | FilePaused -> "Paused", "$r"
-              | FileQueued -> "Queued", "$r"
+              | FileQueued -> "Queued", "$g"
               | FileAborted s -> Printf.sprintf "Aborted %s" s, "$r"
               | _ ->
                   if file.file_download_rate < 10.24 then
-                    "-", ""
+                    "-", "$n"
                   else
-                    Printf.sprintf "%5.1f" (
+                    Printf.sprintf "%4.1f" (
                       file.file_download_rate /. 1024.), "$b"
             in
             [|
-              (Printf.sprintf "%s[%s %-5d]%s"
-                color
-                (net_name file)
+              (Printf.sprintf "%0s[%0s%4d]%0s"
+                  (if !!term_ansi then (color)
+                  else "")
+                (short_net_name file)
                 file.file_num
                   (if format.conn_output = HTML then  
                     Printf.sprintf "[\\<a href=\\\"submit\\?q\\=cancel\\+%d\\\" $S\\>CANCEL\\</a\\>][\\<a href=\\\"submit\\?q\\=%s\\+%d\\\" $S\\>%s\\</a\\>] " 
@@ -751,9 +795,13 @@ let simple_print_file_list finished buf files format =
                       (if downloading file then "PAUSE" else "RESUME")
                   else ""));
               (short_name file);
-              (Printf.sprintf "%5.1f" (percent file));
-              (Int64.to_string file.file_downloaded);
-              (Int64.to_string file.file_size);
+              (Printf.sprintf "%3.1f" (percent file));
+              (if !!improved_telnet then (print_human_readable file file.file_downloaded)
+                else (Int64.to_string file.file_downloaded) );
+              (if !!improved_telnet then (print_human_readable file file.file_size)
+                else (Int64.to_string file.file_size) );
+              (if !!improved_telnet then (print_human_readable file (Int64.sub file.file_size file.file_downloaded))
+                else (Int64.to_string (Int64.sub file.file_size file.file_downloaded)) );
               (Printf.sprintf "%d:%s" (age_to_day file.file_age)
                 ( 
                   let len = Array.length file.file_chunks_age in
@@ -765,11 +813,12 @@ let simple_print_file_list finished buf files format =
                   done;
                   if !min = 0 then "-" else
                     string_of_int (age_to_day !min)));
-              rate ^ "$n";
-	      string_of_int file.file_priority;
+              (Printf.sprintf "%2d/%-4d" (number_of_active_sources file) (number_of_sources file));
+              rate;
+              (Printf.sprintf "%4d" (file.file_priority);)
             |]
         ) files)
-  else
+      else
   if use_html_mods format then 
     html_mods_done_files buf files
   else
@@ -808,7 +857,13 @@ let display_file_list buf o =
     Printf.bprintf buf "Downloaded %d/%d files\n" (List.length !!done_files) 
     (List.length !!files);
 
-   
+  if o.conn_output <> HTML && !!improved_telnet then
+  begin
+    let list = List2.tail_map file_info !!files in
+    let list = Sort.list (fun f1 f2 -> percent f1 >= percent f2) list in
+    simple_print_file_list false buf list o
+  end
+  else
   let list = List2.tail_map file_info !!files in
   let list = 
     try
@@ -868,11 +923,13 @@ let old_print_search buf o results =
 		( "0", "srh", "File", "File (mouseover)" ) ; 
 		( "1", "srh ar", "Size", "Size" ) ; 
 		( "1", "srh ar", "Availability", "A" ) ; 
+		( "1", "srh ar", "Complete Sources", "C" ) ; 
 		( "0", "srh", "Hash (click for bitzi lookup)", "Hash (bitzi click)" ) ; 
 		( "0", "srh", "Tags", "Tags (mouseover)" ) ] ; 
   
   (try
       List.iter (fun (rs,r,avail) ->
+        if !!display_downloaded_results || not r.result_done  then begin
           incr counter;
           if !counter >= !!max_displayed_results then raise Exit;          
           
@@ -910,7 +967,7 @@ let old_print_search buf o results =
               match r.result_names with
                 [] -> ()
               | name :: names ->
-                  Printf.bprintf buf "%s\n" name;
+                  Printf.bprintf buf "%s\n" (shorten name !!max_name_len);
                   List.iter (fun s -> 
                       if use_html_mods o then Printf.bprintf buf "\\<BR\\>";
                       Printf.bprintf buf "       %s\n" s
@@ -947,12 +1004,37 @@ let old_print_search buf o results =
 				| _ -> ())) r.result_tags;
 			if !hash = "" then hash := Md4.to_string r.result_md4;
 
+			let csource = ref "" in
+			List.iter (fun t ->
+			    (match t.tag_name with 
+				"completesources" -> (
+	                   		 match t.tag_value with
+				      String s -> csource := s
+	 	    		    | Uint64 i -> csource := Int64.to_string i
+        	           	    | Fint64 i -> csource := Int64.to_string i
+	        	       	    | _ -> csource := "")
+				| _ -> ())) r.result_tags;
+
+			let cavail = ref "" in
+			List.iter (fun t ->
+			    (match t.tag_name with 
+				"availability" -> (
+	                   		 match t.tag_value with
+				      String s -> cavail := s
+	 	    		    | Uint64 i -> cavail := Int64.to_string i
+        	           	    | Fint64 i -> cavail := Int64.to_string i
+	        	       	    | _ -> cavail := (string_of_int avail))
+				| _ -> ())) r.result_tags;
+
           if use_html_mods o then 
             Printf.bprintf buf "\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
+			\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
 			\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
 			\\<td class=\\\"sr\\\"\\>\\<a href=\\\"http://bitzi.com/lookup/%s\\\"\\>%s\\</a\\>\\</td\\>"
             (size_of_int64 r.result_size)
-			(string_of_int avail)
+			!cavail
+			!csource
+
 			(if String.contains !hash ':' then
 				String.sub !hash 
 				((String.rindex !hash ':')+1)
@@ -966,7 +1048,7 @@ let old_print_search buf o results =
 			Printf.bprintf buf "\\<td class=\\\"sr\\\"\\>";
           List.iter (fun t ->
 				  (match t.tag_name with 
-					| "availability" | "urn" | "FTH"  -> () 
+					| "completesources" | "availability" | "urn" | "FTH"  -> () 
 					| _ -> 
 					Buffer.add_string buf ("\\<span title=\\\"" ^ 
 					get_tag_value t.tag_value ^ "\\\"\\>(" ^ t.tag_name ^ ") \\</span\\>");
@@ -977,10 +1059,11 @@ let old_print_search buf o results =
 		  else
       		List.iter (fun t ->
               Buffer.add_string buf (Printf.sprintf "%-3s "
-                  (if t.tag_name = "availability" then string_of_int avail else
+                  (if t.tag_name = "availability" then !cavail else
 					get_tag_value t.tag_value))
           ) r.result_tags;
           Buffer.add_char buf '\n';
+	end
       ) results;
       if use_html_mods o then Printf.bprintf buf "\\</table\\>"
     with _ -> ())
@@ -1155,7 +1238,7 @@ let print_results buf o results =
                   (
                     let names = r.result_names in
                     let names = if r.result_done then
-                        names @ ["ALREADY DOWNLOADED"] else names in
+                        ["ALREADY DOWNLOADED "] @ names else names in
                     let names = match  r.result_comment with
                         "" -> names
                       |  comment ->
@@ -1165,13 +1248,13 @@ let print_results buf o results =
                       [name] -> name
                     | _ ->
                         let buf = Buffer.create 100 in
-                        Buffer.add_string buf "\\<table\\>\n";
+                        if o.conn_output = HTML then Buffer.add_string buf "\\<table\\>\n";
                         List.iter (fun s -> 
-                            Buffer.add_string buf "\\<tr\\>";
+                            if o.conn_output = HTML then Buffer.add_string buf "\\<tr\\>";
                             Buffer.add_string buf s;
-                            Buffer.add_string buf "\\</tr\\>";
+                            if o.conn_output = HTML then Buffer.add_string buf "\\</tr\\>";
                         ) names;
-                        Buffer.add_string buf "\\</table\\>\n";
+                        if o.conn_output = HTML then Buffer.add_string buf "\\</table\\>\n";
                         
                         Buffer.contents buf
                   )
@@ -1269,12 +1352,12 @@ let print_search buf s o =
     (if s.search_waiting = 0 then "done" else
       (string_of_int s.search_waiting) ^ " waiting");
   
-  if not !!new_print_search then old_print_search buf o results else
+  if o.conn_output != HTML then print_results buf o results else
     begin
-      if o.conn_output = HTML && !!html_checkbox_file_list then
+      if !!html_checkbox_search_file_list then
         print_search_html buf results o s.search_num
       else
-        print_results buf o results 
+        old_print_search buf o results 
     end  
 
 let browse_friends () =
