@@ -21,7 +21,17 @@
 open Md4
 
 open CommonTypes
-open CommonSwarming
+open CommonDownloads
+open BTRate
+
+type torrent = {
+    mutable torrent_name : string;
+    mutable torrent_length : int64;
+    mutable torrent_announce : string;
+    mutable torrent_piece_size : int64;
+    mutable torrent_files :  (string * int64) list;
+    mutable torrent_pieces : Sha1.t array;
+  }
 
 type client = {
     client_client : client CommonClient.client_impl;
@@ -30,26 +40,35 @@ type client = {
     mutable client_sock : tcp_connection;
     mutable client_host : Ip.t * int;
     mutable client_chunks : (int64 * int64) list;
-    mutable client_blocks : Int64Swarmer.block list;
-    mutable client_ranges : Int64Swarmer.range list;
+    mutable client_uploader : Int64Swarmer.uploader option;
+    mutable client_ranges : (int64 * int64 * Int64Swarmer.range) list;
     mutable client_block : Int64Swarmer.block option;
-    mutable client_choked : bool;
+    
+    mutable client_sent_choke : bool; (* we sent a Choke to the client *)
+    mutable client_choked : bool;      (* we received a Choke from the client *)
     mutable client_interested : bool;
     mutable client_uid : Sha1.t;
     
-    mutable client_bitmap : string;
+    mutable client_bitmap : string option;
     mutable client_new_chunks : int list;
     
     mutable client_upload_requests : (int * int64 * int64) list;
     mutable client_allowed_to_write : int64;
-    
-    mutable client_downloaded_rate : int64;
+    mutable client_upload_rate : Rate.t;
+    mutable client_downloaded_rate :  Rate.t;
     mutable client_downloaded : int64;
     mutable client_uploaded : int64;
     mutable client_optimist_time : int;
     
-    mutable client_blocks_sent : Int64Swarmer.block list;
+    mutable client_blocks_sent : (int * int64 * int64) list;
     mutable client_good : bool;
+    mutable client_num_try : int;
+    mutable client_alrd_sent_interested : bool;
+    mutable client_alrd_sent_notinterested : bool;
+    mutable client_interesting : bool;
+    mutable client_incoming : bool;
+    mutable client_registered_bitfield : bool;
+    mutable client_last_optimist : int;
   }
 
 and file = {
@@ -58,13 +77,15 @@ and file = {
     file_tracker : string;
     file_id : Sha1.t;
     file_name : string;
-    file_swarmer : Int64Swarmer.t;
-    file_partition : CommonSwarming.Int64Swarmer.partition;
-    mutable file_clients : (Sha1.t, client) Hashtbl.t ;
+    mutable file_swarmer : Int64Swarmer.t option;
+    mutable file_clients : ((Ip.t*int), client) Hashtbl.t ;
+    mutable file_clients_num : int ;
     mutable file_chunks : Sha1.t array; 
     mutable file_tracker_connected : bool;
     mutable file_tracker_interval : int;
     mutable file_tracker_last_conn : int;
-    mutable file_files : (string * int64 * int64) list;
-    mutable file_blocks_downloaded : Int64Swarmer.block list;
+    mutable file_files : (string * int64) list;
+    mutable file_blocks_downloaded : (int * int64 * int64) list;
+    (* vvv probably a network specific value vvv ?what about file_downloaded?*)
+    mutable file_uploaded : int64;
   }
