@@ -18,7 +18,7 @@
 *)
 
 open Queues
-open CommonSwarming
+open CommonDownloads
 open Printf2
 open Md4
 open CommonOptions
@@ -37,24 +37,17 @@ open GnutellaGlobals
 open GnutellaOptions
 open GnutellaProtocol
 open GnutellaComplexOptions
-
+open CommonHosts
+  
 module DG = CommonGlobals
 module DO = CommonOptions
   
-module GnutellaHandler = struct
-    let init s sock gconn =
-      (if s.server_gnutella2 then
-          Gnutella2Handler.init 
-        else
-          Gnutella1Handler.init) s sock gconn
-  end
-  
 let gnutella_proto = "GNUTELLA/"
 let gnutella_proto_len = String.length gnutella_proto
 
 let retry_and_fake = false
   
-let rec server_parse_header nservers with_accept retry_fake s gconn sock header
+let rec server_parse_header s gconn sock header
   =
 (*   if !verbose_msg_servers then  LittleEndian.dump_ascii header;   *)
   try
@@ -65,9 +58,9 @@ let rec server_parse_header nservers with_accept retry_fake s gconn sock header
         space_pos - gnutella_proto_len) in
     let code = String.sub header (space_pos+1) 3 in
     let lines = Http_client.split_header header in
-    let headers =        
+    let first_line, headers =        
       match lines with
-        [] -> []
+        [] -> "", []
       | line :: headers ->
           let headers = Http_client.cut_headers headers in
           if !verbose_msg_servers then begin
@@ -77,9 +70,24 @@ let rec server_parse_header nservers with_accept retry_fake s gconn sock header
               ) headers;
               lprintf "\n\n"
             end; 
-          headers          
+          line, headers          
     in
     
+    if !verbose_unknown_messages then begin
+        let unknown_header = ref false in
+        List.iter (fun (header, _) ->
+            unknown_header := !unknown_header || not (List.mem header GnutellaProto.known_supernode_headers)
+        ) headers;
+        if !unknown_header then begin
+            lprintf "G2 DEVEL: Supernode Header contains unknown fields\n";
+            lprintf "    %s\n" first_line;
+            List.iter (fun (header, (value,header2)) ->
+                lprintf "    [%s] = [%s](%s)\n" header value header2;
+            ) headers;
+            lprintf "G2 DEVEL: end of header\n";        
+          end;
+      end;
+    
     let ultra_peer = ref false in
     let gnutella2 = ref false in
     let deflate = ref false in
@@ -213,34 +221,30 @@ let rec server_parse_header nservers with_accept retry_fake s gconn sock header
         with _ -> ()
     ) headers;
     List.iter (fun (ip,port,ultrapeer) ->
-        if !gnutella2 && !!g2_enabled then begin
-            lprintf "gnutella2: adding ultrapeer from %s\n" s.server_agent;
-            ignore (new_host ip port ultrapeer 2)
-          end else
-        if (not !gnutella2) && !!g1_enabled then begin
-            lprintf "gnutella1: adding ultrapeer from %s\n" s.server_agent;
-            ignore (new_host ip port ultrapeer 1)
+        if GnutellaProto.is_same_network !gnutella2 then begin
+            lprintf "gnutella: adding ultrapeer from %s\n" s.server_agent;
+            ignore (H.new_host ip port ultrapeer)
           end
     ) !x_ultrapeers;    
     server_must_update (as_server s.server_server);
-    s.server_gnutella2 <-  !gnutella2;
+(*    s.server_gnutella2 <-  !gnutella2; *)
     
     if not !ultra_peer then 
       failwith "DISCONNECT: Not an Ultrapeer";
     if proto < "0.6" then
       failwith (Printf.sprintf "Bad protocol [%s]" proto)
     else
-    if not (!gnutella2 || !!g1_enabled) then
+    if not (!gnutella2) then
       failwith "Protocol Gnutella2 not supported"
     else
     if code <> "200" then begin
-        s.server_connected <- int32_time ();
-        if retry_fake then begin
-            GnutellaGlobals.disconnect_from_server nservers s
-(Closed_for_error "Bad HTTP code");
+        s.server_connected <- int64_time ();
+(*        if retry_fake then begin
+            GnutellaGlobals.disconnect_from_server s
+              (Closed_for_error "Bad HTTP code");
             connect_server nservers with_accept false s.server_host
-            !keep_headers
-          end else
+              !keep_headers
+          end else *)
           failwith  (Printf.sprintf "Bad return code [%s]" code)
       
       end else
@@ -263,7 +267,10 @@ let rec server_parse_header nservers with_accept retry_fake s gconn sock header
     if !verbose_msg_servers then
       lprintf "CONNECT REQUEST: %s\n" (String.escaped msg); 
     write_string sock msg;
-
+    
+    (* Now, change to deflated connection if needed *)
+    if !deflate then
+      deflate_connection sock;
 (*
     if not retry_fake then
       lprintf "******** CONNECTED BY FAKING ********\n";
@@ -271,55 +278,49 @@ let rec server_parse_header nservers with_accept retry_fake s gconn sock header
     
     set_rtimeout sock DG.half_day;
     set_server_state s (Connected (-1));
-    s.server_connected <- int32_time ();    
-    if s.server_gnutella2 then
-      Gnutella2Handler.init s sock gconn
-    else    
-      Gnutella1Handler.init s sock gconn;
-        
+    s.server_connected <- int64_time ();    
+    GnutellaHandler.init s sock gconn;
+  
   with
   | e -> 
       if !verbose_msg_servers then
         lprintf "DISCONNECT WITH EXCEPTION %s\n" (Printexc2.to_string e); 
-      disconnect_from_server nservers s (Closed_for_exception e)
+      disconnect_from_server s (Closed_for_exception e)
 
-and connect_server nservers with_accept retry_fake h headers =  
+and connect_server h headers =  
   let s = match h.host_server with
       None -> 
-        let s = new_server h.host_ip h.host_port in
+        let s = new_server h.host_addr h.host_port in
         h.host_server <- Some s;
         s
     | Some s -> s
   in
   match s.server_sock with
-    ConnectionWaiting -> ()
-  | ConnectionAborted -> s.server_sock <- ConnectionWaiting
-  | Connection _ -> ()
   | NoConnection -> 
       incr nservers;
-      s.server_sock <- ConnectionWaiting;
-      add_pending_connection (fun _ ->
+      
+      GnutellaProto.server_send_ping s.server_sock s;
+      
+      let token =
+      add_pending_connection connection_manager (fun token ->
           decr nservers;
-          match s.server_sock with
-            ConnectionAborted -> s.server_sock <- NoConnection;
-          | Connection _ | NoConnection -> ()
-          | ConnectionWaiting ->
               try
-                if not (Ip.valid s.server_host.host_ip) then
+                if not (Ip.valid s.server_host.host_addr) then
                   failwith "Invalid IP for server\n";
-                let ip = s.server_host.host_ip in
+                let ip = s.server_host.host_addr in
                 let port = s.server_host.host_port in
 (*        if !verbose_msg_servers then begin
             lprintf "CONNECT TO %s:%d\n" (Ip.to_string ip) port;
 end;  *)
-                h.host_tcp_request <- last_time ();
-                let sock = connect "gnutella to server"
+              H.set_request h Tcp_Connect;
+(*                h.host_tcp_request <- last_time (); *)
+                let sock = connect token "gnutella to server"
                     (Ip.to_inet_addr ip) port
                     (fun sock event -> 
                       match event with
                         BASIC_EVENT (RTIMEOUT|LTIMEOUT) -> 
 (*                  lprintf "RTIMEOUT\n"; *)
-                          disconnect_from_server nservers s Closed_for_timeout
+                          disconnect_from_server s Closed_for_timeout
                       | _ -> ()
                   ) in
                 TcpBufferedSocket.set_read_controler sock download_control;
@@ -329,12 +330,12 @@ end;  *)
                 s.server_sock <- Connection sock;
                 incr nservers;
                 set_gnutella_sock sock !verbose_msg_servers
-                  (HttpHeader (server_parse_header nservers true retry_fake s)
+                  (HttpHeader (server_parse_header s)
                 
                 );
                 set_closer sock (fun _ error -> 
 (*            lprintf "CLOSER %s\n" error; *)
-                    disconnect_from_server nservers s error);
+                    disconnect_from_server s error);
                 set_rtimeout sock !!server_connection_timeout;
                 
                 let s = 
@@ -344,18 +345,17 @@ end;  *)
                   Printf.bprintf buf "Listen-IP: %s:%d\r\n"
                     (Ip.to_string (client_ip (Connection sock))) !!client_port;
                   Printf.bprintf buf "Remote-IP: %s\r\n"
-                    (Ip.to_string s.server_host.host_ip);
+                    (Ip.to_string s.server_host.host_addr);
                   
                   if headers = [] then begin
-                      Printf.bprintf buf "User-Agent: %s\r\n" user_agent;
-                      if with_accept then
-                        Printf.bprintf buf "Accept: %s%s\r\n"
-                          (if !!g1_enabled then "application/x-gnutella-packets,"
-                          else "")
-                        (if !!g2_enabled then "application/x-gnutella2"
-                          else "application/x-edonkey"); 
-(* Contribute:  Packet compression is not yet supported...
-Printf.bprintf buf "Accept-Encoding: deflate\r\n"; *)
+                    Printf.bprintf buf "User-Agent: %s\r\n" user_agent;
+                    if GnutellaNetwork.has_accept then
+                      Printf.bprintf buf "Accept: %s\r\n"
+                        GnutellaNetwork.accept_header;
+
+                    if !!deflate_connections then
+                      Printf.bprintf buf "Accept-Encoding: deflate\r\n";
+                    
 (* Other Gnutella headers *)          
                       Printf.bprintf buf "X-Max-TTL: 4\r\n";
                       Printf.bprintf buf "Vendor-Message: 0.1\r\n";
@@ -398,10 +398,13 @@ Printf.bprintf buf "X-Degree: %d\r\n" !!g1_max_ultrapeers;
                   lprintf "SENDING %s\n" (String.escaped s);
                 write_string sock s;
               with e ->
-                  disconnect_from_server nservers s
-                     (Closed_for_exception e)
-      )
-
+                  disconnect_from_server s
+                    (Closed_for_exception e)
+        )
+      in
+      s.server_sock <- ConnectionWaiting token
+  | _ -> ()
+      
 let get_file_from_source c file =
   if connection_can_try c.client_connection_control then begin
       connection_try c.client_connection_control;
@@ -413,14 +416,9 @@ let get_file_from_source c file =
           connection_failed c.client_connection_control;
 
           let uri = (find_download file c.client_downloads).download_uri in
-          if !!g1_enabled then
-            List.iter (fun s ->
-                Gnutella1Proto.server_send_push s uid uri
-            ) !g1_connected_servers;
-          if !!g2_enabled then
-            List.iter (fun s ->
-                Gnutella2Proto.server_send_push s uid uri
-            ) !g2_connected_servers;
+          List.iter (fun s ->
+              GnutellaProto.server_send_push s uid uri
+              ) !connected_servers;
       | _ ->
           GnutellaClients.connect_client c
     end
@@ -448,8 +446,7 @@ let recover_file file =
       with _ -> ()
   ) file.file_uids;
   
-  if !!g1_enabled then Gnutella1.recover_file file;
-  if !!g2_enabled then Gnutella2.recover_file file;
+  GnutellaScheduler.recover_file file;
   ()
 
 let recover_files () =
@@ -471,7 +468,7 @@ let download_file (r : result) =
       get_file_from_source c file;
   ) r.result_sources;
   recover_file file;
-  ()
+  (as_file file)
 
       (*
 
@@ -527,7 +524,9 @@ Remote-IP: 207.5.238.35
 let disconnect_server s r =
   match s.server_sock with
   | Connection sock -> close sock r
-  | ConnectionWaiting -> s.server_sock <- ConnectionAborted
+  | ConnectionWaiting token -> 
+      cancel_token token;
+      s.server_sock <- NoConnection
   | _ -> ()
     
 let ask_for_files () =
@@ -540,16 +539,13 @@ let ask_for_files () =
 
     
 let udp_handler p =
-  let (ip,port) = match p.UdpSocket.addr with
+  let (ip,port) = match p.UdpSocket.udp_addr with
     | Unix.ADDR_INET(ip, port) -> Ip.of_inet_addr ip, port
     | _ -> raise Not_found
   in
-  let buf = p.UdpSocket.content in
+  let buf = p.UdpSocket.udp_content in
   let len = String.length buf in
-  if !!g2_enabled && len > 3 && String.sub buf 0 3 = "GND" then
-    Gnutella2.udp_handler ip port buf
-  else
-    lprintf "Unexpected UDP packet: \n%s\n" (String.escaped buf)
+  GnutellaHandler.udp_handler ip port buf
       
 let _ =
   server_ops.op_server_disconnect <- (fun s -> disconnect_server s
@@ -557,76 +553,3 @@ Closed_by_user);
   server_ops.op_server_remove <- (fun s ->
       disconnect_server s Closed_by_user; 
   )
-  
-let manage_host h =
-  try
-    let current_time = last_time () in
-(*    lprintf "host queue before %d\n" (List.length h.host_queues); *)
-
-(*    lprintf "host queue after %d\n" (List.length h.host_queues); *)
-(* Don't do anything with hosts older than one hour and not responding *)
-    if max h.host_connected h.host_age > last_time () - 3600 then begin
-        host_queue_add workflow h current_time;
-(* From here, we must dispatch to the different queues *)
-        match h.host_kind with
-          1 ->
-            if h.host_udp_request + 600 < last_time () then begin
-(*              lprintf "g1_waiting_udp_queue\n"; *)
-                host_queue_add g1_waiting_udp_queue h current_time;
-              end;
-            if h.host_tcp_request + 600 < last_time () then begin
-                host_queue_add (if h.host_ultrapeer then begin
-(*                    lprintf "g1_ultrapeers_waiting_queue"; *)
-                      g1_ultrapeers_waiting_queue
-                    end else begin
-(*                    lprintf "g1_peers_waiting_queue"; *)
-                      g1_peers_waiting_queue
-                    end) h current_time;
-              end
-        | 2 ->        
-            if h.host_udp_request + 600 < last_time () then begin
-(*              lprintf "g2_waiting_udp_queue\n"; *)
-                host_queue_add g2_waiting_udp_queue h current_time;
-              end;
-            if h.host_tcp_request + 600 < last_time () then
-              host_queue_add (if h.host_ultrapeer then begin
-(*                  lprintf "g2_ultrapeers_waiting_queue"; *)
-                    g2_ultrapeers_waiting_queue
-                  end else begin
-(*                  lprintf "g2_peers_waiting_queue"; *)
-                    g2_peers_waiting_queue
-                  
-                  end) h current_time;
-        | _ ->
-            if h.host_udp_request + 600 < last_time () then begin
-(*              lprintf "g01_waiting_udp_queue\n"; *)
-                host_queue_add g1_waiting_udp_queue h current_time;
-                host_queue_add g2_waiting_udp_queue h current_time;
-              end;
-            if h.host_tcp_request + 600 < last_time () then
-              host_queue_add (if h.host_ultrapeer then begin
-(*                  lprintf "g0_ultrapeers_waiting_queue"; *)
-                    g0_ultrapeers_waiting_queue
-                  end else begin
-(*                  lprintf "g0_peers_waiting_queue"; *)
-                    g0_peers_waiting_queue
-                  end) h current_time;
-      end    else 
-    if max h.host_connected h.host_age > last_time () - 3 * 3600 then begin
-        host_queue_add workflow h current_time;      
-      end else
-(* This host is too old, remove it *)
-      ()
-          
-  with e ->
-      lprintf "Exception %s in manage_host\n" (Printexc2.to_string e)
-      
-let manage_hosts () = 
-  let rec iter () =
-    let h = host_queue_take workflow in
-    manage_host h;
-    iter ()
-  in
-  try iter () with _ -> 
-(*      lprintf "done (set %d len)\n" (Queue.length workflow); *)
-      ()
