@@ -17,6 +17,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open CommonInteractive
+open Int64ops
 open Printf2
 open Md4
 open CommonGlobals
@@ -45,11 +47,18 @@ let download r filenames =
       List.iter (fun (user, filename) ->
           ignore (add_file_client file user filename)
       ) r.result_sources;
-      SlskServers.ask_for_file file
-    end
+      SlskServers.ask_for_file file;
+      as_file file.file_file
+    end else
+    failwith "Download already started"
 
 let _ =
   network.op_network_search <- (fun q buf ->
+      
+(* Shouldn't we filter searches for other things than mp3s ? *)
+(* Shouldn't we also prevent searches if we don't have a correct IP
+  address, since the replies won't probably get in ? *)
+      
       next_token =:= !!next_token + 1;
       let query = q.search_query in
       let words = ref [] in
@@ -74,18 +83,18 @@ let _ =
       SlskGlobals.searches := (!!next_token, q) :: !SlskGlobals.searches;
       let words = String2.unsplit !words ' ' in
       lprintf "SEARCH COMPUTED"; lprint_newline ();
+      let msg = C2S.FileSearchReq {
+          C2S.FileSearch.words = words;
+          C2S.FileSearch.id = !!next_token;
+        } in
       List.iter (fun s ->
-          let msg = C2S.FileSearchReq {
-              C2S.FileSearch.words = words;
-              C2S.FileSearch.id = !!next_token;
-            } in
-          match s.server_sock with
-            None -> ()
-          | Some sock ->
+          do_if_connected s.server_sock (fun sock ->
               lprintf "SENDING SEARCH"; lprint_newline ();
               server_send sock msg; 
               s.server_search <- Some q; 
-              Printf.bprintf  buf "Sending search\n") !connected_servers
+              Printf.bprintf  buf "Sending search\n") 
+      )
+      !connected_servers
   )
   
     
@@ -172,10 +181,13 @@ let _ =
   let module P = GuiTypes in
   file_ops.op_file_info <- (fun file ->
       {
+        P.file_fields = P.Fields_file_info.all;
+
+        P.file_comment = file_comment (as_file file.file_file);
         P.file_name = file_best_name (as_file file.file_file);
         P.file_num = (file_num file);
         P.file_network = network.network_num;
-        P.file_names = [file_best_name (as_file file.file_file)];
+        P.file_names = [file_best_name (as_file file.file_file), P.noips()];
         P.file_md4 = file.file_id;
         P.file_size = file_size file;
         P.file_downloaded = file_downloaded file;
@@ -185,12 +197,13 @@ let _ =
         P.file_sources = None;
         P.file_download_rate = file_download_rate file.file_file;
         P.file_chunks = "0";
-        P.file_availability = "0";
+        P.file_availability = [network.network_num,"0"];
         P.file_format = FormatNotComputed 0;
         P.file_chunks_age = [|0|];
         P.file_age = file_age file;
         P.file_last_seen = BasicSocket.last_time ();
         P.file_priority = file_priority (as_file file.file_file);
+        P.file_uids = [];
       }    
   );
   (*
@@ -201,10 +214,11 @@ let _ =
       | _ -> ()
 );
   *)
-  file_ops.op_file_sources <- (fun file ->
+  file_ops.op_file_all_sources <- (fun file ->
       List2.tail_map (fun c -> as_client c.client_client)
       file.file_clients
-  )
+  );
+  file_ops.op_file_active_sources <- file_ops.op_file_all_sources
 
 let client_of_user user =
   let c = new_client user.user_nick in
@@ -237,10 +251,11 @@ let _ =
         P.client_chat_port = 0 ;
         P.client_connect_time = last_time ();
         P.client_software = "";
+        P.client_emulemod = "";
         P.client_downloaded = zero;
         P.client_uploaded = zero;
         P.client_upload = None;
-		P.client_sock_addr = "";
+(*    client_sock_addr = ""; *)
       }
   );
   client_ops.op_client_browse <- (fun c immediate ->
@@ -277,28 +292,22 @@ let _ =
         | _ -> assert false
       in
       List.iter (fun s ->
-          match s.server_sock with
-            None -> ()
-          | Some sock ->
-              server_send sock msg) !connected_servers  
+          do_if_connected s.server_sock (fun sock ->
+              server_send sock msg)) !connected_servers  
   );
   room_ops.op_room_name <- (fun s -> s.room_name);
   room_ops.op_room_resume <- (fun r ->
       set_room_state r RoomOpened;
       List.iter (fun s ->
-          match s.server_sock with
-            None -> ()
-          | Some sock ->
-              server_send sock (C2S.JoinRoomReq r.room_name)
+          do_if_connected s.server_sock (fun sock ->
+              server_send sock (C2S.JoinRoomReq r.room_name))
       ) !connected_servers
   );
   room_ops.op_room_close <- (fun r ->
       set_room_state r RoomPaused;
       List.iter (fun s ->
-          match s.server_sock with
-            None -> ()
-          | Some sock ->
-              server_send sock (C2S.LeaveRoomReq r.room_name)
+          do_if_connected s.server_sock (fun sock ->
+              server_send sock (C2S.LeaveRoomReq r.room_name))
       ) !connected_servers
       
   );
