@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+
 open AnyEndian
 
 open Printf2
@@ -84,7 +85,7 @@ let get_user_status s pos = (* 4 * 7 = 28 *)
   }, pos + 28
   
 let unknown opcode s =
-  lprintf "Unknown: opcode %d" opcode; lprint_newline ();
+  lprintf "Unknown: opcode %d\n" opcode; 
   dump s
   
 module C2S = struct
@@ -103,9 +104,9 @@ module C2S = struct
           { login = login; password = password; version = version; }
         
         let print t =
-          lprintf "LOGIN login:%s password:%s version:%d" 
-            t.login t.password t.version;
-          lprint_newline () 
+          lprintf "LOGIN login:%s password:%s version:%d\n" 
+            t.login t.password t.version
+          
         
         let write buf t =
           buf_string buf t.login;
@@ -120,9 +121,8 @@ module C2S = struct
         let parse s =  get_int s 0
         
         let print t =
-          lprintf "SETWAITPORT %d" t;
-          lprint_newline () 
-        
+          lprintf "SETWAITPORT %d\n" t
+          
         let write buf t =
           buf_int buf t
       
@@ -140,9 +140,8 @@ module C2S = struct
           { id = id; words = s }
         
         let print t =
-          lprintf "SEARCH %d FOR %s" t.id t.words;
-          lprint_newline () 
-        
+          lprintf "SEARCH %d FOR %s\n" t.id t.words
+          
         let write buf t =
           buf_int buf t.id;
           buf_string buf t.words
@@ -165,76 +164,72 @@ module C2S = struct
     let parse opcode s =
       try
         match opcode with
-        | 1 -> LoginReq (Login.parse s)
-        | 2 -> SetWaitPortReq (SetWaitPort.parse s)
-        | 3 -> 
+        | 1 (*SERVERCODE*) -> LoginReq (Login.parse s)
+        | 2 (*SERVERCODE*)  -> SetWaitPortReq (SetWaitPort.parse s)
+        | 3 (*SERVERCODE*)  -> 
             let user, _ = get_string s 0 in
             GetPeerAddressReq user
-        | 5 -> 
+        | 5 (*SERVERCODE*)  -> 
             let user, _ = get_string s 0 in
             AddUserReq user
-        | 13 ->
+        | 13 (*SERVERCODE*)  ->
             let room, pos = get_string s 0 in
             let msg, pos = get_string s pos in
             SayChatroomReq (room, msg)
-        | 14 -> 
+        | 14 (*SERVERCODE*)  -> 
             let room, _ = get_string s 0 in
             JoinRoomReq room
-        | 15 ->             
+        | 15 (*SERVERCODE*)  ->             
             let room, _ = get_string s 0 in
             LeaveRoomReq room
-(*        | 22 ->
+(*        | 22 (*SERVERCODE*)  ->
 (* MessageUserReq *)
             string : username
               string : message
 *)
             
-        | 26 -> FileSearchReq (FileSearch.parse s)
-(*
-        | 28 -> 
-(* SetStatus *)
-            int : new status (0=offline,1=away,2=online)
+        | 26 (*SERVERCODE*)  -> FileSearchReq (FileSearch.parse s)
+(*        | 28 (*SERVERCODE*)  ->  *)
+(* SetStatus 
+            int : new status (0=offline,1=away,2=online) *)
         
-        | 35 ->
+(*        | 35 (*SERVERCODE*)  ->
 (* SharedFoldersFiles *)
             int: folder count
               int: file count
 *)        
-        | 36 -> 
+        | 36 (*SERVERCODE*)  -> 
             let user, _ = get_string s 0 in
             GetUserStatsReq user
-(*      
-        | 60 ->
-(* PlaceInLineResponse *)
+(*       | 60 (*SERVERCODE*)  ->
+ PlaceInLineResponse 
             string: username
               int : token
               int : place
+*)
+            
+(*        | 67 (*SERVERCODE*)  -> GlobalUserList *)
         
-        | 67 ->
-(* GlobalUserList *)
-        
-        | 68 ->
-(* TunneledMessage *)
+(*        | 68 (*SERVERCODE*)  ->
+ TunneledMessage 
             string : username
               int : code
               int : token
               int : IP address
               int : port
               int : message
-            
-        | 69 ->
-(* PriviledgedUsers *)
-
-        | 92 ->
-(* CheckPrivileges *)
 *)
-        | 1001 -> 
+            
+(*        | 69 (*SERVERCODE*)  -> PriviledgedUsers *)
+
+(*        | 92 (*SERVERCODE*)  -> CheckPrivileges *)
+        | 1001 (*SERVERCODE*)  -> 
             let user, _ = get_string s 0 in
             CantConnectToPeerReq user
         | _ -> raise Not_found
       with
         e -> 
-          lprintf "From client"; lprint_newline ();
+          lprintf "From client\n"; 
           unknown opcode s;
           UnknownReq (opcode, s)
 
@@ -245,19 +240,19 @@ module C2S = struct
       | SetWaitPortReq t -> SetWaitPort.print t  
       | FileSearchReq t -> FileSearch.print t   
       | GetUserStatsReq t -> 
-          lprintf "GetUserStats %s" t; lprint_newline () 
+          lprintf "GetUserStats %s\n" t
       | JoinRoomReq t -> 
-          lprintf "JoinRoomReq %s" t; lprint_newline () 
+          lprintf "JoinRoomReq %s\n" t
       | LeaveRoomReq t -> 
-          lprintf "LeaveRoomReq %s" t; lprint_newline () 
+          lprintf "LeaveRoomReq %s\n" t
       | AddUserReq t -> 
-          lprintf "AddUserReq %s" t; lprint_newline () 
+          lprintf "AddUserReq %s\n" t;
       | GetPeerAddressReq t -> 
-          lprintf "GetPeerAddressReq %s" t; lprint_newline () 
+          lprintf "GetPeerAddressReq %s\n" t
       | CantConnectToPeerReq t -> 
-          lprintf "CantConnectToPeerReq %s" t; lprint_newline () 
+          lprintf "CantConnectToPeerReq %s\n" t
       | SayChatroomReq (room, msg) -> 
-          lprintf "SayChatroomReq %s: %s" room msg; lprint_newline () 
+          lprintf "SayChatroomReq %s: %s\n" room msg
       | UnknownReq (opcode, s) ->  unknown opcode s
           
     let write buf t =
@@ -299,13 +294,10 @@ module S2C = struct
         let print t =
           match t with
             Success (message, ip) ->
-              lprintf "LOGIN ACK: %s" message;
-              lprint_newline ();
-              lprintf "   IP: %s" (Ip.to_string ip);
-              lprint_newline ()
+              lprintf "LOGIN ACK: %s\n" message;
+              lprintf "   IP: %s\n" (Ip.to_string ip)
           | Failure reason ->
-              lprintf "LOGIN FAILURE %s" reason;
-              lprint_newline () 
+              lprintf "LOGIN FAILURE %s\n" reason
         
         let write buf t =
           match t with
@@ -354,11 +346,9 @@ module S2C = struct
 *)
         
         let print t =
-          lprintf "Room list: %d rooms" (List.length t);
-          lprint_newline ();
+          lprintf "Room list: %d rooms\n" (List.length t);
           List.iter (fun (name, nusers) ->
-              lprintf "    %50s  %-10d" name nusers;
-              lprint_newline () 
+              lprintf "    %50s  %-10d\n" name nusers;
           ) t
         
         let write buf t =
@@ -375,7 +365,7 @@ module S2C = struct
           users
         
         let print t =
-          lprintf "PRIVILEDGED USERS:"; lprint_newline ();
+          lprintf "PRIVILEDGED USERS\n:"; 
           List.iter (fun u -> lprintf "%s\n" u) t;
           lprint_newline ()
         
@@ -408,9 +398,8 @@ module S2C = struct
           }
         
         let print t =
-          lprintf "CONNECT TO PEER %s (%s:%d) token %d"
-            t.name (Ip.to_string t.ip) t.port t.token;
-          lprint_newline ()
+          lprintf "CONNECT TO PEER %s (%s:%d) token %d\n"
+            t.name (Ip.to_string t.ip) t.port t.token
         
         let write buf t =
           buf_string buf t.name;
@@ -448,26 +437,49 @@ module S2C = struct
             users stats; }
         
         let print t =
-          lprintf "JOIN ROOM %s:" t.room; lprint_newline ();
+          lprintf "JOIN ROOM %s\n:" t.room; 
           List.iter (fun u ->
-              lprintf "   %s" u.name; lprint_newline ()) t.users;
+              lprintf "   %s\n" u.name) t.users;
           lprint_newline ()
         
         let write buf t =
-          lprintf  "******* JoinRoomReply not implemented *****"; 
-          lprint_newline ();
+          lprintf  "******* JoinRoomReply not implemented *****\n"; 
           exit 1
       
       end
     
     
+    module FileSearch = struct
+        type t = {
+            id : int;
+            words : string;
+          }
+        
+        let parse s =  
+          let id = get_int s 0 in
+          let s, pos = get_string s 4 in
+          { id = id; words = s }
+        
+        let print t =
+          lprintf "SEARCH %d FOR %s\n" t.id t.words
+          
+        let write buf t =
+          buf_int buf t.id;
+          buf_string buf t.words
+      
+      end
+      
     
     type t = 
     | LoginAckReq of LoginAck.t
     | RoomListReq of RoomList.t
     | PriviledgedUsersReq of PriviledgedUsers.t
     | ConnectToPeerReq of ConnectToPeer.t
-    
+
+    | FileSearchReq of 
+(* user *)      string * 
+(* request id *)      int * 
+(* search term *)      string
     | GetPeerAddressReplyReq of 
 (* nick *) string * 
 (* ip *)   Ip.t * 
@@ -503,6 +515,15 @@ module S2C = struct
                    GlobalUserList:67,TunneledMessage:68,PrivilegedUsers:69,
 CantConnectToPeer:1001}
 
+                   Msg83:83,Msg84:84,Msg85:85,ParentInactivityTimeout:86,
+                   SearchInactivityTimeout:87,MinParentsInCache:88,
+                   Msg89:89,DistribAliveInterval:90,
+                   AddToPrivileged:91,CheckPrivileges:92,CantConnectToPeer:1001,
+                   HaveNoParent:71,SearchRequest:93,NetInfo:102,
+                   WishlistSearch:103,WishlistInterval:104,
+                   RoomTickerState:113,RoomTickerAdd:114,
+                   RoomTickerRemove:115,RoomTickerSet:116}
+
 
 Unknown: opcode 36
 ascii: [
@@ -519,110 +540,123 @@ ascii: [
     let parse opcode s =
       try
         match opcode with
-        | 1 -> LoginAckReq (LoginAck.parse s)
-        | 3 -> 
+        | 1 (*SERVERCODE*)  -> LoginAckReq (LoginAck.parse s)
+        | 3 (*SERVERCODE*)  -> 
             let name, pos = get_string s 0 in
             let ip = get_ip s pos in
             let port = get_int s (pos+4) in
             GetPeerAddressReplyReq (name, ip, port)
-        | 5 -> 
+        | 5 (*SERVERCODE*)  -> 
             let s, pos = get_string s 0 in
-            let present = get_int8 s pos in
+            let present = get_uint8 s pos in
             AddUserReplyReq (s, present = 1)
-        | 7 -> 
+        | 7 (*SERVERCODE*)  -> 
             let user, pos = get_string s 0 in
             let status = get_int s pos in
             UserStatusReq (user, status)
-        | 13 -> 
+        | 13 (*SERVERCODE*)  -> 
             let room_name, pos = get_string s 0 in
             let user_name, pos = get_string s pos in
             let message, pos = get_string s pos in
             SayChatroomReq (room_name, user_name, message)
         
-        | 14 -> JoinRoomReplyReq (JoinRoomReply.parse s)
-        | 16 ->
+        | 14 (*SERVERCODE*)  -> JoinRoomReplyReq (JoinRoomReply.parse s)
+        | 16 (*SERVERCODE*)  ->
             let room, pos = get_string s 0 in
             let user, pos = get_string s pos in
             let status, pos = get_user_status s pos in            
             UserJoinedRoomReq (room, user, status)
-        | 17 -> 
+        | 17 (*SERVERCODE*)  -> 
             let room, pos = get_string s 0 in
             let user, pos = get_string s pos in
             UserLeftRoomReq (room, user)
             
-        | 18 -> ConnectToPeerReq (ConnectToPeer.parse s)
-(*            
-        | 22 -> (* Message User *)
+        | 18 (*SERVERCODE*)  -> ConnectToPeerReq (ConnectToPeer.parse s)
+(*      | 22 (*SERVERCODE*)  -> (* Message User *)
             
             int: message ID
             int: timestamp
               string: username
             string: message
+*)
             
-        | 23 ->
+(*        | 23 (*SERVERCODE*)  ->
 (* MessageAcked : otherwise, the server keeps sending *)
             int : message ID
+*)
+
+        | 26 ->
+            let user, pos = get_string s 0 in
+            let search_id = get_int s pos in
+            let search_term, pos = get_string s (pos+4) in
+            FileSearchReq (user, search_id, search_term)
             
-        | 36 ->
-(* GetUserStats *)
+(*        | 36 (*SERVERCODE*) ->
+ GetUserStats 
             string: username
             int: avgspeed
               ...
+*)
             
-        | 40 ->
-(* Queued Downloads *)
+(*        | 40 (*SERVERCODE*) ->
+ Queued Downloads 
             string : username
               int : slotsfull
+*)
             
-        | 60 ->
-(* PlaceInLineResponse *)
+(*        | 60 (*SERVERCODE*) ->
+ PlaceInLineResponse 
             string: username
               int : token
             int : place
-
-        | 62 ->
-(* RoomAdded *)
-            string : room name
+*)
             
-        | 63 ->
-(* RoomRemoved *)
+(*        | 62 (*SERVERCODE*) ->
+ RoomAdded 
+string : room name
+  *)
+            
+(*        | 63 (*SERVERCODE*) ->
+ RoomRemoved 
             string : room name
 *)            
-        | 64 -> RoomListReq (RoomList.parse s)
-(*
-        | 65 ->
-(* ExactFileSearch *)
+
+        | 64 (*SERVERCODE*) -> RoomListReq (RoomList.parse s)
+(*        | 65 (*SERVERCODE*) ->
+ ExactFileSearch 
             int : token
               string : filename
               string : folder
               int : size
               int : checksum
             string : username
+*)
             
-        | 66 ->
-(* AdminMessage *)
+(*        | 66 (*SERVERCODE*) ->
+  AdminMessage 
             string : message
-            
-        | 67 ->
-(* GlobalUserList : comme JoinRoom *)
+*)
+  
+(*        | 67 (*SERVERCODE*) ->
+ GlobalUserList : comme JoinRoom 
 *)
             
             
-        | 69 -> PriviledgedUsersReq (PriviledgedUsers.parse s)
-(*            
-        | 92 ->
-(* CheckPrivileges *)
+        | 69 (*SERVERCODE*) -> PriviledgedUsersReq (PriviledgedUsers.parse s)
+            
+(*        | 92 (*SERVERCODE*) ->
+ CheckPrivileges 
             int : number of days left
-
-        | 1001 ->
-(* Cannot connect to peer *)
+*)
+  
+(*        | 1001 (*SERVERCODE*) ->
+ Cannot connect to peer 
             int : token
 *)            
         | _ -> raise Not_found
       with
         e -> 
-          lprintf "From server (exception %s):" (Printexc2.to_string e); 
-          lprint_newline ();
+          lprintf "From server (exception %s):\n" (Printexc2.to_string e); 
           unknown opcode s;
           UnknownReq (opcode, s)
     
@@ -651,6 +685,9 @@ ascii: [
           lprintf "SAID ON %s BY %s: %s" room user message;
           lprint_newline ();
       | UnknownReq (opcode, s) ->  unknown opcode s
+      | FileSearchReq (user, search_id, search_term) ->
+          lprintf "FileSearchReq (%s,%d,%s)\n"
+            user search_id search_term
     
     let write buf t =
       match t with
@@ -683,7 +720,7 @@ ascii: [
           buf_int buf 7;
           buf_string buf user;
           buf_int buf status
-
+          
       | SayChatroomReq (room, user, message) ->
           buf_int buf 13;
           buf_string buf room;
@@ -693,7 +730,13 @@ ascii: [
       | UnknownReq (opcode, s) -> 
           buf_int buf opcode;
           Buffer.add_string buf s
-  
+
+      | FileSearchReq (user, search_id, search_term) ->
+          buf_int buf 26;
+          buf_string buf user;
+          buf_int buf search_id;
+          buf_string buf search_term
+          
   end
   
   
@@ -708,10 +751,10 @@ module C2C = struct
       }
     
     let get_file s pos =
-      let code = get_int8 s pos in
+      let code = get_uint8 s pos in
       let name, pos = get_string s (pos+1) in
-      let size = get_int64_32 s pos in
-      let size2 = get_int64_32 s (pos+4) in
+      let size = get_uint64_32 s pos in
+      let size2 = get_uint64_32 s (pos+4) in
       let format, pos = get_string s (pos+8) in
       let tags, pos = get_list (fun s pos ->
             (get_int s pos, get_int s (pos+4)), pos+8) s pos
@@ -749,6 +792,7 @@ module C2C = struct
             files : file list;
             freeulslots : int;
             ulspeed : int;
+            inqueue : int;
           }
         
         let parse s = 
@@ -756,16 +800,20 @@ module C2C = struct
           let user, pos = get_string s 0 in
           let id = get_int s pos in
           let files, pos = get_list get_file s (pos+4) in
-          let freeulslots = get_int8 s pos in
+          let freeulslots = get_uint8 s pos in
           let ulspeed = get_int s (pos+1) in
-(*        let ???? = get_int s (pos+5) in *)
+          let inqueue = get_int s (pos+5) in 
           {
             user = user;
             id = id;
             files = files;
             freeulslots = freeulslots;
             ulspeed = ulspeed;
+            inqueue = inqueue;
           }
+
+          (* TODO: Add those results to the list if id = id of the FileSearch we sent --zcat *)
+
       end
     
     module FolderContentsReply = struct
@@ -801,20 +849,33 @@ module C2C = struct
 (* request id *) int *
 (* reason *)     string
     | FolderContentsReq of string
+      
+    | FileSearchRequestReq of 
+(* request id *) int *
+(* search term *) string
+      
     | UnknownReq of int * string
-    
+      (*
+    peercodes = {GetSharedFileList:4, SharedFileList:5, FileSearchRequest:8,
+                FileSearchResult:9,
+                UserInfoRequest:15,UserInfoReply:16, FolderContentsRequest:36,
+                FolderContentsResponse:37, TransferRequest:40,
+                TransferResponse:41,PlaceholdUpload:42,QueueUpload:43,
+                PlaceInQueue:44,UploadFailed:46,QueueFailed:50,PlaceInQueueRequest:51}
+*)
+      
     let parse opcode s =
       try
         match opcode with          
         | 4 -> GetSharedFileListReq
         | 5 -> SharedFileListReq (SharedFileList.parse s)
+        | 8 -> 
+            let search_id = get_int s 0 in
+            let search_term, pos = get_string s 4 in
+            FileSearchRequestReq (search_id, search_term)
         | 9 -> FileSearchResultReq (FileSearchResult.parse s)
-(*        
-        | 15 -> 
-(* UserInfoRequest *)
-        
-        | 16 -> 
-(* UserInfoReply *)
+(*      | 15 ->  UserInfoRequest *)
+(*      | 16 ->  UserInfoReply 
             string : user description
               byte: has a picture ?
               string : present if pic = 1
@@ -832,44 +893,39 @@ module C2C = struct
             let download = get_int s 0 = 0 in
             let req = get_int s 4 in
             let file, pos = get_string s 8 in
-            let size = get_int64_32 s pos in
+            let size = get_uint64_32 s pos in
             TransferRequestReq (download, req, file, size)
         
         | 41 -> 
             let req = get_int s 0 in
-            let allowed = get_int8 s 4 = 1 in
+            let allowed = get_uint8 s 4 = 1 in
             if allowed then
-              let filesize = get_int64_32 s 5 in
+              let filesize = get_uint64_32 s 5 in
               TransferOKReplyReq (req, filesize)
             else
             let reason, pos = get_string s 5 in
             TransferFailedReplyReq (req, reason)
-(*
-        | 42 ->
-(* PlaceholdUpload *)
+(*        | 42 -> PlaceholdUpload 
             string : filename 
+*)
             
-        | 43 -> 
-(* QueueUpload *)
+(*        | 43 -> QueueUpload 
             string : filename
-            
-        | 44 -> 
-(* PlaceInQueue *)
+*)            
+(*        | 44 ->  PlaceInQueue 
             string : filename
               int : place
-            
-        | 46 ->
-(* UploalFailed *)
+*)
+
+(*         | 46 -> UploalFailed 
             string : filename
-            
-        | 50 ->
-(* QueueFailed *)
+*)
+
+(*        | 50 -> QueueFailed 
             string : filename
               string : reason
-            
-        | 51 ->
-(* PlaceInQueueRequest *)
-*)            
+*)
+(*        | 51 -> PlaceInQueueRequest *)            
             
         | _ -> raise Not_found
       with
@@ -885,6 +941,9 @@ module C2C = struct
           lprintf "GetSharedFileListReq"; lprint_newline () 
       | FolderContentsReq folder ->
           lprintf "FolderContentsReq"; lprint_newline ();
+      | FileSearchRequestReq (search_id, search_term) ->
+          lprintf "FileSearchRequestReq (%d, %s)"
+            search_id search_term
       | FolderContentsReplyReq folders ->
           lprintf "FolderContentsReplyReq"; lprint_newline ();
           List.iter (fun (s, dirs) ->
@@ -945,7 +1004,12 @@ module C2C = struct
           buf_int buf req;
           buf_string buf file;
           buf_int64_32 buf size
-
+          
+      | FileSearchRequestReq (search_id, search_term) ->
+          buf_int buf 8;
+          buf_int buf search_id;
+          buf_string buf search_term
+          
       | TransferOKReplyReq (req, filesize) ->
           buf_int buf 41;
           buf_int buf req;
@@ -981,7 +1045,7 @@ let soulseek_handler parse f sock nread =
           let opcode = get_int b.buf (b.pos+4) in
           let data = String.sub b.buf (b.pos+8) (msg_len-4) in
 (*          LittleEndian.dump (String.sub b.buf b.pos (msg_len+4)); *)
-          TcpBufferedSocket.buf_used sock (msg_len + 4);
+          buf_used b (msg_len + 4);
           let t = parse opcode data in
 (*          print t; *)
           f t sock
@@ -1017,7 +1081,7 @@ let server_send sock t =
   let s = server_msg_to_string t in
 
   if !verbose_msg_servers then begin
-      lprintf "SENDING TO SERVER:"; lprint_newline ();
+      lprintf "SENDING TO SERVER:\n"; 
       C2S.print t;
       AnyEndian.dump s;
     end;
